/********************************************************************
 * Copyright  2004 VHA. All rights reserved
 ********************************************************************/
package gov.va.med.esr.service.impl;

// Java Classes
import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.text.SimpleDateFormat;

import org.apache.commons.lang.Validate;
import org.apache.commons.lang.exception.ExceptionUtils;

import com.thoughtworks.xstream.XStream;

import gov.va.med.esr.common.infra.ImpreciseDate;
import gov.va.med.esr.common.infra.ImpreciseDateUtils;
import gov.va.med.esr.common.model.ee.AgentOrangeExposure;
import gov.va.med.esr.common.model.ee.CampLejeuneVerification;
import gov.va.med.esr.common.model.ee.RadiationExposure;
import gov.va.med.esr.common.model.ee.VerificationInfo;
import gov.va.med.esr.common.model.ee.VoaFileAttachment;
import gov.va.med.esr.common.model.financials.FinancialStatement;
import gov.va.med.esr.common.model.financials.IncomeTest;
import gov.va.med.esr.common.model.financials.SpouseFinancials;
import gov.va.med.esr.common.model.insurance.InsuranceSubscriber;
import gov.va.med.esr.common.model.insurance.Medicare;
import gov.va.med.esr.common.model.insurance.PrivateInsurance;
import gov.va.med.esr.common.model.person.Association;
import gov.va.med.esr.common.model.lookup.AddressChangeSource;
import gov.va.med.esr.common.model.lookup.AgentOrangeExposureLocation;
import gov.va.med.esr.common.model.lookup.AuthenticationLevel;
import gov.va.med.esr.common.model.lookup.EmailSourceOfChange;
import gov.va.med.esr.common.model.lookup.FunctionalGroup;
import gov.va.med.esr.common.model.lookup.Indicator;
import gov.va.med.esr.common.model.lookup.InsuredRelationship;
import gov.va.med.esr.common.model.lookup.MaritalStatus;
import gov.va.med.esr.common.model.lookup.NameType;
import gov.va.med.esr.common.model.lookup.PhoneSourceOfChange;
import gov.va.med.esr.common.model.lookup.VAFacility;
import gov.va.med.esr.common.model.lookup.VOAApplicationStatus;
import gov.va.med.esr.common.model.lookup.VOAFormType;
import gov.va.med.esr.common.model.lookup.WkfCaseType;
import gov.va.med.esr.common.model.party.Address;
import gov.va.med.esr.common.model.party.Email;
import gov.va.med.esr.common.model.party.Phone;
import gov.va.med.esr.common.model.person.FullyQualifiedIdentity;
import gov.va.med.esr.common.model.person.Name;
import gov.va.med.esr.common.model.person.Person;
import gov.va.med.esr.common.model.person.PreferredFacility;
import gov.va.med.esr.common.model.person.VOAApplication;
import gov.va.med.esr.common.model.person.DeathRecord;
import gov.va.med.esr.service.VOASearchCriteria;
import gov.va.med.esr.service.VOASearchResultBean;
import gov.va.med.esr.common.model.person.VOAResubmissionData;
import gov.va.med.esr.common.model.person.id.PersonEntityKey;
import gov.va.med.esr.common.model.person.id.VPIDEntityKeyImpl;
import gov.va.med.esr.common.persistent.ee.VoaFileAttachmentDAO;
import gov.va.med.esr.common.persistent.person.VOAApplicationDAO;
import gov.va.med.esr.service.CommsEmailBulletinService;
import gov.va.med.esr.service.EnvironmentParamService;
import gov.va.med.esr.service.MessagingService;
import gov.va.med.esr.service.PersonIdentityTraits;
import gov.va.med.esr.service.PersonService;
import gov.va.med.esr.service.SystemParameterService;
import gov.va.med.esr.service.VOAApplicationService;
import gov.va.med.esr.service.WorkflowService;
import gov.va.med.esr.service.trigger.BulletinTrigger;
import gov.va.med.esr.service.trigger.BulletinTriggerEvent;
import gov.va.med.esr.service.trigger.VOATriggerEvent;
import gov.va.med.fw.persistent.DAOException;
import gov.va.med.fw.service.ServiceException;
import gov.va.med.fw.service.trigger.TriggerRouter;
import gov.va.med.fw.util.StringUtils;
import gov.va.med.ps.model.PatientIdentifier;
import gov.va.med.fw.service.pagination.SearchQueryInfo;
import gov.va.med.esr.common.model.lookup.DeathDataSource;

/**
 * The VOA Application Service implementation.
 * <p/>
 * Project: Common
 *
 * @author DNS   TSAIG
 * @version 3.11
 */
public class VOAApplicationServiceImpl extends AbstractHistoricalInfoServiceImpl implements VOAApplicationService
{

	private static final long serialVersionUID = 8936807483114587339L;

	private static final Object PENDING_CAT_EDIT = "PCE";

	private static VAFacility VOA_VA_FACILITY = null;

	private VOAApplicationDAO voaApplicationDAO = null;

    private VoaFileAttachmentDAO voaFileAttachmentDAO = null;

    private TriggerRouter triggerRouter = null;

    private IdmWebServiceDelegate idmServiceDelegate = null;

    private PersonService personService = null;

    private SystemParameterService systemParameterService = null;

    private MessagingService messagingService = null;

    private WorkflowService workflowService = null;

    private CommsEmailBulletinService bulletinService;

    private InsuredRelationship INSURED_RELATIONSHIP_VETERAN = null;

	public static final String TXT_1010EZ = ("1010EZ");
	public static final String TXT_1010EZR = ("1010EZR");
	public static final String TXT_1010HS = ("1010HS");
	public static final String TXT_1010SH = ("1010SH");
	public static final String TXT_1010CG = ("1010CG");
	public static final String TXT_21526EZ = ("21526EZ");  //CCR13857 new form type

	//VOA_IND =1 for Anonymous/LOA1 only, means person found, Proxy PF added successfully
    //VOA_IND = 2 for Anonymous/LOA1 only. It means MVI extended search not found, person was added explicitly
	public static final Character VOA_IND_1 = '1';
	public static final Character VOA_IND_2 = '2';



    /**
     * Default Constructor.
     */
    public VOAApplicationServiceImpl()
    {
        super();
    }

    /******** Begin of property getters/setters *****/
	public VOAApplicationDAO getVoaApplicationDAO() {
		return voaApplicationDAO;
	}

	public void setVoaApplicationDAO(VOAApplicationDAO voaApplicationDAO) {
		this.voaApplicationDAO = voaApplicationDAO;
	}

	public VoaFileAttachmentDAO getVoaFileAttachmentDAO() {
		return voaFileAttachmentDAO;
	}

	public void setVoaFileAttachmentDAO(VoaFileAttachmentDAO voaFileAttachmentDAO) {
		this.voaFileAttachmentDAO = voaFileAttachmentDAO;
	}

	public TriggerRouter getTriggerRouter() {
		return triggerRouter;
	}

	public void setTriggerRouter(TriggerRouter triggerRouter) {
		this.triggerRouter = triggerRouter;
	}



	public IdmWebServiceDelegate getIdmServiceDelegate() {
		return idmServiceDelegate;
	}

	public void setIdmServiceDelegate(IdmWebServiceDelegate idmServiceDelegate) {
		this.idmServiceDelegate = idmServiceDelegate;
	}

	public PersonService getPersonService() {
		return personService;
	}

	public void setPersonService(PersonService personService) {
		this.personService = personService;
	}

	public SystemParameterService getSystemParameterService() {
		return systemParameterService;
	}

	public void setSystemParameterService(
			SystemParameterService systemParameterService) {
		this.systemParameterService = systemParameterService;
	}

	public WorkflowService getWorkflowService() {
		return workflowService;
	}

	public void setWorkflowService(WorkflowService workflowService) {
		this.workflowService = workflowService;
	}


public CommsEmailBulletinService getBulletinService() {
		return bulletinService;
	}

	public void setBulletinService(CommsEmailBulletinService bulletinService) {
		this.bulletinService = bulletinService;
	}

/******** End of property getters/setters *****/



	public MessagingService getMessagingService() {
		return messagingService;
	}

	public void setMessagingService(MessagingService messagingService) {
		this.messagingService = messagingService;
	}

	/**
	 * Save a VOAApplication
	 * @param app
	 * @return VOAApplication saved
	 * @throws ServiceException
	 */
	public VOAApplication save(VOAApplication app) throws ServiceException
    {
        try
        {
			Validate.notNull(app, "A VOA Application input must not be null.");
            // Save the person
			voaApplicationDAO.saveObject(app);

			// Notice we do not clone this return object that is attached to Hibernate session
			return app;
		}
        catch (DAOException e)
        {
			throw new ServiceException("Failed to persist a VOAApplication", e);
		}
	}

	/**
     * Gets the VOA attachment by person ID
     *
     * @param personId
     * @return The list of VOA attachments, null when empty
     * @throws DAOException
     *         Thrown when a persistence exception occurs.
     */
    @SuppressWarnings("rawtypes")
	public List findVoaAttachmentByPersonId(BigDecimal personId) throws ServiceException {
        try
        {
            return voaFileAttachmentDAO.findVoaAttachmentByPersonId(personId);
        }
        catch (DAOException e)
        {
            throw new ServiceException(e.getMessage(), e);
        }
    }

	/**
     * Gets the VOA attachment by VOA application ID
     *
     * @param applicationId
     * @return The list of VOA attachments, null when empty
     * @throws DAOException
     *         Thrown when a persistence exception occurs.
     */
    @SuppressWarnings("rawtypes")
	public List findVoaAttachmentByApplicationId(BigDecimal applicationId) throws ServiceException {
        try
        {
            return voaFileAttachmentDAO.findVoaAttachmentByApplicationId(applicationId);
        }
        catch (DAOException e)
        {
            throw new ServiceException(e.getMessage(), e);
        }
    }



	/**
     * Update the VOA attachment attributes
     *
     * @param attachment
     * @throws DAOException
     *         Thrown when a persistence exception occurs.
     */
    public void updateAttachmentAttributes(VoaFileAttachment attachment)  throws ServiceException {
        try
        {
            voaFileAttachmentDAO.updateAttachmentAttributes(attachment);
        }
        catch (DAOException e)
        {
            throw new ServiceException(e.getMessage(), e);
        }
    }
    /**
     * Update the Person Id of the VOA attachment with the specified message control id
     * @param msgControlId
     * @param personId
     * @throws ServiceException
     */
    public void updateVoaAttachmentWithPersonId(String msgControlId, BigDecimal personId)  throws ServiceException
    {
        try
        {
            voaFileAttachmentDAO.updateVoaAttachmentWithPersonId(msgControlId, personId);
        }
        catch (DAOException e)
        {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * forward VOA application to common JMS queue
     * @param applicationId VOA application id
     * @param person person object
     * @throws ServiceException
     * NOTE: applicationId and person must not be null
     */

    public void forwardVOAApplication(BigDecimal applicationId, Person person) throws ServiceException
    {
        Validate.notNull(applicationId, "VOA application id must not be null");
        Validate.notNull(person, "Incoming Person must not be null");

    	this.getTriggerRouter().processTriggerEvent(new VOATriggerEvent(applicationId, person));
    }

	/**
     * Gets the VOA Application by VOA application ID
     *
     * @param applicationId
     * @return VOAApplication
     * @throws ServiceException
     *         Thrown when a persistence exception occurs.
     */
    public VOAApplication findVoaApplicationByApplicationId(BigDecimal applicationId) throws ServiceException {

    	Validate.notNull(applicationId, "VOA application id must not be null");

    	try
        {
            return voaApplicationDAO.getByApplicationId(applicationId);
        }
        catch (DAOException e)
        {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List findByFQID(String fqid) throws ServiceException {
   	 try
        {
            return voaApplicationDAO.getByFQID(fqid);
        }
        catch (DAOException e)
        {
            throw new ServiceException(e.getMessage(), e);
        }
   }

    /**
     * Update VOAApplication
     * @param app
     * @return updated VOAApplication
     * @throws ServiceException
     */
    public VOAApplication updateVOAApplication(VOAApplication app) throws ServiceException
    {
        Validate.notNull(app, "VOA application must not be null");
		try {
			this.getVoaApplicationDAO().update(app);
		} catch (DAOException de)
		{
			throw new ServiceException(de);
		}

		return app;
    }

    /**
     * ES 4.0.3_CodeCR-13999
     * removeOrphanedVOAApplication
     * @param app
     * @throws ServiceException
     */

    @SuppressWarnings("rawtypes")
	private void removeOrphanedVOAApplication(VOAApplication app) throws ServiceException
    {
        Validate.notNull(app, "VOA application must not be null");
		try {
//            VOASearchCriteria vsc = new VOASearchCriteria();
//            vsc.setFullQualifiedId(app.getFullQualifiedId());
//            List pendingSubmissions = getVoaApplicationDAO().find(vsc);

            List pendingSubmissions = getVoaApplicationDAO().findByNamedQuery("VOAApplication_GetOrphanedApps", app.getFullQualifiedId());
            for (int i = 0; i < pendingSubmissions.size(); i++) {
            	VOAApplication orphan = (VOAApplication) pendingSubmissions.get(i);
                   orphan.setApplicationStatus((VOAApplicationStatus) this.getLookupService().getByCode(VOAApplicationStatus.class,
                                 VOAApplicationStatus.FORM_PROCESSED_SUCCESSFUL));
                   this.getVoaApplicationDAO().saveObject(orphan);
            }
		} catch (DAOException de) {
			throw new ServiceException(de);
		}
    }


    /**
     * Process VOA Application with application id and person object
     * @param applicationId the application id from VOA Application
     * @param incoming the person comes from the VOA web service
     * @throws ServiceException
     */

    /**
	 * RTC Task 236652 -Fix for When a veteran answers no to the disclose financial details option the VOA form.
	 * This method was changed to public to facilitate the implementation for the task item.
	 */
    public boolean isDSLogonLevel2(VOAApplication app) {
    	if (app.getAuthenticationLevel() == null ||
    			AuthenticationLevel.ANONYMOUS.getCode().equals(app.getAuthenticationLevel().getCode()) ||
    			AuthenticationLevel.DS_LOGON_LEVEL_1.getCode().equals(app.getAuthenticationLevel().getCode()))
    		return false;

    	return true;
    }


    @SuppressWarnings({ "null", "static-access" })
	public Person processVoaApplication(BigDecimal applicationId, Person incoming)
			throws ServiceException {
		Validate.notNull(applicationId, "VOA application id must not be null");
		Validate.notNull(incoming, "Incoming Person must not be null");

		/**if (logger.isDebugEnabled()) {
			logger.debug("Populated Person after procesVOASvc in in processapplication(): " + incoming.toString());

			if (incoming != null){
				logger.debug("income test after coming to voaSvc in processapplication(): " + incoming.getIncomeTest(Calendar.getInstance().get(Calendar.YEAR)-1));

			}
		}*/

		//retrieve VAOApplication
		VOAApplication app = null;
		try {
			app = this.getVoaApplicationDAO().getByApplicationId(applicationId);
		} catch (DAOException de) {

			//log error with submission id for better resolution
			logger.error("VOA Process failed due to exception: (submission id=" +((app != null) ? app.getEntityKey().getKeyValueAsString() : null) +"): " + ExceptionUtils.getFullStackTrace(de));
			if (app != null) {
				//if failed, put failed status and error text
				app.setApplicationStatus((VOAApplicationStatus) this
						.getLookupService().getByCode(VOAApplicationStatus.class,
								VOAApplicationStatus.FORM_SUBMISSION_FAILED));
				//CCR13417: get the stacktrace to get more detail
				app.setErrorText("VOA Process failed due to exception: " + ExceptionUtils.getRootCause(de));

				this.updateVOAApplication(app);
			}
			throw new ServiceException(de);
		}

		if (app == null)
		{
			//throw new ServiceException("Get null VOAApplication with applicationId= "+ applicationId);
			//don't throw exception, nothing we can do
			logger.error("Get null VOAApplication with applicationId= "+ applicationId);
			return null;
		}


		//4.9.	Set VOA Message Status of Accepted Message
		//Upon accepting the VOA message for processing, the system sets the VOA processing status to
		//"Form Processing in Progress".
		app.setApplicationStatus((VOAApplicationStatus) this.getLookupService().getByCode(VOAApplicationStatus.class, VOAApplicationStatus.FORM_PROCESSING_IN_PROGRESS));
		this.updateVOAApplication(app);

		if (this.VOA_VA_FACILITY == null) //set the VOA facility for the first time
			VOA_VA_FACILITY = this.getLookupService().getVaFacilityByStationNumber(VAFacility.CODE_MHV.getCode());

		//CCR12789 - split the code branch for ds login level 2 and anonymous & level 1
		if (this.isDSLogonLevel2(app))
		{
			return processDSLogonLevel2(app, incoming);
		} else {
			return processAnonymousLevel1(app, incoming);
		}
    }

    private void setVeteranIndicator(VOAApplication app, Person incoming) {
    	//TODO: waiting for MVI to implement the veteran indicator, expected date March 2015
    	//For now, Brad directs us to set the default to be true for anonymous and LOA 1
    	incoming.setVeteran(Boolean.TRUE);

    	//if this is not resubmit
    	//then
    	//	queryMSDS
    	//	serialize person object and save the application
    	//
    	//   public void queryMSDSforMSdata() {
//	   logger.info("Adding the msds trigger event to the queue");
//	   addTriggerEvent(new PersonTriggerEvent(PersonTrigger.DestinationType.MESSAGING,
//			   PersonTrigger.TargetType.VISTA, PersonTrigger.DispatchType.QUERY,
//			   PersonTrigger.DataType.ELIGIBILITY));
//	   logger.info("Added the msds trigger event to the queue");
//
//   }
    }

    public void processEnrollment(Boolean vetIndicator) {

    }

    @SuppressWarnings("unchecked")
	private List<PatientIdentifier> getCorrelationsFromCompositeCall(PersonIdentityTraits traits, IdMSearchInfo searchInfo) throws ServiceException {
		//extended search composite call with person traits instead. Since 1309 does not take person traits, use composite call.
		//composite search is always unattended
		Set<PersonIdentityTraits> traitsSet = idmServiceDelegate.searchWithCompositeCall(traits, searchInfo);
		if (traitsSet == null || traitsSet.size() ==0) {
			//ESR 4.0_CodeCR12845_GT_VOA - Null Pointer Exception when submitting Anonymous user Not Known to MVI - Flag ON
			return null;
		}

		//get the first one.
		//TODO: what if there are multiple???
		return traitsSet.iterator().next().getAllCorrelations();

    }


	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void sendPersonNotFoundEmail(VOAApplication app, Person person) {

        BulletinTriggerEvent bulletinTriggerEvent =
            new BulletinTriggerEvent(BulletinTrigger.DataType.Person_Not_Found_for_Veteran_submitted_in_VOA);
		if (!StringUtils.isEmpty(app.getFullQualifiedId()))
			bulletinTriggerEvent.addField("FULLY_QUALIFIED_ID", app.getFullQualifiedId());
		if (app.getIdentityType() != null)
			bulletinTriggerEvent.addField("Name", app.getIdentityType().getName());
		bulletinTriggerEvent.addField("VOA_ENROLLMENT_APP_ID", app.getEntityKey().getKeyValueAsString());
        bulletinTriggerEvent.setPersonId(person.getPersonEntityKey());


			if (bulletinService != null) {
				Map bulletinData = new HashMap();
				bulletinData.put("FULLY_QUALIFIED_ID", app.getFullQualifiedId());
				bulletinData.put("Name", app.getIdentityType().getName());
				bulletinData.put("VOA_ENROLLMENT_APP_ID", app.getEntityKey().getKeyValueAsString());
				String formType = convertFormType(app);


		// Process the event
		try{
			this.getTriggerRouter().processTriggerEvent(bulletinTriggerEvent);
			bulletinService.sendVOAWorkflowEmailBulletin(
					BulletinTrigger.DataType.Person_Not_Found_for_Veteran_submitted_in_VOA_WORKFLOW,
					bulletinData, null, formType);
		} catch (ServiceException ex) {
			logger.error("VOA processAnonymousLevel1 Person not found from MVI - cannot send email notification for appId= "
					+ app.getEntityKey().getKeyValueAsString() + "\nError Message: " + ex.getMessage());
			}
		}
	}

	public String convertFormType(VOAApplication app){
		String formType=null;
		if (app.getFormType().getCode().equals(VOAFormType.VOA_FORM_1010EZ.getCode()))
		{
						formType = TXT_1010EZ;
		}
		else if (app.getFormType().getCode().equals(VOAFormType.VOA_FORM_1010EZR.getCode()))
		{
						formType = TXT_1010EZR;
		}
		else if (app.getFormType().getCode().equals(VOAFormType.VOA_FORM_1010HS.getCode()))
		{
						formType = TXT_1010HS;
		}
		else if (app.getFormType().getCode().equals(VOAFormType.VOA_FORM_1010SH.getCode()))
		{
						formType = TXT_1010SH;
		}
		else if (app.getFormType().getCode().equals(VOAFormType.VOA_FORM_1010CG.getCode()))
		{
						formType = TXT_1010CG;
		}
		else if (app.getFormType().getCode().equals(VOAFormType.VOA_FORM_21526EZ.getCode()))
		{
						formType = TXT_21526EZ;
		}
		else formType = " - Unknown Form";
		return formType;
		}

    //CCR12789 split the branch out to process anonymous and level 1
	private Person processAnonymousLevel1(VOAApplication app, Person incoming) throws ServiceException {
		Validate.notNull(app, "VOA application must not be null");
		Validate.notNull(incoming, "Incoming Person must not be null");
		Person incomingCopy = incoming;

		//determine and set the veteran indicator
		setVeteranIndicator(app, incoming);


		if (logger.isDebugEnabled()) {
			logger.debug("Populated Person after procesVOASvc: " + incoming.toString());

			if (incoming != null){
				logger.debug("income test after coming to voaSvc: " + incoming.getIncomeTest(Calendar.getInstance().get(Calendar.YEAR)-1));

			}
		}

		//get the correlations by fully qualified id or person traits
		PersonIdentityTraits traits = null;
		List<PatientIdentifier> correlations = null;

		IdMSearchInfo searchInfo = new IdMSearchInfo();
		searchInfo.setExtendedSearch(true);
		searchInfo.setDataEntererExtension("SSN= " + app.getSsn());
		searchInfo.setDataEntererFamilyName(app.getLastName());
		searchInfo.setDataEntererGivenName(app.getFirstName());

		if (!StringUtils.isEmpty(app.getFullQualifiedId()) && app.getFullQualifiedId().split("\\^").length == 4){
			String[] ids = app.getFullQualifiedId().split("\\^");
			//retrieve by fully qualified id (1305 with composite & extended search)
			traits =idmServiceDelegate.searchWithCompositeCall(
					new FullyQualifiedIdentity(ids[0], ids[1], ids[2], ids[3]), false, searchInfo);

			//if not found, search by person traits instead (1305 composite call with extended search)
			if (traits == null) {
				//search by person traits (1305 composite call with extended search)
				correlations = getCorrelationsFromCompositeCall(incoming.getIdentityTraits(), searchInfo);
			} else {
				correlations = traits.getAllCorrelations();
			}
		} else {
			//search by person traits (1305 composite call with extended search)
			correlations = getCorrelationsFromCompositeCall(incoming.getIdentityTraits(), searchInfo);
		}
		//set Voa default values, some values can be used for MVI add/update
		setVoaDefaults(incoming, app);

		BigDecimal personId = null;
		if (correlations == null || correlations.size() == 0) {

			//explicit add person, must have the search token from the search above, otherwise throws EXCPTION
			if (StringUtils.isEmpty(searchInfo.getSearchToken()))
				throw new ServiceException("Procoess Anonymous & Level 1 VOA application: Add Person Explicitly with empty search token");

			//set the explict add flag
			searchInfo.setAddPersonExplicit(true);

			//set person VOA_IND = 2
			incoming.setVOAIndicator(VOA_IND_2);

			//explicit add a person to MVI and add person stub
			this.getPersonService().addPersonExplicit(incoming, searchInfo);

			//CCR 13437 set the justAddedFlag
			incoming.setJustAdded(true);

			personId = (BigDecimal) incoming.getEntityKey().getKeyValue();


			/*if (logger.isDebugEnabled()) {
				logger.debug("Populated Person when corre = null or corr.size=0: " + incoming.toString());

				if (incoming != null){
					logger.debug("income test after coming to voaSvc: " + incoming.getIncomeTest(Calendar.getInstance().get(Calendar.YEAR)-1));

				}
			}*/



		} else {
			//correlations found

			//ESR internally needs long vpid because person table supports only long vpid
			String vpid = VPIDEntityKeyImpl.getLongVPID(getICN(correlations));
			incoming.setVPIDValue(vpid);

			//When ES requests a 200ESR correlation, ES will send all the trait information from the form
			//because the form may include additional attributes which were entered by the user
			//(for e.g.. address, phone, maiden name etc) after the ICN was initially created.
			//
			//add 200ESR Correlation if does not exist, also add person stub
			PatientIdentifier esrCorr = get200ESRCorrelation(correlations);

			Person adrPerson = null;
			if (esrCorr == null)
			{
//				5.3.	Person Not Known to ES
//				In the Basic Flow step Search for Person, if the person indicated on the VOA message is found but is not yet known to ES, a person record is added to ES.
				//When ES requests MVI to create a 200ESR correlation,  MVI will create a stub record in person table
				//as it happens today with all other 200ESR correlations.
				this.getPersonService().addESRCorrelation(incoming);

				//the email from Cory Chin on Friday, January 25, 2013 confirmed that:
//				It is our understanding that PSIM currently creates a stub record in person table under the following scenarios.
	//
//				1)	When a new user is added in VistA and when VistA sends a request to MVI to create the national ICN
//				2)	When a new user is added through ESR UI  Add A person functionality
	//
//				A person stub record is not created when a  Add 200 ESR correlation is  made.

				//Thus ES has to insert person stub by inserting a new entry to PS_PERSON_VPID and PERSON table

				PersonEntityKey personKey = null;
				try {
					//MAKE SURE TO COMMENT OUT THE NEXT TWO LINES - only needed for EDEV
					//since EDEV is not fully integrated with MVI: insert into PS_PERSON_VPID manually
//					PersonVPID pv = new PersonVPID(incoming.getVPIDValue());
//					this.getVoaApplicationDAO().insertObject(pv);

					//CCR 12099 - check person stub exists or not before add it.
					//NOTE: a migration data can have person stub without 200ESR Correlation!
					personKey = getPersonService().getPersonIdByVPID(incoming.getVPIDEntityKey());
					if (personKey == null)
					{
						// add to ADR PERSON table (person stub)
						personKey = getPersonService().addPersonStub(incoming.getVPIDEntityKey(), incoming.getVeteran(), incoming.getVOAIndicator()); //insert into PERSON

						//must flush hibernate so the person id will be available right away to avoid null pointer exception
						this.getVoaApplicationDAO().flush();
					}

				} catch (DAOException ex){
					throw new ServiceException ("insert into PS_PERSON_VPID failed: ",ex);
				}
				personId = (BigDecimal)personKey.getKeyValue();

				//get the ADR person with vpid, without person traits
				adrPerson = this.getPersonService().getPersonWithoutIdentityTraits(incoming.getVPIDEntityKey());

				if (adrPerson != null)
					logger.info("adr person is :" + adrPerson.toString());

		        if (adrPerson == null) {
		        	//throw new ServiceException("Cannot process person. onFile person not found for vpid= " + incoming.getVPIDEntityKey().getKeyValueAsString());
					//don't throw exception, nothing we can do
					logger.error("Cannot process person. onFile person not found for vpid= " + incoming.getVPIDEntityKey().getKeyValueAsString());
					return null;
		        }

			} else {

				//logger.info("Incoming person's vpid : " + incoming.getVPIDEntityKey().getKeyValueAsString());

				//person known to ES, get ADR person without person traits
				adrPerson = this.getPersonService().getPersonWithoutIdentityTraits(incoming.getVPIDEntityKey());
				/*if (logger.isDebugEnabled()){
					logger.debug("person details for person known to ES:" + adrPerson);

				}*/

		        if (adrPerson == null) {
		        	//throw new ServiceException("Cannot process person. onFile person not found for vpid= " + incoming.getVPIDEntityKey().getKeyValueAsString());
					//don't throw exception, nothing we can do
					logger.error("Cannot process person. onFile person not found for vpid= " + incoming.getVPIDEntityKey().getKeyValueAsString());
					return null;
		        }

		        if (adrPerson != null){
		        	personId = new BigDecimal(adrPerson.getEntityKey().getKeyValueAsString());
		        	//logger.info("person id is: " + personId);
		        	//logger.info("adrPerson.getEntityKey().getKeyValueAsString() :" + adrPerson.getEntityKey().getKeyValueAsString());
		        }
				//4.7.	Check for Pending Identity Trait Update
				// The system verifies there are no pending updates to the persons identity traits.
				if (PENDING_CAT_EDIT.equals(esrCorr.getIdState())) {
//					5.4.	SUC119.12 - Identity Trait Update is Pending
//					In the basic flow step, Check for Pending Identity Trait Update, the person indicated on the VOA message has Pending updates to their Identity Traits.
//					5.4.1.	Update Message Status for Pending Person Identity Trait Updates
//					The system sets the Processing Status to Form Processing Pending.
//					5.4.2.	Create a Worklist Item
//					An ES Worklist item is created.
//						Functional Group:  EE
//						Item Type: Veteran's Online Application
//						Reason: Identity Trait Updates are pending for the person on the VOA submitted form.
//					The use case ends.

					this.setVOAFormPending(app, incoming, "Pending Catastrophic Edit (PCE), vpid= " + esrCorr.getIdentity());

					this.getWorkflowService().autoCreateCase(
							incoming,
							getLookupService().getFunctionalGroupByCode(FunctionalGroup.EE.getCode()),
							getLookupService().getWkfCaseTypeByCode(WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION.getCode()),
							null,
							"Identity Trait Updates are pending for the person on the VOA submitted form.");
					return null;
				} else
				{
					//4.6.	Update Existing Person Record
//					If the person is known to VA and a record for that person is found in ES, update the persons record with the identity traits received in the message.
//					The use case resumes at the basic flow step Update Existing Person Record.
					//update person 200ESR corrleation
					//in processVOA, retrieve 200ESR Correlation view and compare key elements with VOA forms before update 200ESR Correlation -
					//avoid unnessary update and wipe out certain 200ESR correlation elements
					incomingCopy = updateESRCorrelationIfNeeded(incoming, adrPerson);
					if (incoming.getPreferredFacilities() != null && incoming.getPreferredFacilities().size() > 0) {
						PreferredFacility facility = (PreferredFacility)incoming.getPreferredFacilities().iterator().next();
						facility.setPerson(null);
						incomingCopy.addPreferredFacility(facility);
					}
				}
			}

			//set person id because creating wkfl case needs personId
			incoming.setIdentifier(personId);



			/*if (logger.isDebugEnabled()) {
				logger.debug("Populated Person when corre found: " + incoming.toString());

				if (incoming != null){
					logger.debug("income test after coming to voaSvc: " + incoming.getIncomeTest(Calendar.getInstance().get(Calendar.YEAR)-1));

				}
			}*/


			//person found, set the VOA_IND
			if (!VOA_IND_2.equals(adrPerson.getVOAIndicator())) {
				incoming.setVOAIndicator(VOA_IND_1);

				//CCR 13474: move the add PF inside the condition when VOA_IND != 2
				//if incoming person has a new PF not exist in MVI correlations, add it to MVI
				//NOTE: pass in the correlations so no need for an extra MVI call
				try {
					this.getPersonService().addPreferredFacilityCorrelation(incomingCopy, correlations);

				} catch (ServiceException ex) {
					//CCR12792: When processing a VOA submission, if Add Proxy Prefered Facility fails the
					//Form Processing Status should be set to PENDING and all processing should STOP.
					this.setVOAFormPending(app, incoming, "Request for VOA Proxy Preferred Facility Patient Record Create Failed due to exception: " + ex.getMessage());

					this.getWorkflowService().autoCreateCase(incoming, getLookupService().getFunctionalGroupByCode(FunctionalGroup.EE.getCode()),
							getLookupService().getWkfCaseTypeByCode(WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION.getCode()), null,
							"Request for VOA Proxy Preferred Facility Patient Record Create Failed");

					return null;
				}
			}
		}

		//create workflow item if radiation exposure is true
		processRadiationExposure(incoming);
		//per CR 309168, business decision to accept medicaid from VOA
		//WI 311693
		//processMedicaid(incoming);

		/*if (logger.isDebugEnabled()) {
			logger.debug("Populated Person before sending to processCommonVoa(): " + incoming.toString());

			if (incoming != null){
				logger.debug("income test before processCommonVoa(): " + incoming.getIncomeTest(Calendar.getInstance().get(Calendar.YEAR)-1));

			}
		}
		*/
		return processCommonVOA(app, incoming, correlations, true);
	}


	/**
	 * RTC Task 236652  - When a veteran answers no to the disclose financial details option the VOA form, ES/VOA
	 * backend sets some default fields to the IncomeTest so that Means Test does not go into the Pending status.
	 * Check if the veteran already exists in ES, when the VOA form is submitted anonymously
	 */
	public Person getOnFilePersonPersonForAnonymousPerson(VOAApplication app, Person incoming) throws ServiceException {
			Validate.notNull(app, "VOA application must not be null");
			Validate.notNull(incoming, "Incoming Person must not be null");

			//get the correlations by fully qualified id or person traits
			PersonIdentityTraits traits = null;
			List<PatientIdentifier> correlations = null;

			IdMSearchInfo searchInfo = new IdMSearchInfo();
			searchInfo.setExtendedSearch(true);
			searchInfo.setDataEntererExtension("SSN= " + app.getSsn());
			searchInfo.setDataEntererFamilyName(app.getLastName());
			searchInfo.setDataEntererGivenName(app.getFirstName());

			//retrieve person/correlations from MVI if FQID is provided
			if (!StringUtils.isEmpty(app.getFullQualifiedId()) && app.getFullQualifiedId().split("\\^").length == 4){

				logger.info("FQID is present");

				String[] ids = app.getFullQualifiedId().split("\\^");
				logger.info("IDs : " + ids.toString());

				//retrieve by fully qualified id (1305 with composite & extended search)
				traits = idmServiceDelegate.searchWithCompositeCall(new FullyQualifiedIdentity(ids[0], ids[1], ids[2], ids[3]), false, searchInfo);

				//if not found, search by person traits instead (1305 composite call with extended search)
				if (traits == null) {
					logger.info("MVI composite search returned no traits");
					//search by person traits (1305 composite call with extended search)
					correlations = getCorrelationsFromCompositeCall(incoming.getIdentityTraits(), searchInfo);
				} else {
					logger.info("MVI composite search returned traits, now retrieving correlations");
					correlations = traits.getAllCorrelations();
				}
			}
			else {
				logger.info("No FQID present, MVI composite search performed by traits.");
				//search by person traits (1305 composite call with extended search)
				correlations = getCorrelationsFromCompositeCall(incoming.getIdentityTraits(), searchInfo);
			}

			Person adrPerson = null;
			//person does not exist in ES
			if (correlations == null || correlations.size() == 0) {
				logger.info("no correlations found");
				adrPerson = null;

			} else {
				//correlations found
				//ESR internally needs long vpid because person table supports only long vpid

				String vpid = VPIDEntityKeyImpl.getLongVPID(getICN(correlations));
				logger.info("correlations found, vpid is : " + vpid);
				incoming.setVPIDValue(vpid);

				//verify if person already exists in ES. chk if 200ESR Correlation already exists
				PatientIdentifier esrCorr = get200ESRCorrelation(correlations);
				if (esrCorr == null){
					logger.info("no ESR correlations found for this vpid: " + vpid);
					adrPerson = null;  //Person does not exist in ESR yet
				}
				else {
					//person known to ES, get ADR person without person traits
					logger.info("ESR correlations found for this vpid: " + vpid);
					adrPerson = this.getPersonService().getPersonWithoutIdentityTraits(incoming.getVPIDEntityKey());
					/*if (adrPerson != null){
						logger.info("Person exists in ESR with VPID : " + adrPerson.getVPIDEntityKey().getKeyValueAsString());
					}*/
			        if (adrPerson == null) {
			        	//throw new ServiceException("Cannot process person. onFile person not found for vpid= " + incoming.getVPIDEntityKey().getKeyValueAsString());
						//don't throw exception, nothing we can do
						logger.error("Cannot process person. onFile person not found for vpid= " + incoming.getVPIDEntityKey().getKeyValueAsString());
						return null;
			        }
			        return adrPerson;

				}
		}
			return adrPerson;
  }

	/**
	 * RTC Task 236652  - When a veteran answers no to the disclose financial details option the VOA form, ES/VOA
	 * backend sets some default fields to the IncomeTest so that Means Test does not go into the Pending status.
	 * Check if the veteran already exists in ES, when the VOA form is submitted from the authenticated VOA site
	 */
   @SuppressWarnings("unchecked")
   public Person getOnFilePersonForPersonWithDSLogon(VOAApplication app, Person incoming) throws ServiceException  {
		Validate.notNull(app, "VOA application must not be null");
		Validate.notNull(incoming, "Incoming Person must not be null");

		// Search for Person: search by FQI: VPID/ICN,  EDIPI, USAA, SEC ID
		if (StringUtils.isEmpty(app.getFullQualifiedId())) {
			logger.error("VOA LOA2+ Aplication with empty fully qualified identifier");
			return null;
		}

		List<PatientIdentifier> correlations = idmServiceDelegate.getCorrelationsByFullyQualifiedIdentifier(app.getFullQualifiedId());
		if (correlations == null || correlations.size() == 0) {
			return null;
		}

		//correlations found
		// ESR internally needs long vpid because person table supports only
		// long vpid
		String vpid = VPIDEntityKeyImpl.getLongVPID(getICN(correlations));
		incoming.setVPIDValue(vpid);

		// Check if ES correlation is there/ 200ESR Correlation
		PatientIdentifier esrCorr = get200ESRCorrelation(correlations);

		Person adrPerson = null;
		if (esrCorr == null) {
			//Person Not Known to ES
			logger.info("no esr correlation found - person not known to ES");
			return null;
		} else {
			// existing person, get ADR person without person traits
			adrPerson = this.getPersonService().getPersonWithoutIdentityTraits(incoming.getVPIDEntityKey());
			/*if (adrPerson != null){
				logger.info("person not known to ES: " + adrPerson.getVPIDEntityKey().getKeyValueAsString());
			}*/
			if (adrPerson == null) {
				// throw new
				// ServiceException("Cannot process person. onFile person not found for vpid= "
				// + incoming.getVPIDEntityKey().getKeyValueAsString());
				// don't throw exception, nothing we can do
				logger.error("Cannot process person. onFile person not found for vpid= "+ incoming.getVPIDEntityKey().getKeyValueAsString());
				return null;
			}

		}
		return adrPerson;
	}



	//CCR12789 - split the branch out to process DSLoginLevel2
	@SuppressWarnings("unchecked")
	private Person processDSLogonLevel2(VOAApplication app, Person incoming)
			throws ServiceException {
		Validate.notNull(app, "VOA application must not be null");
		Validate.notNull(incoming, "Incoming Person must not be null");

		// 4.5. Search for Person: search by FQI: VPID/ICN,  EDIPI, USAA, SEC ID
		if (StringUtils.isEmpty(app.getFullQualifiedId())) {
			// throw new ServiceException("VOA LOA2+ Aplication with empty fully qualified identifier");
			// don't throw exception, nothing we can do
			logger.error("VOA LOA2+ Aplication with empty fully qualified identifier");
			return null;
		}

		Person incomingCopy = incoming;

		List<PatientIdentifier> correlations = idmServiceDelegate
				.getCorrelationsByFullyQualifiedIdentifier(app
						.getFullQualifiedId());

		if (correlations == null || correlations.size() == 0) {
			// 5.2. Person Not Found
			// In the Basic Flow step Search for Person, the person indicated
			// in the VOA message is not found (after a query of MVI).
			// 5.2.1. Set VOA Message status to Form Processing Pending
			// When the person (i.e., veteran applicant) indicated in the VOA
			// message is not found, the system sets the VOA processing status
			// to Form Processing Pending.
			// 5.2.2. Create a Worklist Item
			// An ES Worklist item is created.
			//  Functional Group: EE
			//  Item Type: Veteran's Online Application
			//  Reason: Person not found .
			//
			// The use case ends.

			setVOAFormPending(app, incoming,
					"Person not found from MVI with FQID= "
							+ app.getFullQualifiedId());

			// Trigger Letter SUC141.16: 17711 Person Not Found after a query of
			// MVI for an incoming Veterans Online Application message. Send
			// Email Bulletin
			// send bulletin
			sendPersonNotFoundEmail(app, incoming);
			return null;
		}

		//correlations found

		// ESR internally needs long vpid because person table supports only
		// long vpid
		String vpid = VPIDEntityKeyImpl.getLongVPID(getICN(correlations));
		incoming.setVPIDValue(vpid);

		BigDecimal personId = null;

		// When ES requests a 200ESR correlation, ES will send all the trait
		// information from the form
		// because the form may include additional attributes which were entered
		// by the user
		// (for e.g.. address, phone, maiden name etc) after the ICN was
		// initially created.
		//
		// add 200ESR Correlation if does not exist, also add person stub
		PatientIdentifier esrCorr = get200ESRCorrelation(correlations);

		//set Voa default values, some values can be used for MVI add/update
		setVoaDefaults(incoming, app);

		Person adrPerson = null;
		if (esrCorr == null) {
			// 5.3. Person Not Known to ES
			// In the Basic Flow step Search for Person, if the person indicated
			// on the VOA message is found but is not yet known to ES, a person
			// record is added to ES.
			// When ES requests MVI to create a 200ESR correlation, MVI will
			// create a stub record in person table
			// as it happens today with all other 200ESR correlations.
			this.getPersonService().addESRCorrelation(incoming);

			// the email from Cory Chin on Friday, January 25, 2013 confirmed
			// that:
			// It is our understanding that PSIM currently creates a stub record
			// in person table under the following scenarios.
			//
			// 1) When a new user is added in VistA and when VistA sends a
			// request to MVI to create the national ICN
			// 2) When a new user is added through ESR UI  Add A person
			// functionality
			//
			// A person stub record is not created when a  Add 200 ESR
			// correlation is made.

			// Thus ES has to insert person stub by inserting a new entry to
			// PS_PERSON_VPID and PERSON table

			PersonEntityKey personKey = null;
			try {
				// MAKE SURE TO COMMENT OUT THE NEXT TWO LINES - only needed for
				// EDEV
				// since EDEV is not fully integrated with MVI: insert into
				// PS_PERSON_VPID manually
				// PersonVPID pv = new PersonVPID(incoming.getVPIDValue());
				// this.getVoaApplicationDAO().insertObject(pv);

				// CCR 12099 - check person stub exists or not before add it.
				// NOTE: a migration data can have person stub without 200ESR
				// Correlation!
				personKey = getPersonService().getPersonIdByVPID(
						incoming.getVPIDEntityKey());
				if (personKey == null) {
					// add to ADR PERSON table (person stub)
					personKey = getPersonService().addPersonStub(
							incoming.getVPIDEntityKey()); // insert into PERSON

					// must flush hibernate so the person id will be available
					// right away to avoid null pointer exception
					this.getVoaApplicationDAO().flush();
				}

			} catch (DAOException ex) {
				throw new ServiceException(
						"insert into PS_PERSON_VPID failed: ", ex);
			}
			personId = (BigDecimal) personKey.getKeyValue();

			// get the ADR person with vpid, without person traits
			adrPerson = this.getPersonService().getPersonWithoutIdentityTraits(
					incoming.getVPIDEntityKey());

			if (adrPerson == null) {
				// throw new
				// ServiceException("Cannot process person. onFile person not found for vpid= "
				// + incoming.getVPIDEntityKey().getKeyValueAsString());
				// don't throw exception, nothing we can do
				logger
						.error("Cannot process person. onFile person not found for vpid= "
								+ incoming.getVPIDEntityKey()
										.getKeyValueAsString());
				return null;
			}

			// creating wkfl case needs personId
			incoming.setIdentifier(personId);
		} else {
			// existing person, get ADR person without person traits
			adrPerson = this.getPersonService().getPersonWithoutIdentityTraits(
					incoming.getVPIDEntityKey());

			if (adrPerson == null) {
				// throw new
				// ServiceException("Cannot process person. onFile person not found for vpid= "
				// + incoming.getVPIDEntityKey().getKeyValueAsString());
				// don't throw exception, nothing we can do
				logger
						.error("Cannot process person. onFile person not found for vpid= "
								+ incoming.getVPIDEntityKey()
										.getKeyValueAsString());
				return null;
			}

			personId = new BigDecimal(adrPerson.getEntityKey()
					.getKeyValueAsString());
			incoming.setIdentifier(personId);

			// 4.7. Check for Pending Identity Trait Update
			// The system verifies there are no pending updates to the persons
			// identity traits.
			if (PENDING_CAT_EDIT.equals(esrCorr.getIdState())) {
				// 5.4. SUC119.12 - Identity Trait Update is Pending
				// In the basic flow step, Check for Pending Identity Trait
				// Update, the person indicated on the VOA message has Pending
				// updates to their Identity Traits.
				// 5.4.1. Update Message Status for Pending Person Identity
				// Trait Updates
				// The system sets the Processing Status to Form Processing
				// Pending.
				// 5.4.2. Create a Worklist Item
				// An ES Worklist item is created.
				//  Functional Group: EE
				//  Item Type: Veteran's Online Application
				//  Reason: Identity Trait Updates are pending for the person
				// on the VOA submitted form.
				// The use case ends.

				this.setVOAFormPending(app, incoming,
						"Pending Catastrophic Edit (PCE), vpid= "
								+ esrCorr.getIdentity());

				this
						.getWorkflowService()
						.autoCreateCase(
								incoming,
								getLookupService().getFunctionalGroupByCode(
										FunctionalGroup.EE.getCode()),
								getLookupService()
										.getWkfCaseTypeByCode(
												WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION
														.getCode()), null,
								"Identity Trait Updates are pending for the person on the VOA submitted form.");
				return null;
			} else {
				// 4.6. Update Existing Person Record
				// If the person is known to VA and a record for that person is
				// found in ES, update the persons record with the identity
				// traits received in the message.
				// The use case resumes at the basic flow step Update Existing
				// Person Record.
				// update person 200ESR corrleation
				// in processVOA, retrieve 200ESR Correlation view and compare
				// key elements with VOA forms before update 200ESR Correlation
				// -
				// avoid unnessary update and wipe out certain 200ESR
				// correlation elements
				incomingCopy = updateESRCorrelationIfNeeded(incoming, adrPerson);
				if (incoming.getPreferredFacilities() != null && incoming.getPreferredFacilities().size() > 0) {
					PreferredFacility facility = (PreferredFacility)incoming.getPreferredFacilities().iterator().next();
					facility.setPerson(null);
					incomingCopy.addPreferredFacility(facility);
				}
			}
		}

    	//for DS logon level2, the VOA submissions are from veterans
    	incoming.setVeteran(Boolean.TRUE);

		//person found, set the VOA_IND
    	//CCR13419- removed it so that in the postProcessVOASubmission()
    	//when the enrollment status = PENDING does not prevent sharing with VistA sit for LOA2+
/*		if (!VOA_IND_2.equals(adrPerson.getVOAIndicator())) {
			incoming.setVOAIndicator(VOA_IND_1);
		}*/

		//if incoming person has a new PF not exist in MVI correlations, add it to MVI
		//NOTE: pass in the correlations so no need for an extra MVI call
		try {
			this.getPersonService().addPreferredFacilityCorrelation(incomingCopy, correlations);
		} catch (ServiceException ex) {
			//CCR12792: When processing a VOA submission, if Add Proxy Prefered Facility fails the
			//Form Processing Status should be set to PENDING and all processing should STOP.
			this.setVOAFormPending(app, incoming, "Request for VOA Proxy Preferred Facility Patient Record Create Failed due to exception: " + ex.getMessage());

			this.getWorkflowService().autoCreateCase(incoming, getLookupService().getFunctionalGroupByCode(FunctionalGroup.EE.getCode()),
					getLookupService().getWkfCaseTypeByCode(WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION.getCode()), null,
					"Request for VOA Proxy Preferred Facility Patient Record Create Failed");

			return null;
		}

		processRadiationExposure(incoming);
		//per CR 309168, business decision to accept medicaid from VOA
		//WI 311693
		//processMedicaid(incoming);

    	return processCommonVOA(app, incoming, correlations, false);
	}

	@SuppressWarnings({ "unused", "rawtypes" })
	private boolean hasANewPreferredFacility(Person incoming, List<PatientIdentifier> correlations) {

		if (incoming.getPreferredFacility() == null || incoming.getPreferredFacilities().size() == 0 )
			return false;
		if (correlations == null || correlations.size() == 0)
			return true;

		String PF = ((PreferredFacility) incoming.getPreferredFacilities().iterator().next()).getFacility().getStationNumber();
		if (PF == null)
			return false;

//		MVI iRSD 2.6.1.2.5	ES to Proxy Add to VistA
//		If the VOA application form includes a preferred facility for the Veteran and there is not a
//		preferred facility correlation in the MVI record, ES will request creation of a preferred VistA correlation.
		Set stationNumbers = new HashSet();
		Iterator it = correlations.iterator();

		PatientIdentifier pid = null;
		while (it.hasNext())
		{
			pid = (PatientIdentifier)it.next();
			if (PF.equals(pid.getStationNumber()))
				return false; //matched found, not a new PF
		}

		return true;
	}
	/*
	 * Extract the common process at the second half of the VOA process for bother level 2 and above and anaonymouns & level 1
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private Person processCommonVOA(VOAApplication app, Person incoming, List<PatientIdentifier> correlations, boolean isAnonymousLOA1) throws ServiceException {

		Validate.notNull(incoming.getPersonEntityKey(), "Incoming Person here must have person id not null");

		// update person id
		BigDecimal personId = (BigDecimal)incoming.getPersonEntityKey().getKeyValue();

		app.setPersonId(personId);
		this.updateVOAApplication(app);

		if (app.getFileAttachments() != null && app.getFileAttachments().size() > 0) {
			// CCR 11996 make sure file attachment has person id
			for (Iterator it = app.getFileAttachments().iterator(); it.hasNext();) {
				VoaFileAttachment att = (VoaFileAttachment) it.next();
				att.setPersonId(personId);
			}

			// 5.3.1. SUC2031.15.1 - Attachment Indicator Test For Presence
			// In step [1436] Apply Processing Requirements, if this is an
			// incoming message from the Veteran's Online Application and the
			// Evidence of Attachments is Yes, create an E&E Case as follows:
			// Type: Veteran's Online Application
			// Reason: VOA Attached Evidence Provided by Applicant.
			this.getWorkflowService().autoCreateCase(incoming, getLookupService().getFunctionalGroupByCode(FunctionalGroup.EE.getCode()),
					getLookupService().getWkfCaseTypeByCode(WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION.getCode()), null,
					"VOA Attached Evidence Provided by Applicant.");
		}

		VAFacility sendingFacility = VOA_VA_FACILITY;

		VerificationInfo info = new VerificationInfo();
		info.setSiteNumber(sendingFacility.getCode());

		// VOA process may generate multiple workload cases and cannot share the
		// same workflow case object
		// created for potential usage by Rule. so remove the code for that


		// processVOA() is a cloned & revised from exisint processZ07() because
		// both share similar processing logic
		// it performs many tasks, but mainly 4.10 Prpcess Person Information
		// UC:
		// 4.8. Delete In-Process Income Test
		// Delete any income test that is in an In-Process state.
		// 4.10. Process Person Information &
		// 4.11. Accept Person Record
		// 4.12. Query VBA (in processVOA()
		// getEventRuleService().processMessageEvents(), which envokes
		// TriggerMessage ilog rule)
		// Trigger the Query VBA use case to obtain from VBA (MVR) eligibility
		// information on the veteran indicated
		// in the VOA message if any of the following conditions are true:
		//  The VOA submission is for an update to an existing person with a
		// Veteran Primary Eligibility Code
		// or Veteran Indicator is Yes and the VBA Query Status is NULL.
		// 4.13. Query MSDS (in processVOA()
		// getEventRuleService().processMessageEvents(), which envokes
		// TriggerMessage ilog rule)
		// Trigger the Process Send Receive MSDS Service use to obtain Military
		// Data for the Veteran indicated in the VOA message if any of the
		// following conditions are true:
		//  The VOA Submission is for a new record and the Veteran Indicator is
		// "Yes".
		//  The VOA submission received has a Veteran Indicator = Yes and the
		// MSDS Query status = Queried  No Data Received and the total number
		// of completed MSDS Queries is greater than 2 (i.e., retry count).
		//  If the incoming message contains new Military Service Episode.
		//  If the incoming message contains new OEF/OIF data.
		//  If the MSDS Reconciliation Indicator is equal to Y.
		//  If the incoming message contains Military Service data that is
		// different than the on file Military Service Data.
		//  If the incoming message contains OEF/OIF data that is different
		// than the on file OEF/OIF Data.
		// 4.14. Process MSDS Data
		// Execute the MI Process Person MSDS Data UC use case.


		//set the CLV registration date with VOA application date before processing the ilog rules
		CampLejeuneVerification cl = (CampLejeuneVerification)incoming.getSpecialFactorByType(CampLejeuneVerification.class);
		if (cl != null)
			cl.setRegistrationDate(new ImpreciseDate(app.getCreatedOn()));


		Person result = null;
		try {
			result = this.getMessagingService().processVOA(incoming, sendingFacility, info, isAnonymousLOA1, correlations);

			/*if (logger.isDebugEnabled()) {
				logger.debug("Incomign Person after calling processVOA(): " + incoming.toString());
				logger.debug(" Result Person after calling processVOA(): " + result.toString());

				if (incoming != null){
					logger.debug("income test of incomign after processVoa(): " + incoming.getIncomeTest(Calendar.getInstance().get(Calendar.YEAR)-1));
					logger.debug("result person income test after processVoa() result...: " + result.getIncomeTest(Calendar.getInstance().get(Calendar.YEAR)-1));

				}
			}		*/
		}
		catch (AddProxyPreferredFacilityException aex) {
			//CCR12792: When processing a VOA submission, if Add Proxy Prefered Facility fails the
			//Form Processing Status should be set to PENDING and all processing should STOP.
			//The workload case is created when the exception is thrown, so just put it in pending status & resubmit queue
			//CCR13417: get the stacktrace to get more detail
			logger.error("Request for VOA Proxy Preferred Facility Patient Record Create Failed due to exception: (submission id=" +app.getEntityKey().getKeyValueAsString() +"): "+ ExceptionUtils.getFullStackTrace(aex));
			this.setVOAFormPending(app, incoming, "Request for VOA Proxy Preferred Facility Patient Record Create Failed due to exception: " + ExceptionUtils.getRootCause(aex));
			return null;

		} catch (ServiceException ex) {
			//CCR13417: log it with submission id for better resolution
			logger.error("VOA Process submission failed due to exception: (submission id=" +app.getEntityKey().getKeyValueAsString() +"): " + ExceptionUtils.getFullStackTrace(ex));
			// if failed, put failed status and error text
			app.setApplicationStatus((VOAApplicationStatus) this.getLookupService().getByCode(VOAApplicationStatus.class, VOAApplicationStatus.FORM_SUBMISSION_FAILED));
			//CCR13417: get the stacktrace to get more detail
			//Modified the exception handling for logging in to database.
			//app.setErrorText("VOA Process submission failed due to exception: " + ExceptionUtils.getRootCause(ex));
			app.setErrorText("VOA Process submission failed due to exception: " + ExceptionUtils.getCause(ex));

			this.updateVOAApplication(app);

			return null;
			// throw ex; //CCR12575: do not re-throw, jsut exit so the whole
			// transaction will not roll back and leave the app in status =1
		} catch (Throwable t) {
			//for some reasons, it throws null pointer exception for the next line, comment it out for now until the reason is found
			//TODO this was commented out..test before putting in upper env..
			//app.setApplicationStatus((VOAApplicationStatus) this.getLookupService().getByCode(VOAApplicationStatus.class, VOAApplicationStatus.FORM_SUBMISSION_FAILED));
			//CCR13417: get the stacktrace to get more detail
			logger.error("Throwable exception during VOA Process submission: (submission id=" +app.getEntityKey().getKeyValueAsString() +"): " + ExceptionUtils.getFullStackTrace(t));
			//use full stack trace here because root cause is null for Throwable
			app.setErrorText("Throwable exception during VOA Process submission: " + ExceptionUtils.getFullStackTrace(t));

			this.updateVOAApplication(app);
			//t.printStackTrace();

			return null;
		}

		// 5.5. Person Information Not Accepted
		// In the basic flow step, Process Person Information, the data in an
		// incoming VOA message is not accepted by the Process Person
		// Information use case.
		// 5.5.1. Update Message Status for Person Information Not Accepted
		// The system sets the Processing Status to Form Processing Pending.
		// 5.5.2. Create a Worklist Item
		// An ES Worklist item is created.
		//  Functional Group: EE
		//  Item Type: Veteran's Online Application
		//  Reason: The information for the person on the VOA submitted form
		// was not accepted.

		// The use case ends.
		if (result.getEnrollmentDetermination() == null && result.getEnrollmentDeterminations().size() == 0) {
			this.setVOAFormPending(app, incoming, "The information for the person on the VOA submitted form was not accepted.");

			this.getWorkflowService().autoCreateCase(incoming, getLookupService().getFunctionalGroupByCode(FunctionalGroup.EE.getCode()),
					getLookupService().getWkfCaseTypeByCode(WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION.getCode()), null,
					"The information for the person on the VOA submitted form was not accepted.");

			return null;
		}

		// 4.15. Set VOA Processing Status to "Form Processed Successfully"
		app.setApplicationStatus((VOAApplicationStatus) this.getLookupService().getByCode(VOAApplicationStatus.class, VOAApplicationStatus.FORM_PROCESSED_SUCCESSFUL));
		this.updateVOAApplication(app);


		// INC000001219868
		//RTC Task 236652  - veteran selects No for the disclose financial details from the VOA UI  (added null check)
		if (app.getFullQualifiedId() != null){
			removeOrphanedVOAApplication(app);
		}


		// SUC141.18 Bulletin Triggers Decision: decision table row #4:
		// A confirmation email is sent to the veteran when their VOA form
		// submission is accepted for EE processing.

		if (incoming.getEmails() != null && incoming.getEmails().size() > 0) {
			String toList[] = new String[incoming.getEmails().size()];
			int i = 0;
			for (Iterator it = incoming.getEmails().iterator(); it.hasNext();) {
				toList[i] = ((Email) it.next()).getAddress();
				i++;
			}

			if (bulletinService != null) {
				Map bulletinData = new Hashtable();
				bulletinData.put("FirstName", incoming.getLegalName().getGivenName());
				bulletinData.put("LastName", incoming.getLegalName().getFamilyName());
				bulletinData.put("PreferredFacility", incoming.getPreferredFacility().getDescription());
				bulletinData.put("SubmissionTime", app.getCreatedOn());
				bulletinData.put("SubmissionId", app.getEntityKey().getKeyValueAsString());

				try {

					bulletinService.sendEmailBulletinTo(BulletinTrigger.DataType.Initial_Submission_Confirmation, bulletinData, toList);
				} catch (Exception mex) {
					//nothing we can do, log it
					logger.error("VOA end of process - cannot send email notification for appId= " + app.getEntityKey().getKeyValueAsString()
							+ "\nError Message: " + mex.getMessage());
				}
			}
		}


		return result;
	}

	private void setVOAFormPending(VOAApplication app, Person incoming, String errorTxt)
			throws ServiceException {
		app.setApplicationStatus((VOAApplicationStatus) this.getLookupService()
				.getByCode(VOAApplicationStatus.class,
						VOAApplicationStatus.FORM_PROCESSING_PENDING));
		app.setErrorText(errorTxt);

		VOAResubmissionData data = app.getVOAResubmissionData();
		if (data == null)
		{
			data = new VOAResubmissionData();
			data.setVoaApplicationId((BigDecimal)app.getEntityKey().getKeyValue());
			app.setVOAResubmissionData(data);
		}

		try {
			data.setApplicationData(this.serializeXML(incoming));
		} catch (SQLException se)
		{
			throw new ServiceException("processVoaApplication failed when setVOAFormPending - unable to serialize Person object. Submission id = " +
					app.getEntityKey().getKeyValueAsString(), se);
		}

		if (incoming.getEntityKey() != null)
			app.setPersonId((BigDecimal)incoming.getEntityKey().getKeyValue());

		this.updateVOAApplication(app);

	}

    private Person updateESRCorrelationIfNeeded(Person incoming, Person adrPerson) throws ServiceException
    {
    	//email from Sudha on 2/1/2013
    	//So if 200ESR correlation exists in MVI, you will retrieve it , you will check each one of the form elements below
    	//and if they are not null, compare it with 200 ESR correlation and if different update it.
    	//If nothing is different, you will not send an update 200 ESR correlation.

    	/***
    	 * RTC Defect # 277655 - VOA sending MVI update with incorrect traits (SDM I7407534FY16)
    	 * Fix is to not use VOA traits to send it to MVI as update 200 ESR correlation
    	 * instead use MVI/ESR traits as part of update ESR correlation and only update address and phone numbers.
    	 */
    	PersonIdentityTraits esrTraits = this.getPersonService().getESRCorrelation(incoming.getVPIDEntityKey());
    	if (esrTraits == null)
    		throw new ServiceException ("Process VOA application - 200ESR Person Traits does not exist for vpid = " + incoming.getVPIDEntityKey().getKeyValue());


    	boolean isUpdated = false;
    	Person adrPersonCopy = (Person)adrPerson.clone();

    	/***
    	 * RTC Defect # 277655 - VOA sending MVI update with incorrect traits (SDM I7407534FY16)
    	 * Fix is to not use VOA traits to send it to MVI as update 200 ESR correlation
    	 * instead use MVI/ESR traits as part of update ESR correlation and only update address and phone numbers.

    	//last, first, middle name & suffix
    	if (incoming.getLegalName() != null && esrTraits.getLegalName() != null )
    	{
    		Name voaName = incoming.getLegalName();
    		Name esrName = esrTraits.getLegalName();
    		if (!StringUtils.isEmpty(voaName.getFamilyName()) && !voaName.getFamilyName().equalsIgnoreCase(esrName.getFamilyName())) {
    			//esrName.setFamilyName(voaName.getFamilyName());
    			isUpdated = true;
    		}
    		if (!StringUtils.isEmpty(voaName.getGivenName()) && !voaName.getGivenName().equalsIgnoreCase(esrName.getGivenName())) {
    			//esrName.setGivenName(voaName.getGivenName());
    			isUpdated = true;
    		}
    		if (!StringUtils.isEmpty(voaName.getMiddleName()) && !voaName.getMiddleName().equalsIgnoreCase(esrName.getMiddleName())) {
    			//esrName.setMiddleName(voaName.getMiddleName());
    			isUpdated = true;
    		}
    		if (!StringUtils.isEmpty(voaName.getSuffix()) && !voaName.getSuffix().equalsIgnoreCase(esrName.getSuffix())) {
    			//esrName.setSuffix(voaName.getSuffix());
    			isUpdated = true;
    		}

    	}

    	//other name used
    	if (incoming.getAliasName() != null && esrTraits.getNames() != null )
    	{
    		Name voaName = incoming.getAliasName();
    		Name esrName = Name.getNameOfType(esrTraits.getNames(),  NameType.ALIAS_NAME.getName());
    		if (esrName == null) {
    			//esrName = voaName;
    			isUpdated = true;
    		} else {
	    		if (!StringUtils.isEmpty(voaName.getFamilyName()) && !voaName.getFamilyName().equalsIgnoreCase(esrName.getFamilyName())) {
	    			//esrName.setFamilyName(voaName.getFamilyName());
	    			isUpdated = true;
	    		}
	    		if (!StringUtils.isEmpty(voaName.getGivenName()) && !voaName.getGivenName().equalsIgnoreCase(esrName.getGivenName())) {
	    			//esrName.setGivenName(voaName.getGivenName());
	    			isUpdated = true;
	    		}
	    		if (!StringUtils.isEmpty(voaName.getMiddleName()) && !voaName.getMiddleName().equalsIgnoreCase(esrName.getMiddleName())) {
	    			//esrName.setMiddleName(voaName.getMiddleName());
	    			isUpdated = true;
	    		}
    		}
    	}
    	//mother maiden name
    	if (!StringUtils.isEmpty(incoming.getMothersMaidenName()) && !incoming.getMothersMaidenName().equalsIgnoreCase(esrTraits.getMothersMaidenName()))
    	{
    		//esrTraits.setMothersMaidenName(incoming.getMothersMaidenName());
    		isUpdated = true;
    	}
    	//ssn text
    	if (incoming.getOfficialSsn() != null && !StringUtils.isEmpty(incoming.getOfficialSsn().getSsnText()) &&
    			(esrTraits.getSsn() == null || !incoming.getOfficialSsn().getSsnText().equalsIgnoreCase(esrTraits.getSsn().getSsnText())))
    	{
    		//esrTraits.setSsn(incoming.getOfficialSsn());
    		isUpdated = true;
    	} else {
    		if (incoming.getOfficialSsn() != null && esrTraits.getSsn() != null) {
	    		//esrSSN may have ssn verification status and other ssn values. Don't want to wipe out those.
	    		//since the voaSSN has the same ssn text, copy esrSSN to voaSSN
	    		incoming.removeSsn(incoming.getOfficialSsn());
	    		incoming.addSsn(esrTraits.getSsn());
    		}
    	}
    	//DOD yyyyMMdd
    	if (incoming.getDeathRecord() != null && incoming.getDeathRecord().getDeathDate() != null &&
    			!incoming.getDeathRecord().getDeathDate().getStandardFormat().equals(esrTraits.getDateOfDeathText()))
    	{
    		//esrTraits.setDateOfDeathText(incoming.getDeathRecord().getDeathDate().getStandardFormat());
    		isUpdated = true;
    	}
    	//gender
    	if (incoming.getGender() != null && !incoming.getGender().equals(esrTraits.getGender()))
    	{
    		//esrTraits.setGender(incoming.getGender());
    		isUpdated = true;
    	}
    	//Place of Birth City & State
    	if (incoming.getBirthRecord() != null)
    	{
    		if (esrTraits.getBirthRecord() == null)
    		{
    			//esrTraits.setBirthRecord(incoming.getBirthRecord());
    			isUpdated = true;
    		} else
    		{
    			if (!StringUtils.isEmpty(incoming.getBirthRecord().getCity()) &&
    				 (StringUtils.isEmpty(esrTraits.getBirthRecord().getCity()) ||
    				   !incoming.getBirthRecord().getCity().equalsIgnoreCase(esrTraits.getBirthRecord().getCity())))
    			{
    				//esrTraits.getBirthRecord().setCity(incoming.getBirthRecord().getCity());
    				isUpdated = true;
    			}
    			if (!StringUtils.isEmpty(incoming.getBirthRecord().getState()) &&
    					(StringUtils.isEmpty(esrTraits.getBirthRecord().getState()) ||
    						!incoming.getBirthRecord().getState().equalsIgnoreCase(esrTraits.getBirthRecord().getState())))
    			{
    				//esrTraits.getBirthRecord().setState(incoming.getBirthRecord().getState());
    				isUpdated = true;
    			}
    		}
    	}
		***/

    	Name esrName = esrTraits.getLegalName();
    	if (esrName != null) {
    		adrPersonCopy.setLegalName(esrName);
    	}

    	if (esrTraits.getNames() != null) {
    		esrName = Name.getNameOfType(esrTraits.getNames(),  NameType.ALIAS_NAME.getName());
    		if (esrName != null) {
    			adrPersonCopy.setAliasName(esrName);
    		}
    	}

    	if (!StringUtils.isEmpty(esrTraits.getMothersMaidenName()))
    	{
    		adrPersonCopy.setMothersMaidenName(esrTraits.getMothersMaidenName());
    	}

    	//ssn text
    	if (esrTraits.getSsn() != null)
    	{
    		if (adrPersonCopy.getOfficialSsn() != null) {
    			adrPersonCopy.removeSsn(adrPersonCopy.getOfficialSsn());
    		}
    		adrPersonCopy.addSsn(esrTraits.getSsn());
    	}

    	//DOD yyyyMMdd
    	if (!StringUtils.isEmpty(esrTraits.getDateOfDeathText()))
    	{
    		DeathRecord deathRecord = new DeathRecord();
    		Date date = stringToDate(esrTraits.getDateOfDeathText(), ImpreciseDateUtils.YYYYMMDD);
    		ImpreciseDate dateWithTime = ImpreciseDateUtils.createImpreciseDateWithoutTime(date);
            String impreciseString = dateWithTime.toStandardFormat();
            if (impreciseString.length() > 8) {
                impreciseString = impreciseString.substring(0, 8);
            }
            ImpreciseDate dateWithoutTime = new ImpreciseDate(impreciseString);
            deathRecord.setDeathDate(dateWithoutTime);
            deathRecord.setDeathReportDate(new Date()); //default it to current date for now as death reporting date
            deathRecord.setLazarusDate(null);
            deathRecord.setDataSource(
    			   this.getLookupService().getDeathDataSourceByCode(
    					   DeathDataSource.CODE_DEATH_CERT_ON_FILE.getCode()));
            deathRecord.setFacilityReceived(
    			   this.getLookupService().getVaFacilityByCode(VAFacility.CODE_HEC.getCode()));
    		adrPersonCopy.setDeathRecord(deathRecord);
    	}

    	//gender
    	if (esrTraits.getGender() != null)
    	{
    		adrPersonCopy.setGender(esrTraits.getGender());
    	}

    	//Place of Birth City & State
    	if (esrTraits.getBirthRecord() != null)
    	{
    		adrPersonCopy.setBirthRecord(esrTraits.getBirthRecord());
    	}

    	//NOTE: permanent address and home phone are not part of person traits from MVI
    	//permanent address
    	/***
    	 * RTC Defect # 277655 - VOA sending MVI update with incorrect traits (SDM I7407534FY16)
    	 * Fix is to not use VOA traits to send it to MVI as update 200 ESR correlation
    	 * instead use MVI/ESR traits as part of update ESR correlation and only update address and phone numbers.
    	 ***/
		Address newAddr = incoming.getPermanentAddress();
		Address oldAddr = adrPerson.getPermanentAddress();

		if (newAddr != null)
		{
			if (oldAddr == null)
			{
				isUpdated = true;
				Address newAddrCopy = (Address)newAddr.clone();
				newAddrCopy.setPerson(null);
				adrPersonCopy.addAddress(newAddrCopy);

			} else {
				if ( (!StringUtils.isEmpty(newAddr.getLine1()) && !newAddr.getLine1().equalsIgnoreCase(oldAddr.getLine1())) ||
						(!StringUtils.isEmpty(newAddr.getLine2()) && !newAddr.getLine2().equalsIgnoreCase(oldAddr.getLine2()) )||
						(!StringUtils.isEmpty(newAddr.getLine3()) && !newAddr.getLine3().equalsIgnoreCase(oldAddr.getLine3()) )||
						(!StringUtils.isEmpty(newAddr.getCity()) && !newAddr.getCity().equalsIgnoreCase(oldAddr.getCity()) )||
						(!StringUtils.isEmpty(newAddr.getState()) && !newAddr.getState().equalsIgnoreCase(oldAddr.getState()) )||
						(!StringUtils.isEmpty(newAddr.getZipCode()) && !newAddr.getZipCode().equalsIgnoreCase(oldAddr.getZipCode()) )||
						(!StringUtils.isEmpty(newAddr.getCountry()) && !newAddr.getCountry().equalsIgnoreCase(oldAddr.getCountry())) )
				{
					isUpdated = true;
					adrPersonCopy.removeAddress(oldAddr);
					Address newAddrCopy = (Address)newAddr.clone();
					newAddrCopy.setPerson(null);
					adrPersonCopy.addAddress(newAddrCopy);
				}
			}
		}

    	/***
    	 * RTC Defect # 277655 - VOA sending MVI update with incorrect traits (SDM I7407534FY16)
    	 * Fix is to not use VOA traits to send it to MVI as update 200 ESR correlation
    	 * instead use MVI/ESR traits as part of update ESR correlation and only update address and phone numbers.
    	 ***/
		//Home Phone
		if (incoming.getHomePhone() != null && !StringUtils.isEmpty(incoming.getHomePhone().getPhoneNumber()) &&
			  (adrPerson.getHomePhone() == null || !incoming.getHomePhone().getPhoneNumber().equalsIgnoreCase(adrPerson.getHomePhone().getPhoneNumber())))
		{
			isUpdated = true;
			if (adrPerson.getHomePhone() != null) {
				adrPersonCopy.removePhone(adrPerson.getHomePhone());
			}
			Phone homePhoneCopy = (Phone)incoming.getHomePhone().clone();
			homePhoneCopy.setPerson(null);
			adrPersonCopy.addPhone(homePhoneCopy);
		}

    	//something changed, need update 200ESR Correlation
    	/***
    	 * RTC Defect # 277655 - VOA sending MVI update with incorrect traits (SDM I7407534FY16)
    	 * Fix is to not use VOA traits to send it to MVI as update 200 ESR correlation
    	 * instead use MVI/ESR traits as part of update ESR correlation and only update address and phone numbers.
    	 ***/
    	if (isUpdated)
    	{
    		this.getPersonService().updateProfileForESRCorrelation(adrPersonCopy);
    	}

    	return adrPersonCopy;
    }

    @SuppressWarnings("rawtypes")
	private PatientIdentifier get200ESRCorrelation(List correlations)
    {
    	if (correlations == null || correlations.size() == 0)
    		return null;

    	PatientIdentifier pid = null;

		Iterator it = correlations.iterator();
		while (it.hasNext())
		{
			pid = (PatientIdentifier)it.next();
			if ("200ESR".equals(pid.getStationNumber()))
				return pid;
		}

		return null;
    }

    @SuppressWarnings("rawtypes")
	private String getICN(List correlations)
    {
    	if (correlations == null || correlations.size() == 0)
    		return null;

    	PatientIdentifier pid = null;

		Iterator it = correlations.iterator();
		while (it.hasNext())
		{
			pid = (PatientIdentifier)it.next();
			if ("200M".equals(pid.getStationNumber()))
				return pid.getIdentity();
		}

		return null;
    }

    @SuppressWarnings({ "rawtypes", "unused" })
	private void setVoaDefaults(Person incoming, VOAApplication app) throws ServiceException
    {
    	//VOA applications do not have all the necessary attributes needed for ES processing
    	//set the default value here

    	Date appLogDate = app.getCreatedOn();

    	//5.5.1.	SUC2031.17.1 - Permanent Address Country Not Required
    	//In step [1539], if on the incoming message Permanent Address Country is null
    	//then ES is to default Permanent Address Country to United States.
    	Address addr = incoming.getPermanentAddress();
//    	if (addr != null && StringUtils.isEmpty(addr.getCountry()))
//    		addr.setCountry(Country.CODE_USA.getName());


		//since voa does not have address updated date, use the receive timestamp
		//this will be used to compare with onFile date to ignore or upload the address
    	Address address = incoming.getPermanentAddress();
		if (address != null) {
			address.setChangeDate(appLogDate);
			address.setChangeSource(this.getLookupService().getAddressChangeSourceByCode(
		            AddressChangeSource.CODE_VAMC.getName())); //CCR12588
			address.setChangeSite(incoming.getPreferredFacility()); //CCR12588
		}

		//set the default phone change date so the phones can be loaded. VOA phones do not have change dates
		Set phones = incoming.getPhones();
		if (phones != null && phones.size() > 0)
		{
		    Iterator iter = phones.iterator();
		    while( iter.hasNext() ) {
		    	Phone ph = ((Phone)iter.next());
		    	ph.setChangeDate(appLogDate);
		    	ph.setSourceOfChange(this.getLookupService().getPhoneSourceOfChangeByCode(PhoneSourceOfChange.CODE_VAMC.getName()));//CCR12588
		    	ph.setSiteOfChange(incoming.getPreferredFacility());//CCR12588
		    }
		}

		//CCR12576
		Set emails = incoming.getEmails();
		if (emails != null && emails.size() > 0)
		{
		    Iterator iter = emails.iterator();
		    while( iter.hasNext() ) {
		    	Email em = ((Email)iter.next());
		    	em.setChangeDate(appLogDate);
		    	em.setSourceOfChange(this.getLookupService().getEmailSourceOfChangeByCode(EmailSourceOfChange.CODE_VAMC.getName()));
		    	em.setSiteOfChange(incoming.getPreferredFacility());
		    }
		}

		//since incoming voa records do not have an associations updated date, use the application log date
		//CCR 12536: Z05 requires Update Date on Associations in order to upload data to VistA
		Set associations = incoming.getAssociations();
		if (associations != null && associations.size() > 0)
		{

			Association assoc = null;
			for( Iterator it= associations.iterator();it.hasNext();)
			{
				assoc = (Association) it.next();
				assoc.setLastUpdateDate(appLogDate);
			}
		}

		//If the AO indicator is Yes then the AO Location is Vietnam.
		AgentOrangeExposure aoe = incoming.getAgentOrangeExposure();
	    if (aoe != null && aoe.getAgentOrangeExposureIndicator() != null &&
	    		Indicator.YES.getCode().equals(aoe.getAgentOrangeExposureIndicator().getCode()))
	    {
	    	aoe.setLocation(this.getLookupService().getAgentOrangeExposureLocationByCode(AgentOrangeExposureLocation.CODE_VIETNAM.getCode()));
	    }

		//This should be moved to ilog rule - processSpecialFactors
//		5.10.1	In Step [4083], if this is an incoming message from the Veteran's Online Application then accept the Radiation Exposure information.
//		5.10.1.2 Create an EE Case
//		Create EE Case (Type: Veteran's Online Application;  Reason: "Radiation Exposure Indicated")
//		5.10.1.4 Update Message Status - moved out of this method into DSLevel2
//		The status of the message is set to Form Processing Pending and the use case resumes at step [4083] for SW Asia Conditions.

//		5.13.	SUC2031.34 - Veteran's Online Application Appointment Request
//		In step [3846] if this is an incoming message from the Veteran's Online Application and the Appointment Request Date is True (Yes)
//		then set the date in ES to the current date.  If the Appointment Request Date is False (No)
//		set the date in ES to null and the use case resumes at step [4032].
		if (Boolean.TRUE.equals(incoming.getAppointmentRequestResponse()))
			incoming.setAppointmentRequestDate(appLogDate);


		//need to getInsurances to avoid null pointer exception if internalInsurances is null
		if (incoming.getInsurances() != null && incoming.getMedicareInsurances() != null && incoming.getMedicareInsurances().size() > 0)
		{
			//ES handles only two types of Medicare
			//1. Part A: one Medicare object with Part A true
			//2: Part A & B: two Medicare objects - one Part A as stated above and one with both Part A and Part B true.
			//please see gov.va.med.esr.ui.demographic.action.InsuranceConversionService.convertFormToPerson(InsuranceMedicareForm, Person)
			//front end only pass in one Medicare object
			Medicare mc = (Medicare)incoming.getMedicareInsurances().iterator().next();
			mc.setReportSite(VOA_VA_FACILITY);

			//mc.setReportSource(reportSource) //need this ???
			//CCR 13311 remove medicare Part B
/*			if (mc.getEnrolledInPartB() != null && mc.getEnrolledInPartB() )
			{
				//the Medicare object passed in has both Part A & Part B data

				//need Part A Medicare object with just Part A data
				Medicare mcA = (Medicare) mc.clone();
				//clean out Part B entries
				mcA.setEnrolledInPartB(Boolean.FALSE);
				mcA.setPartBEffectiveDate(null);

				incoming.addInsurance(mcA);

				//Part A and B: Part A & B both true with just Part B effective date, remove part A effective data
				mc.setPartAEffectiveDate(null);
			} */
		}

		//CCR 12086: Z04 requires insured relationship to pass consistency check, default to veteran
		if (incoming.getPrivateInsurances() != null && incoming.getPrivateInsurances().size() > 0)
		{
			//initilize INSURED_RELATIONSHIP_VETERAN if null
			if (INSURED_RELATIONSHIP_VETERAN == null)
				INSURED_RELATIONSHIP_VETERAN  = this.getLookupService().getInsuredRelationshipByCode(InsuredRelationship.CODE_VETERAN.getCode());

			PrivateInsurance ins = null;
			for( Iterator it= incoming.getPrivateInsurances().iterator();it.hasNext();)
			{
				ins = (PrivateInsurance) it.next();
				InsuranceSubscriber sub= ins.getSubscriber() == null ? new InsuranceSubscriber() : ins.getSubscriber();
				sub.setRelationship(INSURED_RELATIONSHIP_VETERAN);
			}
		}

		//CCR12027 - set the default Date Medicaid Last Updated to the application received date.
		if (incoming.getMedicaidFactor() != null)
			incoming.getMedicaidFactor().setLastUpdateDate(appLogDate);


		Integer prevCalendarYear = new Integer(Calendar.getInstance().get(Calendar.YEAR)-1);
		IncomeTest it = incoming.getIncomeTest(prevCalendarYear);
		if (it != null)
		{
			//CCR 12071: set send for adjudication default false to avoid AdjudicationResponseRequiredException and
			//set the VOA MT status directly without prompt user for send case to adjudication Y/N
			it.setSendForAdjudication(Boolean.FALSE);

			//CCR12075 - VOA does not send the data, but VOA forms cannot be submitted unless veterans "CONSENT TO COPAYS"
			//so set default agree to pay deductible = true
			it.setAgreesToPayDeductible(Boolean.TRUE);
		}

		//CCR12344: per Ray's email on 5/23/2013:
		//The new requirement for Married Last Calendar Year:
		//5.12.5.4.1	 Set Married Last Calendar Year
		//If on the incoming web service message, the veterans marital status is Married, Separated,  Divorced,  or Widowed
		//and the date of marriage is prior to the current calendar year, then set the Married Last Calendar Year to Yes, otherwise set to No.
		MaritalStatus status = incoming.getMaritalStatus();
		FinancialStatement stmt = incoming.getFinancialStatement(prevCalendarYear);
		Boolean marritedLastYear = false; //default
		if (status != null &&
				(MaritalStatus.CODE_MARRIED.getCode().equals(status.getCode()) || (MaritalStatus.CODE_SEPARATED.getCode().equals(status.getCode()))||
						MaritalStatus.CODE_DIVORCED.getCode().equals(status.getCode()) || (MaritalStatus.CODE_WIDOWED.getCode().equals(status.getCode()))) &&
				stmt != null && stmt.getSpouseFinancials() != null && stmt.getSpouseFinancials().size() != 0 )
		{
			for (Iterator iter = stmt.getSpouseFinancials().iterator(); iter.hasNext();)
			{
				SpouseFinancials spf = (SpouseFinancials)iter.next();
				if (spf.getReportedOn() != null &&
						spf.getReportedOn().getMarriageDate() != null &&
						spf.getReportedOn().getMarriageDate().compareTo(new ImpreciseDate("1/1/" + prevCalendarYear)) <0)
				{
					//married before last calendar year
					marritedLastYear = true;
					break;
				}
			}
		}
		if (stmt != null)
			stmt.setMarriedLastCalendarYear(marritedLastYear);

    }

	@SuppressWarnings("rawtypes")
	public void resubmit(List applicationIds) throws ServiceException {

		// Resubmit VOA Applications
		List voaApps = find(applicationIds);
		for (Iterator i = voaApps.iterator(); i.hasNext();) {
			VOAApplication voaApp = (VOAApplication) i.next();
			resubmitVoaApplication(voaApp);
			}
	}

	/**
	 * Find VOA Applications by ids
	 * TODO optimize to get the cases in bulk query
	 * @param applicationIds
	 * @return
	 * @throws ServiceException
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public List find(List applicationIds) throws ServiceException {
		List voaApps = new ArrayList();
		try {
			for (Iterator i = applicationIds.iterator(); i.hasNext();) {
	            BigDecimal appId = (BigDecimal) i.next();
	            VOAApplication voaApp = getVoaApplicationDAO().getByApplicationId(appId);
					if (voaApp != null) {
						voaApps.add(voaApp);
					}
				}
		}
		catch (DAOException e) {
			throw new ServiceException("VOA Application retrieval Failed ", e);
		}
		return voaApps;
	}

    /**
     * Get a list of VOAApplications that are available for resubmission
     * @return List of VOAApplications
     * @throws ServiceException
     */
    @SuppressWarnings("rawtypes")
	public List getResubmissionVOAApplications() throws ServiceException {

    	List lst = null;
    	try {
    		lst= this.getVoaApplicationDAO().getVOAResubmissionApplications();

    	} catch (DAOException dex)
    	{
    		throw new ServiceException ("Error getting Resubmission VOA form list", dex);
    	}
    	return lst;
    }

    /**
     * Resubmit a VOAApplication
     * @param app
     * @throws ServiceException
     */
    public void resubmitVoaApplication(VOAApplication voaApp) throws ServiceException
    {
    	Validate.notNull(voaApp, "Resubmission VOAApplication must not be null");
    	Validate.notNull(voaApp.getEntityKey(), "Resubmission VOAApplication id must not be null");
    	Validate.notNull(voaApp.getVOAResubmissionData(), "Resubmission VOAApplication Data must not be null");
    	Validate.notNull(voaApp.getEntityKey(), "Resubmission VOAApplication id must not be null");

    	if (voaApp.getVOAResubmissionData() == null || voaApp.getVOAResubmissionData().getApplicationData() == null)
    		throw new ServiceException("Resubmit VoaApplication failed with empty resubmission data. Submission id = " + voaApp.getEntityKey().getKeyValueAsString());


    	BigDecimal appId = (BigDecimal) voaApp.getEntityKey().getKeyValue();
    	Person person = null;
    	try {
    		person = deserializeXML(voaApp.getVOAResubmissionData().getApplicationData());
    	} catch (SQLException se)
    	{
    		throw new ServiceException("Resubmit VoaApplication failed: can not serialize person object. Submission id = " + voaApp.getEntityKey().getKeyValueAsString(), se);
    	}

    	//update the status of resubmission and null out the serialized data
		voaApp.setApplicationStatus((VOAApplicationStatus) this
				.getLookupService().getByCode(VOAApplicationStatus.class,
						VOAApplicationStatus.FORM_RESUBMITTED));
    	voaApp.getVOAResubmissionData().setApplicationData(null);

		try {
			this.getVoaApplicationDAO().update(voaApp);
			//must flush hibernate to avoid HibernateOptimisticLockingFailureException
			//because the next transaction forwardVOAApplication() is ASYNC and starts a new txn
			this.getVoaApplicationDAO().flush();
		} catch (DAOException ex)
		{
			throw new ServiceException("Update VOA Resubmition failed: ", ex);
		}

    	//start process the voa application ASYNC because ESR UI can submit multiple resubmissions and needs
		//a non-block call to go back to UI quickly
    	this.forwardVOAApplication(appId, person);

    }

	/**
	 * Common search method for retrieving the pending Submissions
	 */
	@SuppressWarnings("rawtypes")
	public List search(SearchQueryInfo searchQueryInfo) throws ServiceException {
			return searchPendingSubmissions(searchQueryInfo);
	}

	@SuppressWarnings({ "rawtypes", "unused" })
	public List searchPendingSubmissions(SearchQueryInfo searchQueryInfo) throws ServiceException {

		List pendingSubmissions = null;
		try {
			VOASearchCriteria searchCriteria = (VOASearchCriteria) searchQueryInfo;
			List applicationIds = null;


			// update search criteria to include ApplicationIds
			if (applicationIds != null && applicationIds.size() > 0)
			{
				searchCriteria.setApplicationIds(applicationIds);
			}

			// Pending Submission Search Results
            pendingSubmissions = getVoaApplicationDAO().find(searchCriteria);

            for ( int i=0; pendingSubmissions!= null && i< pendingSubmissions.size(); i++)
            {
            	VOASearchResultBean voaBean = (VOASearchResultBean) pendingSubmissions.get(i);
             	BigDecimal applicationId = voaBean.getApplicationId();
            }

		}
		catch (DAOException e) {
			throw new ServiceException("Pending Submissions Search Failed", e);
		}
		return pendingSubmissions;
	}

	/*private void processMedicaid(Person incoming) throws ServiceException {
		//Cr 13953 // WI 193006
		//open work item if VOA applicant claims in reciept of medicaid

		//later reversed with CR/WI 309168
		//removed references to this method in this class

		if (incoming != null && incoming.getMedicaidFactor() != null && incoming.getMedicaidFactor().getEligibleForMedicaid()) {

			Person onfile = this.getPersonService().getPersonWithoutIdentityTraits(incoming.getVPIDEntityKey());
			//if onfile has no medicaid, do not add from voa, set to null and open work item
			if ((onfile==null) || (onfile != null && (onfile.getMedicaidFactor() == null || onfile.getMedicaidFactor().getEligibleForMedicaid() == false))) {
				incoming.setMedicaidFactor(null);
				this.getWorkflowService().autoCreateCase(
						incoming,
						getLookupService().getFunctionalGroupByCode(FunctionalGroup.EE.getCode()),
						getLookupService().getWkfCaseTypeByCode(WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION.getCode()),
						null,
						"Medicaid is indicated");
			}
		}
	}*/

    private void processRadiationExposure(Person incoming) throws ServiceException
    {
		//This should be moved to ilog rule - processSpecialFactors
//		5.10.1	In Step [4083], if this is an incoming message from the Veteran's Online Application then accept the Radiation Exposure information.
//		5.10.1.2 Create an EE Case
//		Create EE Case (Type: Veteran's Online Application;  Reason: "Radiation Exposure Indicated")
//		5.10.1.4 Update Message Status - moved from setVOADefaults method because record is not persisted there yet.
//		The status of the message is set to Form Processing Pending and the use case resumes at step [4083] for SW Asia Conditions.
        RadiationExposure rexp = incoming.getRadiationExposure();
        if (rexp != null && rexp.getRadiationExposureIndicator() != null &&
        		Indicator.YES.getCode().equals(rexp.getRadiationExposureIndicator().getCode()))
        {
			this.getWorkflowService().autoCreateCase(
					incoming,
					getLookupService().getFunctionalGroupByCode(FunctionalGroup.EE.getCode()),
					getLookupService().getWkfCaseTypeByCode(WkfCaseType.CODE_VETERANS_ONLINE_APPLICATION.getCode()),
					null,
					"Radiation Exposure Indicated");
        }
    }


    //serialize Person object to xml
	private String serializeXML(Person p)throws SQLException {
		if (p == null)
			return null;

		return new XStream().toXML(p);
	}

	//deserialize xml to Person object
	private Person deserializeXML(String xml) throws SQLException {

		if (xml == null)
			return null;

		return (Person)new XStream().fromXML(xml);
	}
	public Date stringToDate(String string, String dateFormatStr){
		Date date = null;
		try{
			SimpleDateFormat dateFormat = new SimpleDateFormat(dateFormatStr);
		    date = dateFormat.parse(string);

		}catch (Exception ex){

			logger.error("Exception @stringToDate  :", ex);
		}
		return  date;
	}
}