/********************************************************************
 * Copyright  2013 VHA. All rights reserved
 ********************************************************************/

package gov.va.med.fw.hl7.segment;

// Java classes
import java.util.List;

// Framework classes
import gov.va.med.fw.hl7.InvalidSegmentException;
import gov.va.med.fw.hl7.Segment;
import gov.va.med.fw.hl7.constants.SegmentConstants;

/**
 * A ZHP segment containing information about health Benefit plans
 * CCR12064 
 * @author DNS   zhangs
 * @version 1.0
 */

public class ZHP extends Segment {

    /**
	 * An instance of serialVersionUID
	 */
	private static final long serialVersionUID = 3332144149113231000L;

	/**
     * Constructs an empty segment. This constructor is ususally usd by derived
     * segment classes to populate its own segment data.
     */
    public ZHP() {
        super();
        setName(SegmentConstants.ZHP);
    }

    /**
     * Constructs a segment of elements from a raw data string. This constructor
     * is usually used to construct an inbound message from a raw message's
     * segment data.
     * 
     * @param data
     *            A raw data string
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public ZHP(String data) throws InvalidSegmentException {
        super(data);
    }

    /**
     * Constructs a segment of elements from a raw data string, an array of
     * valid encoding character strings, and a element delimiter string. This
     * constructor is usually used to construct an inbound message from a raw
     * message's segment data, a specific encoding characters, and a field
     * delimiter.
     * 
     * @param encodingChars
     *            An array of encoding characters
     * @param elementDelimiter
     *            A field delimiter
     * @param segmentData
     *            A raw data string
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public ZHP(String[] encodingChars, String elementDelimiter,
            String segmentData) throws InvalidSegmentException {
        super(encodingChars, elementDelimiter, segmentData);
    }

    /**
     * Constructs a segment from a segment name and a list of elements. This
     * constructor is usually used to construct an outbound message from a list
     * of specific segment elements, and a segment name.
     * 
     * @param name
     *            A segment name
     * @param elements
     *            A list of segment elements
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public ZHP(String name, List elements) throws InvalidSegmentException {
        super(name, elements);
    }

    /**
     * Constructs a segment from an array of encoding characters, a field
     * delimiter, a segment name, and a list of field data. This constructor is
     * usually used to construct an outbound message from a list of specific
     * segment elements, element delimiter, and encoding characters.
     * 
     * @param encodingChars
     *            An array of encoding characters
     * @param elementDelimiter
     *            A field delimiter
     * @param segmentName
     *            A segment name
     * @param segmentElements
     *            A list of field data
     * @throws InvalidSegmentException
     *             Thrown if failed to construct a segment due to missing
     *             parameters or invalid segment data to parse.
     */
    public ZHP(String[] encodingChars, String elementDelimiter,
            String segmentName, List segmentElements)
            throws InvalidSegmentException {
        super(encodingChars, elementDelimiter, segmentName, segmentElements);
    }

    /**
     * 
     * SEQUENTIAL NUMBER
     * 
     * @param input
     *            A value of SetID
     */
    public void setSetID(String input) {
        setElement(input, 0);
    }

    /**
     * 
     * SEQUENTIAL NUMBER
     * 
     * @return String A value of SetID
     */
    public String getSetID() {
        return getElement(0);
    }

    /**
     * 
     * Health Benefit Plan Code  (VISTA Messaging HL7)
     * 
     * @param input
     *            A value of Health Benefit Plan Code
     */
    public void setHBPCode(String input) {
        setElement(input, 1);
    }

    /**
     * 
     * Health Benefit Plan Code  (VISTA Messaging HL7)
     * 
     * @return String A value of Health Benefit Plan Code
     */
    public String getHBPCode() {
        return getElement(1);
    }
    /**
     * 
     * Health Benefit Plan Last Updated DATE  (VISTA Messaging HL7)
     * 
     * @param input
     *            A value of Health Benefit Plan Last Updated DATE 
     */
    public void setHBPLastUpdatedDate(String input) {
        setElement(input, 2);
    }

    /**
     * 
     * Health Benefit Plan Last Updated DATE (Precise) (VISTA Messaging HL7)
     * 
     * @return String A value of ReviewDate
     */
    public String getHBPLastUpdatedDate() {
        return getElement(2);
    }

    
    /**
     * 
     * Health Benefit Plan Last Updated BY Source(VISTA Messaging HL7)
     * 
     * @param input
     *            A value of UpdatedBySource
     */
    public void setHBPLastUpdatedBySource(String input) {
        setElement(input, 3);
    }

    /**
     * 
     * Health Benefit Plan Last Updated BY Source(VISTA Messaging HL7)
     * 
     * @return String A value of UpdatedBySource
     */
    public String getHBPLastUpdatedBySource() {
        return getElement(3);
    }
    
    /**
     * 
     * Health Benefit Plan Last Updated BY Site(VISTA Messaging HL7)
     * 
     * @param input
     *            A value of UpdatedBySite
     */
    public void setHBPLastUpdatedBySite(String input) {
        setElement(input, 4);
    }

    /**
     * 
     * Health Benefit Plan Last Updated BY Site(VISTA Messaging HL7)
     * 
     * @return String A value of UpdatedBySite
     */
    public String getHBPLastUpdatedBySite() {
        return getElement(4);
    }


    
}