package gov.va.med.esr.ui.util;

import gov.va.med.esr.ui.common.util.DateUtils;
import gov.va.med.esr.ui.common.util.JspUtils;
import gov.va.med.esr.ui.militaryservice.action.MilitaryServiceActionHelper;
import gov.va.med.esr.common.infra.ImpreciseDate;
import gov.va.med.esr.common.infra.ImpreciseDateUtils;
import gov.va.med.esr.common.model.SiteYear;
import gov.va.med.esr.common.model.financials.DependentFinancials;
import gov.va.med.esr.common.model.financials.FinancialStatement;
import gov.va.med.esr.common.model.financials.IncomeTest;
import gov.va.med.esr.common.model.financials.SpouseFinancials;
import gov.va.med.esr.common.model.financials.PatientVisitSummary;
import gov.va.med.esr.common.model.insurance.Medicare;
import gov.va.med.esr.common.model.lookup.Country;
import gov.va.med.esr.common.model.lookup.Indicator;
import gov.va.med.esr.common.model.lookup.NameType;
import gov.va.med.esr.common.model.lookup.PhoneType;
import gov.va.med.esr.common.model.lookup.AssociationType;
import gov.va.med.esr.common.model.lookup.VAFacility;
import gov.va.med.esr.common.model.lookup.MaritalStatus;
import gov.va.med.esr.common.model.lookup.DecorationStatus;
import gov.va.med.esr.common.model.lookup.Relationship;
import gov.va.med.esr.common.model.lookup.RaceType;
import gov.va.med.esr.common.model.lookup.EthnicityType;
import gov.va.med.esr.common.model.lookup.Religion;
import gov.va.med.esr.common.model.lookup.EmploymentStatus;
import gov.va.med.esr.common.model.lookup.BenefitType;
import gov.va.med.esr.common.model.lookup.Gender;
import gov.va.med.esr.common.model.lookup.AgentOrangeExposureLocation;
import gov.va.med.esr.common.model.insurance.InsurancePolicy;
import gov.va.med.esr.common.model.insurance.PrivateInsurance;
import gov.va.med.esr.common.model.ee.AgentOrangeExposure;
import gov.va.med.esr.common.model.ee.EnvironmentalContaminationExposure;
import gov.va.med.esr.common.model.ee.MedicaidFactor;
import gov.va.med.esr.common.model.ee.MilitaryService;
import gov.va.med.esr.common.model.ee.CombatEpisode;
import gov.va.med.esr.common.model.ee.MilitaryServiceEpisode;
import gov.va.med.esr.common.model.ee.MilitaryServiceSiteRecord;
import gov.va.med.esr.common.model.ee.NoseThroatRadium;
import gov.va.med.esr.common.model.ee.PurpleHeart;
import gov.va.med.esr.common.model.ee.RadiationExposure;
import gov.va.med.esr.common.model.ee.ServiceConnectionAward;
import gov.va.med.esr.common.model.ee.PrisonerOfWar;
import gov.va.med.esr.common.model.ee.Application;

import gov.va.med.esr.common.model.party.Address;
import gov.va.med.esr.common.model.party.Email;
import gov.va.med.esr.common.model.party.Phone;
import gov.va.med.esr.common.model.person.Association;
import gov.va.med.esr.common.model.person.Name;
import gov.va.med.esr.common.model.person.Person;
import gov.va.med.esr.common.model.person.SSN;
import gov.va.med.esr.common.model.person.Spouse;
import gov.va.med.esr.common.model.person.Dependent;
import gov.va.med.esr.common.model.person.Race;
import gov.va.med.esr.common.model.person.Ethnicity;
import gov.va.med.esr.common.model.person.Employment;
import gov.va.med.esr.common.model.person.BirthRecord;
import gov.va.med.esr.common.model.person.id.VPIDEntityKey;
import gov.va.med.esr.service.PersonIdentityTraits;
import gov.va.med.fw.util.StringUtils;
import gov.va.med.person.idmgmt.VPID;
import gov.va.med.esr.common.infra.TriState;

import gov.va.med.esr.ui.financials.action.FinancialsConstants;

/* iText-2.1.4.jar */
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfImportedPage;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.AcroFields;
import com.lowagie.text.pdf.PdfStamper;
import com.lowagie.text.Paragraph;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.Element;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.Phrase;
import com.lowagie.text.Document;
import com.lowagie.text.PageSize;
import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.DocumentException;
import java.text.SimpleDateFormat;
import java.io.DataOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;

import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Map;
import java.util.Set;
import java.util.HashMap;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Date;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import java.awt.Color;

import javax.servlet.http.HttpServletResponse;

/**
 * @author Wen Lin - Financials1010EZApplication is used by financial overview page to
 *         generate the 1010EZ form in PDF format for printing. This class fills
 *         in the values in a fillable 1010EZ PDF form. If the application
 *         contains additional information (as indicated on the form by
 *         specifying that the applicant use a separate sheet for additional
 *         information), an additional PDF document is also created and appended
 *         to the 1010EZ form.
 */

public class Financials1010EZApplication {
    private static final Logger log = Logger.getLogger(Financials1010EZApplication.class);
   //  private static final String fillable1010EZFileName = "C:\\temp\\Fillable1010EZHealthcareApplication.pdf";
   //  private static final String fillable1010EZFileName = "Fillable1010EZHealthcareApplication.pdf";
    
    private HashMap fieldMap = new HashMap();
    private Person person = null;
    private MilitaryService militaryService = null;
    private Set insurances;
    private Set privateinsurances;
    private Set medicares;
    private Set dependentFinancials;
    private SpouseFinancials sf = null;
    private FinancialStatement ftmt = null;
    private Integer incomeYear;
    private boolean FinDisclosureFlag = false;
    private  Spouse spouse = null;
    
    private List patientVisitSummaries = null;
    private AcroFields fields;
    private PdfReader reader;
    private PdfStamper stamper;
    private Document additionalInsDocument;
    private Document additionalChildDocument;
//    private Document additionalChildFinDocument;
    private boolean isAdditionalDocumentInsNeeded = false;
    private boolean isAdditionalDocumentChildNeeded = false;
    private Font font_helve ;
    private Font font_courier ;
    private Font font_times ;
    private String formFileName;
    private String additionalInsFileName;  
    private String additionalChildFileName;
//    private String additionalChildFinFileName;
    private String finalFileName;
    private String filePath;
    private String templateLocation;
    private InputStream fillablePDF;
    private FileOutputStream outStaticForm;
    private FileOutputStream outAdditionalInsForm;
    private FileOutputStream outAdditionalChildForm;
//    private FileOutputStream outAdditionalChildFinForm;
    private FileOutputStream outCombinedForm;
//    private HttpServletResponse response;
    private String veteranName = "";
    private String veteranSSN = "";
    /**
     * Constructor
     * 
     * @param person
     *            Person object
     * @parm incomeYear
     * 			  Integer of the income year selected
     * @param templatepath
     *            The location where the template file was saved
     */
   // public Financials1010EZApplication(Person person, Integer incomeYear, String templatePath, String outputPath) {  
   public Financials1010EZApplication(Person person, Integer incomeYear, InputStream is, String outputPath) {  	
    	this.person = person;
    	this.incomeYear = incomeYear;
    	// this.templateLocation = templatePath;
    	this.filePath = outputPath;
    	this.fillablePDF = is;	
    	
        initializeFieldMap();
        String personId = person.getEntityKey().getKeyValueAsString();
        setOutputFiles(personId);

        this.militaryService = person.getMilitaryService();
        this.privateinsurances = person.getPrivateInsurances();
        this.ftmt = person.getFinancialStatement(incomeYear);
        if (this.ftmt == null) {
            this.ftmt = new FinancialStatement();
        }
        this.sf = ftmt.getRecentSpouseFinancials();
        this.dependentFinancials = ftmt.getDependentFinancials();
       
        if ( privateinsurances != null && privateinsurances.size() > 1 ) 
            this.initializeAdditionalInsPDF();
        
        if ( dependentFinancials != null && dependentFinancials.size() > 1 )
        	this.initializeAdditionalChildPDF();
        
        initializePDF();
        
        this.font_helve  = FontFactory.getFont(FontFactory.HELVETICA_BOLD, 8);
        this.font_courier = FontFactory.getFont(FontFactory.COURIER, 8);
        this.font_times = FontFactory.getFont(FontFactory.TIMES_ROMAN, 8);
    }

    private void setOutputFiles(String vetId){
    	String currDate = new SimpleDateFormat("yyyyMMddHHmmss").format(Calendar.getInstance().getTime());
    	currDate = currDate==null?"":currDate.trim();
    	// String curDate = new SimpleDateFormat("yyyyMMddHHmmss").format(DateUtils.getCurrentDateTime());
    	String fSeparater = FinancialsConstants.PDF_FILE_SEPARATER;
    	String fForm = FinancialsConstants.PDF_FILL_FORM;
    	String fInsurance = FinancialsConstants.PDF_ADDITIONAL_INSURANCE;
    	String fChild = FinancialsConstants.PDF_ADDITIONAL_DEPENDENT;
    	String fFinal = FinancialsConstants.PDF_FINAL_OUTPUT;
    	String fExt = FinancialsConstants.PDF_EXTENSION;
    	
    	// this.filePath = FinancialsConstants.PDF_OUTPUT_FILE_PATH;
    	this.formFileName = vetId+fSeparater+fForm+fSeparater+currDate+fExt;
    	this.additionalInsFileName = vetId+fSeparater+fInsurance+fSeparater+currDate+fExt;
    	this.additionalChildFileName = vetId+fSeparater+fChild+fSeparater+currDate+fExt;
    	this.finalFileName = vetId+fSeparater+fFinal+fSeparater+currDate+fExt;
    	// this.formFileName = "1010EZ_Form1.pdf";
    	// this.additionalInsFileName = "1010EZ_Additional_InsForm2.pdf";
    	// this.additionalChildFileName = "1010EZ_Additional_ChildForm2.pdf";
    	// this.finalFileName = "1010EZ_Final_Complete_Form.pdf";	
    }
    
    private String getFormFileName(){
    	return this.formFileName;
    }
    private String getAdditionalInsuranceFileName(){
    	return this.additionalInsFileName;
    }
    private String getAdditionalDependentFileName(){
    	return this.additionalChildFileName;
    }
    private String getFinalFileName(){
    	return this.finalFileName;
    }
    private String getOutputFilePath(){
    	return this.filePath;
    }
    
    /**
     * Initialize the PDF
     * 
     */
    private void initializePDF() {
        try {
        	// String templateFile = this.templateLocation!=null && !this.templateLocation.endsWith("/")? this.templateLocation+"/WEB-INF/"+fillable1010EZFileName:this.templateLocation+"WEB-INF/"+fillable1010EZFileName ;
           // this.fillablePDF = readTemplate(templateFile);
            reader = new PdfReader(this.fillablePDF);
            if (isAdditionalDocumentInsNeeded || isAdditionalDocumentChildNeeded) {
                outStaticForm = new FileOutputStream(this.getOutputFilePath() + this.getFormFileName());
                stamper = new PdfStamper(reader, outStaticForm, '\0');
            } else {
                outCombinedForm = new FileOutputStream(this.getOutputFilePath() + this.getFinalFileName());
                stamper = new PdfStamper(reader, outCombinedForm, '\0');
            }
            fields = stamper.getAcroFields();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void initializeAdditionalInsPDF() {
        isAdditionalDocumentInsNeeded = true;
        try {
            additionalInsDocument = new Document(PageSize.A4, 25, 25, 50, 50);
            outAdditionalInsForm = new FileOutputStream(this.getOutputFilePath()+ this.getAdditionalInsuranceFileName());
            PdfWriter.getInstance(additionalInsDocument, outAdditionalInsForm);
            additionalInsDocument.open();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void initializeAdditionalChildPDF() {
        isAdditionalDocumentChildNeeded = true;
        try {
            additionalChildDocument = new Document(PageSize.A4, 25, 25, 50, 50);
            outAdditionalChildForm = new FileOutputStream(this.getOutputFilePath()+ this.getAdditionalDependentFileName());
            PdfWriter.getInstance(additionalChildDocument, outAdditionalChildForm);
            additionalChildDocument.open();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Fill each section of the fillable PDF, and create additional files if
     * necessary. If additional files are necessary, the two PDFs are
     * concatenated.
     * 
     */
    public void get1010EZ() throws Exception {
        try {

            // Set section 1 - General Information
            fillSection1();

            // Set section 2 - Insurance Information
            fillSection2();

            // Section 3 - Employment Information
            fillSection3();
            
             // Section 4 - Military Service Information 
            fillSection4();
            
            // Section 5 - Financial Disclosure 
            fillSection5();
          
             // Section 7 - Spouse/Dependent Financial Information
             fillSection7();
             
             // Section 8 - Previous Calendar Year Gross Annual Income of Veteran, Spouse, and Dependent Children 
             // Section 9 - Previous Calendar Year Deductible Expenses
             // Section 10 - Previous Calendar Year Net Worth
             // fill section 8, 9 and 10 
             fillSection8_9_10();

            // Section 12 - Assignment of Benefits
            fillSection12();

            stamper.setFormFlattening(true);
            reader.close();
            stamper.close();
            
            if (isAdditionalDocumentInsNeeded) {
                additionalInsDocument.close();
                outAdditionalInsForm.close();
            }
            if (isAdditionalDocumentChildNeeded){
            	additionalChildDocument.close();
                outAdditionalChildForm.close();
            }
            if (isAdditionalDocumentInsNeeded || isAdditionalDocumentChildNeeded)
            	outStaticForm.close();
            else 
            	outCombinedForm.close();
            
        } catch (Exception e) {
            e.printStackTrace();
            reader.close();
            try {
            	if (isAdditionalDocumentInsNeeded) {
                    additionalInsDocument.close();
                    outAdditionalInsForm.close();
                }
                if (isAdditionalDocumentChildNeeded){
                	additionalChildDocument.close();
                    outAdditionalChildForm.close();
                }
                if (isAdditionalDocumentInsNeeded || isAdditionalDocumentChildNeeded)
                	outStaticForm.close();
                else 
                	outCombinedForm.close();
                stamper.close();
            } catch (Exception e2) {
                e.printStackTrace();
            }
        } finally {
            reader.close();
            try {
            	if (isAdditionalDocumentInsNeeded) {
                    additionalInsDocument.close();
                    outAdditionalInsForm.close();
                }
                if (isAdditionalDocumentChildNeeded){
                	additionalChildDocument.close();
                    outAdditionalChildForm.close();
                }
                if (isAdditionalDocumentInsNeeded || isAdditionalDocumentChildNeeded)
                	outStaticForm.close();
                else 
                	outCombinedForm.close();
                stamper.close();

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        // Concatenate pdfs
        try {
        	 if (isAdditionalDocumentInsNeeded || isAdditionalDocumentChildNeeded) {
                log.debug("Concatenating the pdf documents in 1010EZ");
                List pdfs;
                OutputStream output;
                FileInputStream inStaticForm = null;
                FileInputStream inAdditionalInsForm = null;
                FileInputStream inAdditionalChildForm = null;
                pdfs = new ArrayList();
                File file1 = new File(this.getOutputFilePath() + this.getFormFileName());
                inStaticForm = new FileInputStream(file1);
                pdfs.add(inStaticForm);
                
                File file2 =null;
                File file3 = null;
                if (isAdditionalDocumentInsNeeded){
                	file2 = new File(this.getOutputFilePath()+ this.getAdditionalInsuranceFileName());
                	inAdditionalInsForm = new FileInputStream(file2);
                    pdfs.add(inAdditionalInsForm);
                }
                if (isAdditionalDocumentChildNeeded){
                	file3 = new File(this.getOutputFilePath()+ this.getAdditionalDependentFileName());
                    inAdditionalChildForm = new FileInputStream(file3);
                    pdfs.add(inAdditionalChildForm);
                }
                output = new FileOutputStream(this.getOutputFilePath() + this.getFinalFileName());
                concatPDFs(pdfs, output, true);
                output.close();
                inStaticForm.close();

                if (inAdditionalInsForm != null)
                	inAdditionalInsForm.close();
                if (inAdditionalChildForm!=null)
                	inAdditionalChildForm.close();
        
                if (file1.exists()) file1.delete();
                if (isAdditionalDocumentInsNeeded){
                	if (file2.exists()) file2.delete();
                }
                if (isAdditionalDocumentChildNeeded){
                	if (file3.exists()) file3.delete();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    /**
     * Initialize map containing all field names in the fillable PDF
     * 
     */
    private void initializeFieldMap() {
        fieldMap.put("Section1:Question1", "Text1");
        fieldMap.put("Section1:Question2", "Text2");
        fieldMap.put("Section1:Question3", "Text3");
        fieldMap.put("Section1:Question4:Male", "Check Box20");
        fieldMap.put("Section1:Question4:Female", "Check Box21");
        fieldMap.put("Section1:Question5:Yes", "Check Box22");
        fieldMap.put("Section1:Question5:No", "Check Box23");
        fieldMap.put("Section1:Question6:AlaskaNative", "Check Box24");
        fieldMap.put("Section1:Question6:Asian", "Check Box25");
        fieldMap.put("Section1:Question6:White", "Check Box26");
        fieldMap.put("Section1:Question6:Black", "Check Box27");
        fieldMap.put("Section1:Question6:NativeHawaiian", "Check Box28");
        fieldMap.put("Section1:Question7", "Text4");
        fieldMap.put("Section1:Question8", "Text7");
        fieldMap.put("Section1:Question9", "Text5");
        fieldMap.put("Section1:Question9A", "Text8");
        fieldMap.put("Section1:Question10", "Text6");
        fieldMap.put("Section1:Question11", "Text10");
        fieldMap.put("Section1:Question11A", "Text11");
        fieldMap.put("Section1:Question11B", "Text12");
        fieldMap.put("Section1:Question11C", "Text13");
        fieldMap.put("Section1:Question11D", "Text14");
        fieldMap.put("Section1:Question11E", "Text15");
        fieldMap.put("Section1:Question11F", "Text16");
        fieldMap.put("Section1:Question11G", "Text17");
        fieldMap.put("Section1:Question11H", "Text18");
        fieldMap.put("Section1:Question12:HealthServices", "Check Box29");
        fieldMap.put("Section1:Question12:NursingHome", "Check Box30");
        fieldMap.put("Section1:Question12:Domiciliary", "Check Box31");
        fieldMap.put("Section1:Question12:Dental", "Check Box32");
        fieldMap.put("Section1:Question13", "Text19");
        fieldMap.put("Section1:Question14:Yes", "Check Box33");
        fieldMap.put("Section1:Question14:No", "Check Box34");
        fieldMap.put("Section1:Question15:Yes", "Check Box35");
        fieldMap.put("Section1:Question15:No", "Check Box36");
        fieldMap.put("Section1:Question15:Location","Text22");
        fieldMap.put("Section1:Question16:Married", "Check Box37");
        fieldMap.put("Section1:Question16:NeverMarried", "Check Box38");
        fieldMap.put("Section1:Question16:Separated", "Check Box39");
        fieldMap.put("Section1:Question16:Widowed", "Check Box40");
        fieldMap.put("Section1:Question16:Divorced", "Check Box41");
        fieldMap.put("Section1:Question16:Unknown", "Check Box42");
        fieldMap.put("Section1:Question17", "Text45");
        fieldMap.put("Section1:Question17A", "Text46");
        fieldMap.put("Section1:Question17B", "Text47");
        fieldMap.put("Section1:Question18", "Text50");
        fieldMap.put("Section1:Question18A", "Text48");
        fieldMap.put("Section1:Question18B", "Text49");
        fieldMap.put("Section1:Question19:EmergencyContact", "Check Box43");
        fieldMap.put("Section1:Question19:NextOfKin", "Check Box44");

        // Section 2
        fieldMap.put("Section2:Question1:Yes", "Check Box61");
        fieldMap.put("Section2:Question1:No", "Check Box62");
        fieldMap.put("Section2:Question2", "Text53");
        fieldMap.put("Section2:Question3", "Text54");
        fieldMap.put("Section2:Question4", "Text55");
        fieldMap.put("Section2:Question5", "Text56");
        fieldMap.put("Section2:Question6:Yes", "Check Box63");
        fieldMap.put("Section2:Question6:No", "Check Box64");
        fieldMap.put("Section2:Question7:Yes", "Check Box65");
        fieldMap.put("Section2:Question7:No", "Check Box66");
        fieldMap.put("Section2:Question7A", "Text57");
        fieldMap.put("Section2:Question8:Yes", "Check Box68"); 
        fieldMap.put("Section2:Question8:No", "Check Box67"); 
        fieldMap.put("Section2:Question8A", "Text58");
        fieldMap.put("Section2:Question9", "Text60");
        fieldMap.put("Section2:Question10", "Text59");
        fieldMap.put("Section2:Question11:Yes", "Check Box69");
        fieldMap.put("Section2:Question11:No", "Check Box70");
        fieldMap.put("Section2:Question12:Yes", "Check Box71");
        fieldMap.put("Section2:Question12:No", "Check Box72");

        // Section 3
        fieldMap.put("Section3:Question1:FullTime", "Check Box73");
        fieldMap.put("Section3:Question1:NotEmployed", "Check Box74");
        fieldMap.put("Section3:Question1:Retired", "Check Box75");
        fieldMap.put("Section3:Question1:PartTime", "Check Box76");
        fieldMap.put("Section3:Question1:RetirementDate", "Text9");
        fieldMap.put("Section3:Question1A", "Text81");
        fieldMap.put("Section3:Question2:FullTime", "Check Box77");
        fieldMap.put("Section3:Question2:NotEmployed", "Check Box78");
        fieldMap.put("Section3:Question2:Retired", "Check Box80");
        fieldMap.put("Section3:Question2:PartTime", "Check Box79");
        fieldMap.put("Section3:Question2:RetirementDate", "Text20");
        fieldMap.put("Section3:Question2A", "Text82");

        // Section 4
        fieldMap.put("Section4:Question1", "Text83");
        fieldMap.put("Section4:Question1A", "Text84");
        fieldMap.put("Section4:Question1B", "Text85");
        fieldMap.put("Section4:Question1C", "Text86");
        fieldMap.put("Section4:Question1D", "Text87");
        fieldMap.put("Section4:Question2A:Yes", "Check Box88");
        fieldMap.put("Section4:Question2A:No", "Check Box97");
        fieldMap.put("Section4:Question2B:Yes", "Check Box89");
        fieldMap.put("Section4:Question2B:No", "Check Box96");
        fieldMap.put("Section4:Question2C:Yes", "Check Box90");
        fieldMap.put("Section4:Question2C:No", "Check Box110");
        fieldMap.put("Section4:Question2C1", "Text21");
        fieldMap.put("Section4:Question2D:Yes", "Check Box91");
        fieldMap.put("Section4:Question2D:No", "Check Box94");
        fieldMap.put("Section4:Question2E:Yes", "Check Box92");
        fieldMap.put("Section4:Question2E:No", "Check Box93");
        fieldMap.put("Section4:Question2E1:Yes", "Check Box98");
        fieldMap.put("Section4:Question2E1:No", "Check Box99");
        fieldMap.put("Section4:Question2F:Yes", "Check Box100");
        fieldMap.put("Section4:Question2F:No", "Check Box101");
        fieldMap.put("Section4:Question2G:Yes", "Check Box102");
        fieldMap.put("Section4:Question2G:No", "Check Box103");
        fieldMap.put("Section4:Question2H:Yes", "Check Box109");
        fieldMap.put("Section4:Question2H:No", "Check Box104");
        fieldMap.put("Section4:Question2I:Yes", "Check Box108");
        fieldMap.put("Section4:Question2I:No", "Check Box105");
        fieldMap.put("Section4:Question2J:Yes", "Check Box107");
        fieldMap.put("Section4:Question2J:No", "Check Box106");

        // Section 5
        fieldMap.put("Section5:No", "Check Box113");
        fieldMap.put("Section5:Yes", "Check Box114");

        // Section 7
        fieldMap.put("Section7:Question1", "Text115");
        fieldMap.put("Section7:Question1A", "Text117");
        fieldMap.put("Section7:Question1B", "Text118");
        fieldMap.put("Section7:Question1C", "Text123");
        fieldMap.put("Section7:Question1D", "Text122");
        fieldMap.put("Section7:Question1E", "Text124");
        fieldMap.put("Section7:Question2", "Text116");
        fieldMap.put("Section7:Question2A:Son", "Check Box45");
        fieldMap.put("Section7:Question2A:Daughter", "Check Box46");
        fieldMap.put("Section7:Question2A:Stepson", "Check Box47");
        fieldMap.put("Section7:Question2A:Stepdaughter", "Check Box48");
        fieldMap.put("Section7:Question2B", "Text119");
        fieldMap.put("Section7:Question2C", "Text120");
        fieldMap.put("Section7:Question2D", "Text121");
        fieldMap.put("Section7:Question2E:Yes", "Check Box49");
        fieldMap.put("Section7:Question2E:No", "Check Box50");
        fieldMap.put("Section7:Question2F:Yes", "Check Box51");
        fieldMap.put("Section7:Question2F:No", "Check Box52");
        fieldMap.put("Section7:Question2G", "Text61");
        fieldMap.put("Section7:Question3:Spouse", "Text125");
        fieldMap.put("Section7:Question3:Child", "Text126");
        fieldMap.put("Section7:Question3:Yes", "Check Box53");  //Did you provide support Yes
        

        // Section 8
        fieldMap.put("Section8:Question1:Veteran", "Text127");
        fieldMap.put("Section8:Question1:Spouse", "Text128");
        fieldMap.put("Section8:Question1:Child", "Text129");
        fieldMap.put("Section8:Question2:Veteran", "Text132");
        fieldMap.put("Section8:Question2:Spouse", "Text131");
        fieldMap.put("Section8:Question2:Child", "Text130");
        fieldMap.put("Section8:Question3:Veteran", "Text133");
        fieldMap.put("Section8:Question3:Spouse", "Text134");
        fieldMap.put("Section8:Question3:Child", "Text135");

        // Section 9

        fieldMap.put("Section9:Question1", "Text136");
        fieldMap.put("Section9:Question2", "Text137");
        fieldMap.put("Section9:Question3", "Text138");

        // Section 10
        fieldMap.put("Section10:Question1:Veteran", "Text139");
        fieldMap.put("Section10:Question1:Spouse", "Text140");
        fieldMap.put("Section10:Question1:Child", "Text141");
        fieldMap.put("Section10:Question2:Veteran", "Text144");
        fieldMap.put("Section10:Question2:Spouse", "Text143");
        fieldMap.put("Section10:Question2:Child", "Text142");
        fieldMap.put("Section10:Question3:Veteran", "Text145");
        fieldMap.put("Section10:Question3:Spouse", "Text146");
        fieldMap.put("Section10:Question3:Child", "Text147");

        // Section 12
        fieldMap.put("Section12:Date", "Text148");
    }
    
    /**
     * Concatenate fillable PDF document and additional PDF document
     * 
     * @param streamOfPDFFiles
     *            List of PDF documents as InputStream objects
     * @param outputStream
     *            The resulting OutputStream object
     * @param paginate
     *            Boolean indicating whether to paginate the resulting document
     */
    private void concatPDFs(List streamOfPDFFiles, OutputStream outputStream,
            boolean paginate) {
        Document document = new Document();
        try {
            List pdfs = streamOfPDFFiles;
            List readers = new ArrayList();
            int totalPages = 0;
            Iterator iteratorPDFs = pdfs.iterator(); // Create Readers for the pdfs.
            while (iteratorPDFs.hasNext()) {
                InputStream pdf = (InputStream) iteratorPDFs.next();
                PdfReader pdfReader = new PdfReader(pdf);
                readers.add(pdfReader);
                totalPages += pdfReader.getNumberOfPages();
            }
            // Create a writer for the outputstream
            PdfWriter writer = PdfWriter.getInstance(document, outputStream);
            document.open();
            // BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA,BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
            BaseFont bf = BaseFont.createFont(BaseFont.COURIER,BaseFont.CP1252, BaseFont.NOT_EMBEDDED);
            PdfContentByte cb = writer.getDirectContent();
            // Holds the PDF // data
            PdfImportedPage page;
            int currentPageNumber = 0;
            int pageOfCurrentReaderPDF = 0;
            Iterator iteratorPDFReader = readers.iterator();
            // Loop through the PDF files and add to the output.
            while (iteratorPDFReader.hasNext()) {
                PdfReader pdfReader = (PdfReader) iteratorPDFReader.next();
                // Create a new page in the target for each source page.
                while (pageOfCurrentReaderPDF < pdfReader.getNumberOfPages()) {
                    document.newPage();
                    pageOfCurrentReaderPDF++;
                    currentPageNumber++;
                    page = writer.getImportedPage(pdfReader,pageOfCurrentReaderPDF);
                    cb.addTemplate(page, 0, 0);
                    // Code for pagination.
                    if (paginate) {
                        cb.beginText();
                        cb.setFontAndSize(bf, 10);
                        cb.showTextAligned(PdfContentByte.ALIGN_LEFT, ""+ currentPageNumber + " of " + totalPages, 520,5, 0);
                        cb.endText();
                    }
                }
                pageOfCurrentReaderPDF = 0;
            }
            outputStream.flush();
            document.close();
            outputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (document.isOpen())
                document.close();
            try {
                if (outputStream != null)
                    outputStream.close();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    /**
     * Fill SECTION I - GENERAL INFORMATION of the fillable PDF document
     * @throws DocumentException
     * @throws IOException
     */
    private void fillSection1() throws DocumentException, IOException {

        Name legalName = null;
        Name aliasName = null;

        // get name from person
        Set names = person.getNames();                              
        aliasName = Name.getNameOfType(names, NameType.ALIAS_NAME.getName());
        legalName = Name.getNameOfType(names, NameType.LEGAL_NAME.getName());
        
        String p_legalname = this.getLastFirstMiddleByComma(legalName);
        String p_aliasname = aliasName != null ? aliasName.getFormattedName(): "";
        this.veteranName = p_legalname;
        
        fields.setField((String)fieldMap.get("Section1:Question1"),p_legalname);
        fields.setField((String)fieldMap.get("Section1:Question2"),p_aliasname);
        fields.setField((String)fieldMap.get("Section1:Question3"), person.getMothersMaidenName());

        if (person.getGender() != null) {
        	if (Gender.MALE.getCode().equalsIgnoreCase(person.getGender().getCode()))
                fields.setField((String)fieldMap.get("Section1:Question4:Male"), "Yes");
            else if (Gender.FEMALE.getCode().equalsIgnoreCase(person.getGender().getCode()))
                fields.setField((String)fieldMap.get("Section1:Question4:Female"), "Yes");
        }
        
        Ethnicity ethnicity = person.getEthnicity();
        if (ethnicity!=null && ethnicity.getEthnicityType()!=null ){
        	EthnicityType ethnicitytype = ethnicity.getEthnicityType();
        	if (ethnicitytype!=null){
        		if (EthnicityType.CODE_HISPANIC.getCode().equals(ethnicitytype.getCode()))
        			fields.setField((String)fieldMap.get("Section1:Question5:Yes"),"Yes");
        		else 
        			if ( EthnicityType.CODE_NOT_HISPANIC.getCode().equals(ethnicitytype.getCode()))
        				fields.setField((String)fieldMap.get("Section1:Question5:No"),"Yes");
        	}
        }
        
        Set raceset = person.getRaces();
        if (raceset!=null){
        	for (Iterator iterator = raceset.iterator(); iterator.hasNext();) {
        		Race race = (Race) iterator.next();
        		RaceType raceType= race.getRaceType();
        		if (raceType!=null){
        			if (RaceType.CODE_AMERICAN_INDIAN_ALASKA.getCode().equals(raceType.getCode()))
        				fields.setField((String)fieldMap.get("Section1:Question6:AlaskaNative"),"Yes"); 
        			else if (RaceType.CODE_ASIAN.getCode().equals(raceType.getCode())) 
        				fields.setField((String)fieldMap.get("Section1:Question6:Asian"),"Yes"); 
        			else if (RaceType.CODE_BLACK_AFRICAN.getCode().equals(raceType.getCode())) 
        				fields.setField((String)fieldMap.get("Section1:Question6:Black"),"Yes"); 
        			else if (RaceType.CODE_WHITE.getCode().equals(raceType.getCode())) 
        				fields.setField((String)fieldMap.get("Section1:Question6:White"),"Yes"); 
        			else if (RaceType.CODE_HAWAIIAN_PACIFIC.getCode().equals(raceType.getCode())) 
        				fields.setField((String)fieldMap.get("Section1:Question6:NativeHawaiian"),"Yes"); 
        		}
        	}
        }
        
       SSN officialSSN = person.getOfficialSsn();
       this.veteranSSN = officialSSN!=null? JspUtils.displaySSN(officialSSN.getSsnText()):"";
       
       fields.setField((String)fieldMap.get("Section1:Question7"),officialSSN!=null? JspUtils.displaySSN(officialSSN.getSsnText()):"");
       ImpreciseDate birthdate = null;
       if (person.getBirthRecord() != null)
            birthdate = person.getBirthRecord().getBirthDate();
       fields.setField((String)fieldMap.get("Section1:Question8"), person.getClaimFolderNumber());
       fields.setField((String)fieldMap.get("Section1:Question9"), DateUtils.format(birthdate, null));
       fields.setField((String)fieldMap.get("Section1:Question9A"),person.getBirthRecord()!=null ? this.getBirthPlace(person.getBirthRecord().getCity(),person.getBirthRecord().getState()):"");
       fields.setField((String)fieldMap.get("Section1:Question10"),person.getReligion()!=null ? person.getReligion().getName():"");

        // Get the permanent address
        Address permAddress = person.getPermanentAddress();
        if (permAddress != null) {
            String stateProvince = null;
            String zipPostalCode = null;
            if (Country.isUSAddress(permAddress.getCountry())) {
                stateProvince = permAddress.getState();
                zipPostalCode = permAddress.getZipCode();
                if ((StringUtils.isNotEmpty(zipPostalCode))
                        && (StringUtils.isNotEmpty(permAddress.getZipPlus4()))) {
                    zipPostalCode = zipPostalCode + "-"
                            + permAddress.getZipPlus4();
                }
            } else {
                stateProvince = permAddress.getProvince();
                zipPostalCode = permAddress.getPostalCode();
            }

            StringBuffer resultBuffer = new StringBuffer();
            // Append the first 3 lines
            permAddress.appendAddressPieceOnNewLine(resultBuffer, permAddress.getLine1());
            permAddress.appendAddressPieceOnNewLine(resultBuffer, permAddress.getLine2());
            permAddress.appendAddressPieceOnNewLine(resultBuffer, permAddress.getLine3());

            fields.setField((String)fieldMap.get("Section1:Question11"),resultBuffer.toString());
            fields.setField((String)fieldMap.get("Section1:Question11A"),permAddress.getCity());
            fields.setField((String)fieldMap.get("Section1:Question11B"),stateProvince);
            fields.setField((String)fieldMap.get("Section1:Question11C"),zipPostalCode);
            fields.setField((String)fieldMap.get("Section1:Question11D"),permAddress.getCounty());
        }

        Phone homephone = person.getHomePhone();
        if (homephone != null)
            fields.setField((String)fieldMap.get("Section1:Question11E"),homephone.getFormattedPhone());

        // Get the contact methods
        ArrayList emails = new ArrayList(person.getEmails());
        String cellphonenumber = "";
        String pagephonenumber = "";

        Phone cellphone = Phone.getPhoneOfType(person.getPhones(),PhoneType.CODE_MOBILE.getCode());
        if (cellphone != null)
            cellphonenumber = cellphone.getFormattedPhone();

        Phone pagephone = Phone.getPhoneOfType(person.getPhones(),PhoneType.CODE_PAGER.getCode());
        if (pagephone != null)
            pagephonenumber = pagephone.getFormattedPhone();

        String emailaddress = "";
        for (Iterator iterator = emails.iterator(); iterator.hasNext();) {
            Email vemail = (Email) iterator.next();
            emailaddress = vemail.getAddress();
            if (emailaddress != null && StringUtils.isNotEmpty(emailaddress))
                break;
        }

        fields.setField((String)fieldMap.get("Section1:Question11G"),cellphonenumber);
        fields.setField((String)fieldMap.get("Section1:Question11H"),pagephonenumber);
        fields.setField((String)fieldMap.get("Section1:Question11F"),emailaddress);
        fields.setField((String)fieldMap.get("Section1:Question13"), person.getMostRecentPreferredFacility() != null ? person.getMostRecentPreferredFacility().getFacilityName() : "");
     
        // question 12 - new VOA
        Application application = person.getApplication();
        if (application!=null){
        	BenefitType benefitType = application.getBenefitType();
        	if (benefitType!=null){
        		if (BenefitType.CODE_HEATH_SERVICES.getCode().equals(benefitType.getCode()) 
        				 || BenefitType.CODE_ENROLLMENT.getCode().equals(benefitType.getCode()) )
        			fields.setField((String)fieldMap.get("Section1:Question12:HealthServices"),"Yes");
        		else if (BenefitType.CODE_DENTAL.getCode().equals(benefitType.getCode()) )
        			fields.setField((String)fieldMap.get("Section1:Question12:Dental"),"Yes");
        		else if (BenefitType.CODE_DOMICILIARY.getCode().equals(benefitType.getCode()) )
        			fields.setField((String)fieldMap.get("Section1:Question12:Domiciliary"),"Yes");
        		else if (BenefitType.CODE_NURSING_HOME.getCode().equals(benefitType.getCode()) )
        			fields.setField((String)fieldMap.get("Section1:Question12:NursingHome"),"Yes");
        	}
        }
      
        // VOA Only check Question15:NO, otherwise Question15:Yes. 
        patientVisitSummaries = this.getPatientVisitSummaries();
        if (patientVisitSummaries != null && patientVisitSummaries.size() > 0) {
			//get the first record and return the date if used sort in getPatientVisitSummaries()
			// PatientVisitSummary patientVisitSummary = (PatientVisitSummary)patientVisitSummaries.get(0);
        	PatientVisitSummary patientVisitSummary = PatientVisitSummary.getLastestVisitedSummary(patientVisitSummaries);
        	fields.setField((String)fieldMap.get("Section1:Question15:Yes"),"Yes");
			fields.setField((String)fieldMap.get("Section1:Question15:Location"),patientVisitSummary!=null && patientVisitSummary.getFacilityVisited()!=null ? patientVisitSummary.getFacilityVisited().getName():""); ;
        }
        else
        	fields.setField((String)fieldMap.get("Section1:Question15:No"),"Yes");

        MaritalStatus mstatus = person.getMaritalStatus();
        if (mstatus != null) {
            if (mstatus.getName().equalsIgnoreCase("MARRIED")) {
                fields.setField((String)fieldMap.get("Section1:Question16:Married"), "Yes");
            } else if (mstatus.getName().equalsIgnoreCase("NEVER MARRIED")) {
                fields.setField((String)fieldMap.get("Section1:Question16:NeverMarried"), "Yes");
            } else if (mstatus.getName().equalsIgnoreCase("SEPARATED")) {
                fields.setField((String)fieldMap.get("Section1:Question16:Separated"), "Yes");
            } else if (mstatus.getName().toUpperCase().indexOf("WIDOW") >= 0) {
                fields.setField((String)fieldMap.get("Section1:Question16:Widowed"), "Yes");
            } else if (mstatus.getName().equalsIgnoreCase("UNKNOWN")) {
                fields.setField((String)fieldMap.get("Section1:Question16:Unknown"), "Yes");
            } else if (mstatus.getName().equalsIgnoreCase("DIVORCED")) {
                fields.setField((String)fieldMap.get("Section1:Question16:Divorced"), "Yes");
            }
        }
        List associations = new ArrayList(person.getAssociations());
        Collections.sort(associations);
        Association primary_next_of_kin = Association.getAssociationOfType(person.getAssociations(),AssociationType.CODE_PRIMARY_NEXT_OF_KIN.getCode());
        Association emergency_contact = Association.getAssociationOfType(person.getAssociations(), AssociationType.CODE_EMERGENCY_CONTACT.getCode());

        if (primary_next_of_kin != null) {
            Address p_address = primary_next_of_kin.getAddress();
            String a_addr = p_address!=null ? p_address.getFormattedAddress():"";
            String a_role = primary_next_of_kin.getRelationship();
            String a_name = primary_next_of_kin.getRepresentativeName() != null ? primary_next_of_kin.getRepresentativeName().getFormattedName(): "";
            String a_h_phone = Phone.getFormattedPhone(primary_next_of_kin.getPrimaryPhone()); // primary=home, alternate=work
            String a_w_phone = Phone.getFormattedPhone(primary_next_of_kin.getAlternatePhone());
            fields.setField((String)fieldMap.get("Section1:Question17"),a_name + "\n" + a_addr + "\n" + a_role);
            fields.setField((String)fieldMap.get("Section1:Question17A"),a_h_phone);
            fields.setField((String)fieldMap.get("Section1:Question17B"), a_w_phone);
        }
        
        if (emergency_contact != null) {
            Address e_address = emergency_contact.getAddress();
            String e_addr = e_address!=null ? e_address.getFormattedAddress():"";
            String e_role = emergency_contact.getRelationship();
            String e_name = emergency_contact.getRepresentativeName() != null ? emergency_contact.getRepresentativeName().getFormattedName(): "";
            String e_h_phone = Phone.getFormattedPhone(emergency_contact.getPrimaryPhone());
            String e_w_phone = Phone.getFormattedPhone(emergency_contact.getAlternatePhone());
            fields.setField((String) fieldMap.get("Section1:Question18"),e_name + "\n" + e_addr + "\n" + e_role);
            fields.setField((String) fieldMap.get("Section1:Question18A"),e_h_phone);
            fields.setField((String) fieldMap.get("Section1:Question18B"),e_w_phone);
         }
        
        /* VOA Question14 Appoinment left unchecked base on UC  */
        //CCR12088 -  VOA - Appointment Request Indicator not printing on 1010EZ
        if (person.getAppointmentRequestResponse() != null)
        {
        	if (person.getAppointmentRequestResponse())
        		fields.setField((String)fieldMap.get("Section1:Question14:Yes"),"Yes");
        	else
        		fields.setField((String)fieldMap.get("Section1:Question14:No"),"Yes");
        }
        	
        /* Question19 to left unchecked based on UC Specification */
    }

    /**
     * Fill SECTION II - INSURANCE INFORMATION of the fillable PDF document
     * 
     * @throws DocumentException
     * @throws IOException
     */

    private void fillSection2() throws DocumentException, IOException {

        insurances = person.getInsurances(); // set of insurancePolicy
        medicares = person.getMedicareInsurances();
        MedicaidFactor medicaidfactor = person.getMedicaidFactor();

        if (medicaidfactor != null )
            if ( Boolean.TRUE.equals(medicaidfactor.getEligibleForMedicaid()))
                fields.setField((String)fieldMap.get("Section2:Question6:Yes"),"Yes");
        else if ( Boolean.FALSE.equals(medicaidfactor.getEligibleForMedicaid()))
            fields.setField((String)fieldMap.get("Section2:Question6:No"),"Yes");

        if (insurances != null && insurances.size() != 0)
            fields.setField((String)fieldMap.get("Section2:Question1:Yes"),"Yes");
        else
            fields.setField((String)fieldMap.get("Section2:Question1:No"),"Yes");

        List sortpinsr = sortPrivateInsurance (privateinsurances); 
        int ins_cnt = 0;
        if (sortpinsr != null && sortpinsr.size() != 0) {
            boolean isFirst = true;
            boolean addFirst = true;
            for (Iterator iterator = sortpinsr.iterator(); iterator.hasNext();) {
                PrivateInsurance insurance = (PrivateInsurance) iterator.next();
                ins_cnt ++;
                if (isFirst) {
                    fields.setField((String)fieldMap.get("Section2:Question2"),getInsuranceCompanyAddrPhone(insurance));
                    fields.setField((String)fieldMap.get("Section2:Question3"),insurance.getSubscriber() != null ? insurance.getSubscriber().getName() : "");
                    fields.setField((String)fieldMap.get("Section2:Question4"),insurance.getPolicyNumber());
                    fields.setField((String)fieldMap.get("Section2:Question5"),insurance.getGroupNumber());
                    isFirst = false;
                } else {
                	PdfPTable table = new PdfPTable(3);
                	table.setKeepTogether(true);
                	table.setWidthPercentage(100f);
            		float[] widths = {45f,30f,25f};
            		table.setWidths(widths);
                	if (addFirst){
                		 PdfPCell cell = new PdfPCell(new Paragraph("APPLICATION FOR HEALTH BENEFITS, Continued",FontFactory.getFont(FontFactory.HELVETICA, 11,Font.BOLD, Color.BLACK)));
                		 cell.setHorizontalAlignment(Element.ALIGN_CENTER);
                		 table.addCell(cell);
                		 cell.setHorizontalAlignment(Element.ALIGN_LEFT);
                		 table.addCell(new Phrase("VETERAN'S NAME  (Last, First, Middle)\n\n"+veteranName, font_times));
                		 table.addCell( new Phrase("SOCIAL SECURITY NUMBER\n\n"+veteranSSN, font_times));
                		 addFirst = false;
                	}
                    PdfPCell cell = new PdfPCell(new Paragraph("SECTION II - INSURANCE INFORMATION (Continuation #"+ins_cnt+")",FontFactory.getFont(FontFactory.HELVETICA, 8,Font.BOLD, Color.BLACK)));
                    cell.setColspan(3);
                    cell.setHorizontalAlignment(Element.ALIGN_CENTER);
                    cell.setBackgroundColor(Color.lightGray);
                    table.addCell(cell);
                    
                    cell = new PdfPCell(new Paragraph("2.HEALTH INSURANCE COMPANY NAME,ADDRESS AND TELEPHONE NUMBER\n"+getInsuranceCompanyAddrPhone(insurance), font_courier));
                    cell.setColspan(3);
                    cell.setHorizontalAlignment(Element.ALIGN_LEFT);
                    table.addCell(cell);
              
                    String policyholder = insurance.getSubscriber() != null ? insurance.getSubscriber().getName() : "";
                    table.addCell(new Phrase("3.NAME OF POLICY HOLDER\n"+policyholder, font_courier));
                    table.addCell(new Phrase("4.POLICY NUMBER\n"+insurance.getPolicyNumber(), font_courier));
                    table.addCell(new Phrase("5.GROUP CODE\n"+insurance.getGroupNumber(), font_courier));
                    additionalInsDocument.add(table);
                }
            }
        }

        if (medicares != null && medicares.size() != 0) {
            // only Part A or Part A and B in current ESR if Part A and B there
            // are two medicare
            // records - one part A (Part A indicator ture, part B false) and
            // another part B (with both part A and part B indicator True)
            Medicare medicare = null;
            Medicare LatestMedicareA = null;
            Medicare LatestMedicareB = null;
            
            for (Iterator iterator = medicares.iterator(); iterator.hasNext();) {
            	medicare = (Medicare) iterator.next();
            	// Medicare B
                if (Boolean.TRUE.equals(medicare.isEnrolledInPartB())){
    				if(LatestMedicareB == null) {
    					LatestMedicareB = medicare;
    				} else {
    					// if(LatestMedicareB.getCreatedOn().before(medicare.getCreatedOn()))
    					if(LatestMedicareB.getModifiedOn().before(medicare.getModifiedOn()))
    						LatestMedicareB = medicare;
    				}
    			}else
    				// medicare A
    				if (Boolean.TRUE.equals(medicare.isEnrolledInPartA())){
    					if(LatestMedicareA == null) {
        					LatestMedicareA = medicare;
        				} else {
        					if(LatestMedicareA.getModifiedOn().before(medicare.getModifiedOn()))
        						LatestMedicareA = medicare;
        				}
    				}
            }
            if (LatestMedicareB != null) {
                fields.setField((String)fieldMap.get("Section2:Question8:Yes"), "Yes");
                fields.setField((String)fieldMap.get("Section2:Question8A"), DateUtils.format(LatestMedicareB.getPartBEffectiveDate(), null));
            }
            else fields.setField((String)fieldMap.get("Section2:Question8:No"),"Yes");
            
            if (LatestMedicareA != null) {
                fields.setField((String)fieldMap.get("Section2:Question7:Yes"), "Yes");
                fields.setField((String)fieldMap.get("Section2:Question7A"), DateUtils.format(LatestMedicareA.getPartAEffectiveDate(),null));
                fields.setField((String)fieldMap.get("Section2:Question9"), LatestMedicareA.getNameOnMedicareCard());
                fields.setField((String)fieldMap.get("Section2:Question10"), LatestMedicareA.getMedicareClaimNumber());
            }
            else  fields.setField((String)fieldMap.get("Section2:Question7:No"), "Yes");
        }
        // section 2 question 11 and 12 left unchecked
    }

    /**
     * Fill SECTION III - EMPLOYMENT INFORMATION of the fillable PDF document
     * 
     * @throws DocumentException
     * @throws IOException
     */

    private void fillSection3() throws DocumentException, IOException {
    	Employment vetEmp = person.getEmployment();
    	if (vetEmp!=null){
    		// CCR 10673
    		ImpreciseDate retiredate = vetEmp.getRetirementDate();
    		EmploymentStatus employmentstatus = vetEmp.getEmploymentStatus();
    		if (employmentstatus!=null){
    			if (EmploymentStatus.CODE_FULL_TIME.getCode().equals(employmentstatus.getCode())) {
    				fields.setField((String)fieldMap.get("Section3:Question1:FullTime"), "Yes");
    				fields.setField((String)fieldMap.get("Section3:Question1A"),getEmployerCompanyAddrPhone(vetEmp));
    			}
    			else if (EmploymentStatus.CODE_ACTIVE_MILITARYSERVICE.getCode().equals(employmentstatus.getCode()) ||
    			 EmploymentStatus.CODE_SELF_EMPLOYED.getCode().equals(employmentstatus.getCode()) )
    			{
    				fields.setField((String)fieldMap.get("Section3:Question1:FullTime"), "Yes");
    				 fields.setField((String)fieldMap.get("Section3:Question1A"),getEmployerCompanyAddrPhone(vetEmp));
    			}
    			else if (EmploymentStatus.CODE_PART_TIME.getCode().equals(employmentstatus.getCode()) )	
    			{
    				fields.setField((String)fieldMap.get("Section3:Question1:PartTime"), "Yes");
    				fields.setField((String)fieldMap.get("Section3:Question1A"),getEmployerCompanyAddrPhone(vetEmp));
    			}
    			else if (EmploymentStatus.CODE_NOT_EMPLOYED.getCode().equals(employmentstatus.getCode()))
    				  fields.setField((String)fieldMap.get("Section3:Question1:NotEmployed"), "Yes");
    			else if (EmploymentStatus.CODE_RETIRED.getCode().equals(employmentstatus.getCode()))
    			{
    				 fields.setField((String)fieldMap.get("Section3:Question1:Retired"), "Yes");
    				 fields.setField((String)fieldMap.get("Section3:Question1:RetirementDate"),DateUtils.format(retiredate, null));
    				 fields.setField((String)fieldMap.get("Section3:Question1A"),getEmployerCompanyAddrPhone(vetEmp));
    			}
    		}
    	}
    	// spouse employment information
        if (sf != null) {
            spouse = sf.getReportedOn();
        }
        if (spouse != null && spouse.getEmployment()!= null) {
        	Employment spEmp = spouse.getEmployment();
        	// CCR 10673
        	ImpreciseDate retiredate = spEmp.getRetirementDate();
    		EmploymentStatus employmentstatus = spEmp.getEmploymentStatus();
    		if (employmentstatus!=null){
    			if (EmploymentStatus.CODE_FULL_TIME.getCode().equals(employmentstatus.getCode())) {
    				fields.setField((String)fieldMap.get("Section3:Question2:FullTime"), "Yes");
    				fields.setField((String)fieldMap.get("Section3:Question2A"),getEmployerCompanyAddrPhone(spEmp));
    			}
    			else if (EmploymentStatus.CODE_ACTIVE_MILITARYSERVICE.getCode().equals(employmentstatus.getCode()) ||
    			 EmploymentStatus.CODE_SELF_EMPLOYED.getCode().equals(employmentstatus.getCode()) )
    			{
    				fields.setField((String)fieldMap.get("Section3:Question2:FullTime"), "Yes");
    				fields.setField((String)fieldMap.get("Section3:Question2A"),getEmployerCompanyAddrPhone(spEmp));
    			}
    			else if (EmploymentStatus.CODE_PART_TIME.getCode().equals(employmentstatus.getCode()) )	
    			{
    				fields.setField((String)fieldMap.get("Section3:Question2:PartTime"), "Yes");
    				fields.setField((String)fieldMap.get("Section3:Question2A"),getEmployerCompanyAddrPhone(spEmp));
    			}
    			else if (EmploymentStatus.CODE_NOT_EMPLOYED.getCode().equals(employmentstatus.getCode()))
    				  fields.setField((String)fieldMap.get("Section3:Question2:NotEmployed"), "Yes");
    			else if (EmploymentStatus.CODE_RETIRED.getCode().equals(employmentstatus.getCode()))
    			{
    				 fields.setField((String)fieldMap.get("Section3:Question2:Retired"), "Yes");
    				 fields.setField((String)fieldMap.get("Section3:Question2:RetirementDate"),DateUtils.format(retiredate, null));
    				 fields.setField((String)fieldMap.get("Section3:Question2A"),getEmployerCompanyAddrPhone(spEmp));
    			}
    		}
        }
    }

    /**
     * Fill SECTION IV - MILITARY SERVICE INFORMATION of the fillable PDF
     * document
     * 
     * @throws DocumentException
     * @throws IOException
     */

    private void fillSection4() throws DocumentException, IOException, Exception {
        PurpleHeart ph = person.getPurpleHeart();
        if (ph != null && ph.getPhIndicator() != null )
        { 
        	if ( Boolean.TRUE.equals(ph.getPhIndicator()) && ph.getStatus() != null 
                &&  DecorationStatus.CODE_CONFIRMED.getCode().equals(ph.getStatus().getCode()) ) {
        		fields.setField((String)fieldMap.get("Section4:Question2A:Yes"),"Yes");
        	} else if (isFalse(ph.getPhIndicator()))
        		fields.setField((String)fieldMap.get("Section4:Question2A:No"),"Yes");
        }

        PrisonerOfWar pow = person.getPrisonerOfWar();
        if (pow != null && pow.getPowIndicator() != null ){
            if ( Boolean.TRUE.equals(pow.getPowIndicator().toBoolean())) {
            fields.setField((String)fieldMap.get("Section4:Question2B:Yes"),"Yes");
            } else if (  Boolean.FALSE.equals(pow.getPowIndicator().toBoolean())) {
            fields.setField((String)fieldMap.get("Section4:Question2B:No"), "Yes");
            }
        }

        ServiceConnectionAward scAward = person.getServiceConnectionAward();
        if (scAward != null && scAward.getServiceConnectedPercentage() != null ) {
            if ( scAward.getServiceConnectedPercentage().intValue() > 0
                    && Boolean.TRUE.equals(scAward.getServiceConnectedIndicator())) {
                fields.setField((String)fieldMap.get("Section4:Question2C:Yes"), "Yes");
                fields.setField((String)fieldMap.get("Section4:Question2C1"),scAward.getServiceConnectedPercentage().toString());
            } else {
                fields.setField((String)fieldMap.get("Section4:Question2C:No"), "Yes");
            }
        } else
            fields.setField((String)fieldMap.get("Section4:Question2C:No"),"Yes");

        	// TODO exposure location Vietnam
        AgentOrangeExposure aoe = person.getAgentOrangeExposure();
        if (aoe != null && aoe.getAgentOrangeExposureIndicator() != null ){
        	if ( Indicator.YES.getCode().equals(aoe.getAgentOrangeExposureIndicator().getCode()) 
                &&  aoe.getLocation() != null && AgentOrangeExposureLocation.CODE_VIETNAM.getCode().equalsIgnoreCase(aoe.getLocation().getCode()) ) 
        	{
        		fields.setField((String)fieldMap.get("Section4:Question2G:Yes"),"Yes");
        	}
        	else fields.setField((String)fieldMap.get("Section4:Question2G:No"),"Yes");
        }

        // Radiation exposure
        RadiationExposure rexp = person.getRadiationExposure();
        if (rexp != null && rexp.getRadiationExposureIndicator() != null )
        {
             if( Indicator.YES.getCode().equals(rexp.getRadiationExposureIndicator().getCode())
                && rexp.getExposureMethod()!= null ) 
             {
            	   fields.setField((String)fieldMap.get("Section4:Question2H:Yes"),"Yes");
             } else fields.setField((String)fieldMap.get("Section4:Question2H:No"),"Yes");
        }

        // SW Asia conditions (Environmental Contamination Exposure)
        EnvironmentalContaminationExposure eexp = person.getEnvironmentalContaminationExposure();
        if (eexp != null && eexp.getEnvironmentalContaminationExposureIndicator() != null
                && Indicator.YES.getCode().equals(eexp.getEnvironmentalContaminationExposureIndicator().getCode())) {
            fields.setField((String)fieldMap.get("Section4:Question2F:Yes"),"Yes");
        } else 
            fields.setField((String)fieldMap.get("Section4:Question2F:No"),"Yes");
       
        // EnvironmentalContaminationExposure contExp = (EnvironmentalContaminationExposure)person.getSpecialFactorByType(EnvironmentalContaminationExposure.class);
        // Indicator envInd = (contExp != null) ? contExp.getEnvironmentalContaminationExposureIndicator() : null;
        // form.setExposedToContaminants((envInd != null) ? envInd.getCode() : Indicator.UNKNOWN.getCode());
      
        // person.getClinicalDeterminations();
        
        NoseThroatRadium ntr = (NoseThroatRadium)person.getClinicalDeterminationByType(NoseThroatRadium.class);
        if (ntr != null && ntr.getReceivingTreatment()!=null ){
        	// StringUtils.isNotEmpty(ntr.getVerificationDate().toString()) 
        	if ( ntr.getVerificationDate() != null  && TriState.TRUE.equals(ntr.getReceivingTreatment()) ){
        		fields.setField((String)fieldMap.get("Section4:Question2I:Yes"),"Yes");
        	}
        	else if (ntr.getVerificationDate() == null && TriState.FALSE.equals(ntr.getReceivingTreatment()) ){
        		fields.setField((String)fieldMap.get("Section4:Question2I:No"),"Yes");
        	}
        }
        // No is a definitive answer that we do not have and suggest the box be left unchecked
        // else fields.setField((String)fieldMap.get("Section4:Question2I:No"),"Yes");

        if (militaryService != null) {
          if ( militaryService.getDisabilityRetirementIndicator() != null
          && Indicator.YES.getCode().equals(this.getIndicatorCode(militaryService.getDisabilityRetirementIndicator())))
           {
                fields.setField((String)fieldMap.get("Section4:Question2E1:Yes"), "Yes");
            } else if ( militaryService.getDisabilityRetirementIndicator() != null
                    && Indicator.NO.getCode().equals(this.getIndicatorCode(militaryService.getDisabilityRetirementIndicator()))){
                fields.setField((String)fieldMap.get("Section4:Question2E1:No"), "Yes");
            }
          
          if (militaryService.getDischargeDueToDisability() != null &&
            Indicator.YES.getCode().equals(this.getIndicatorCode(militaryService.getDischargeDueToDisability())) )
            // if (militaryService.getDischargeDueToDisability() != null && militaryService.getDischargeDueToDisability().booleanValue() ) 
            {
                fields.setField((String)fieldMap.get("Section4:Question2E:Yes"), "Yes");
            } else if (militaryService.getDischargeDueToDisability() != null &&
                    Indicator.NO.getCode().equals(this.getIndicatorCode(militaryService.getDischargeDueToDisability())) ){
                fields.setField((String)fieldMap.get("Section4:Question2E:No"), "Yes");
            }
        } 
        /*
        else {
            fields.setField((String) fieldMap.get("Section4:Question2E1:No"),"No");
            fields.setField((String) fieldMap.get("Section4:Question2E:No"),"No");
        }
        */
        String militaryServiceNumber,branchOfService,entryDate,separationDate,dischargeType;
        MilitaryServiceSiteRecord latestmilitaryServiceSiteRecord = this.getLatestMilitaryServiceSiteRecord();
        MilitaryServiceEpisode latestEpisode = getLatestMilitaryServiceEpisode(latestmilitaryServiceSiteRecord);
        if (latestEpisode!=null){
        	militaryServiceNumber = latestEpisode.getServiceNumber();
			branchOfService = latestEpisode.getServiceBranch() == null ? "" : latestEpisode.getServiceBranch().getDescription();
            entryDate = JspUtils.displayValue(latestEpisode.getStartDate(),JspUtils.STANDARD_DATE_FORMAT);
            separationDate = JspUtils.displayValue(latestEpisode.getEndDate(),JspUtils.STANDARD_DATE_FORMAT);
            dischargeType = latestEpisode.getDischargeType() == null ? "" : latestEpisode.getDischargeType().getDescription();
			 fields.setField((String)fieldMap.get("Section4:Question1"), branchOfService);
			 fields.setField((String)fieldMap.get("Section4:Question1A"), entryDate);
		     fields.setField((String)fieldMap.get("Section4:Question1B"), separationDate);
		     fields.setField((String)fieldMap.get("Section4:Question1C"), dischargeType);
		     fields.setField((String)fieldMap.get("Section4:Question1D"),  militaryServiceNumber);
        }

        ImpreciseDate afterDate = ImpreciseDateUtils.createImpreciseDate(1998, 11, 11);
        boolean isServeCombat = false;
        Set combatepisodes = militaryService.getCombatEpisodes();
        MilitaryServiceActionHelper.sortCombatEpisodes(combatepisodes);
        if (combatepisodes != null && ! combatepisodes.isEmpty()){
            List combats = MilitaryServiceActionHelper.sortCombatEpisodes(combatepisodes);
            for (Iterator comb =combats.iterator(); comb.hasNext();) {
                CombatEpisode cbats = (CombatEpisode)comb.next(); 
                ImpreciseDate combatStartDate = cbats.getStartDate();
                if (combatStartDate != null && combatStartDate.compareTo(afterDate) == 1 ){
                    fields.setField((String) fieldMap.get("Section4:Question2D:Yes"),"Yes");
                    isServeCombat = true;
                    break;
                }    
            }
        }
        if (!isServeCombat)
            fields.setField((String) fieldMap.get("Section4:Question2D:No"),"Yes");
    }
        
     private void fillSection5() throws DocumentException, IOException { 
        
    	 IncomeTest test = person.getIncomeTest(this.incomeYear);
         if ( test == null || test.getDiscloseFinancialInformation() == null)
        	 FinDisclosureFlag = false;
         
         else {
	         if(isTrue(test.getDiscloseFinancialInformation()))  {
	             fields.setField((String) fieldMap.get("Section5:Yes"),"Yes");
	             FinDisclosureFlag = true;
	         }
	         else if(isFalse(test.getDiscloseFinancialInformation()))  {
	             fields.setField((String) fieldMap.get("Section5:No"),"Yes");
	             FinDisclosureFlag = false;
	         }
         }
         // test only
        // FinDisclosureFlag = true;
     }

    /**
     * Fill SECTION VII - DEPENDENT INFORMATION of the fillable PDF document
     * 
     * @throws DocumentException
     * @throws IOException
     */
 private void fillSection7() throws DocumentException, IOException { 
       
     if (spouse != null) {
         fields.setField((String)fieldMap.get("Section7:Question1"),spouse.getName()!= null ? getLastFirstMiddleByComma(spouse.getName()): "");
         fields.setField((String)fieldMap.get("Section7:Question1A"),spouse.getMaidenName());
         fields.setField((String)fieldMap.get("Section7:Question1B"),spouse.getOfficialSsn()!= null ? JspUtils.displaySSN(spouse.getOfficialSsn().getSsnText()): "");
         fields.setField((String)fieldMap.get("Section7:Question1C"),DateUtils.format(spouse.getDob(), null));
         fields.setField((String)fieldMap.get("Section7:Question1D"),DateUtils.format(spouse.getMarriageDate(), null));
         // new VOA
         String spAddr = spouse.getAddress()!= null? spouse.getAddress().getFormattedAddress():"";
         String spPhone = spouse.getHomePhone()!=null? spouse.getHomePhone().getFormattedPhone():"";
         fields.setField((String)fieldMap.get("Section7:Question1E"),spAddr+"\n"+spPhone); 
         if (FinDisclosureFlag && sf.getLivedWithPatient()!= null && !sf.getLivedWithPatient().booleanValue()){
             //fields.setField((String)fieldMap.get("Section7:Question3:Spouse"),ftmt.getContributionToSpouse()!=null? ftmt.getContributionToSpouse().toString(): "");
        	 fields.setField((String)fieldMap.get("Section7:Question3:Yes"),"Yes");  //Did you provide support Yes
         }   
     }
     
     DependentFinancials df = null;  
     Dependent dependent = null;
     int dep_cnt = 0;
     int add_cnt = 0;
    
     if (dependentFinancials!=null && dependentFinancials.size() != 0) { 
        boolean isFirst = true;
        boolean addFirst = true;
        for (Iterator itr =dependentFinancials.iterator(); itr.hasNext();) {
            df = (DependentFinancials) itr.next();
            dependent = df.getReportedOn();
            if ( dependent != null) {
            	dep_cnt++;
                if (isFirst){
                    fields.setField((String)fieldMap.get("Section7:Question2"), dependent.getName()!=null ? getLastFirstMiddleByComma(dependent.getName()):"" ); 
                    if (dependent.getRelationship()!=null){
	                    if( Relationship.CODE_SON.getCode().equals(dependent.getRelationship().getCode()) )
	                        fields.setField((String)fieldMap.get("Section7:Question2A:Son"),"Yes"); 
	                    else if ( Relationship.CODE_DAUGHTER.getCode().equals(dependent.getRelationship().getCode()) )
	                         fields.setField((String)fieldMap.get("Section7:Question2A:Daughter"),"Yes"); 
	                    else if ( Relationship.CODE_STEPSON.getCode().equals(dependent.getRelationship().getCode()) )
	                    	fields.setField((String)fieldMap.get("Section7:Question2A:Stepson"),"Yes");
	                    else if ( Relationship.CODE_STEPDAUGHTER.getCode().equals(dependent.getRelationship().getCode()) )
	                         fields.setField((String)fieldMap.get("Section7:Question2A:Stepdaughter"),"Yes");    
                    }
                    fields.setField((String)fieldMap.get("Section7:Question2B"),dependent.getOfficialSsn()!=null? JspUtils.displaySSN(dependent.getOfficialSsn().getSsnText()):"" );
                    fields.setField((String)fieldMap.get("Section7:Question2C"),DateUtils.format(dependent.getStartDate(), null));
                    fields.setField((String)fieldMap.get("Section7:Question2D"),DateUtils.format(dependent.getDob(), null));
                   
                    if (df.getIncapableOfSelfSupport()!=null && df.getIncapableOfSelfSupport().booleanValue())
                        fields.setField((String)fieldMap.get("Section7:Question2E:Yes"),"Yes");
                    else
                        fields.setField((String)fieldMap.get("Section7:Question2E:No"),"Yes"); 
                    
                    if (df.getAttendedSchool()!=null && df.getAttendedSchool().booleanValue())
                        fields.setField((String)fieldMap.get("Section7:Question2F:Yes"),"Yes");
                    else
                        fields.setField((String)fieldMap.get("Section7:Question2F:No"),"Yes"); 
                    
                    if (FinDisclosureFlag ){
                        if (df.getLivedWithPatient()!= null && !df.getLivedWithPatient().booleanValue())
                            //fields.setField((String)fieldMap.get("Section7:Question3:Child"),df.getAmountContributedToSupport()!=null? df.getAmountContributedToSupport().toString(): ""); 
                        	fields.setField((String)fieldMap.get("Section7:Question3:Yes"),"Yes");  //Did you provide support Yes
                        	fields.setField((String)fieldMap.get("Section7:Question2G"),df.getEducationExpense()!=null && df.getEducationExpense().getAmount()!=null? df.getEducationExpense().getAmount().toString():"");
                    }
                    isFirst = false;  
                }
                else{
                	 add_cnt ++;
                	 PdfPTable table=new PdfPTable(5);
                	 table.setKeepTogether(true);
                	 table.setWidthPercentage(100f);
                	 table.setSpacingAfter(0f);
                	 float[] widths = {23f,22f,5f,25f,25f};
               		 table.setWidths(widths);
                     if ( add_cnt % 4 == 1 || addFirst ){
	               		 PdfPCell cell = new PdfPCell(new Paragraph("APPLICATION FOR HEALTH BENEFITS, Continued",FontFactory.getFont(FontFactory.HELVETICA, 11,Font.BOLD, Color.BLACK)));
	               		 cell.setHorizontalAlignment(Element.ALIGN_CENTER);
	               		 cell.setColspan(2);
	               		 table.addCell(cell);
	               		 cell = new PdfPCell(new Paragraph("VETERAN'S NAME  (Last, First, Middle)\n\n"+veteranName, font_times));
	               		 cell.setColspan(2);
	               		 cell.setHorizontalAlignment(Element.ALIGN_LEFT);
	               		 table.addCell(cell); 
	               		 table.addCell( new Phrase("SOCIAL SECURITY NUMBER\n\n"+veteranSSN, font_times));
	               		 addFirst = false;
	               	}
                    PdfPCell cell = new PdfPCell (new Paragraph ("Section VII - DEPENDENT INFORMATION (Child #"+dep_cnt+")",FontFactory.getFont(FontFactory.HELVETICA, 8, Font.BOLD,Color.BLACK ))); 
                    cell.setColspan (5); 
                    cell.setHorizontalAlignment(Element.ALIGN_CENTER); 
                    cell.setBackgroundColor (Color.LIGHT_GRAY);  
                    table.addCell (cell); 
                    
                    String depname = dependent.getName()!=null ? getLastFirstMiddleByComma(dependent.getName()):"";
                    cell = new PdfPCell(new Phrase("2.CHILD'S NAME(Last, First , Middle Name)\n\n"+depname, font_courier));
                    cell.setColspan(3);
                    table.addCell(cell);
                   
                    String deprelation=dependent.getRelationship()!=null ? dependent.getRelationship().getDescription(): "";
                    cell = new PdfPCell(new Phrase("2A.CHILD'S RELATIONSHIP TO YOU\n\n"+deprelation, font_courier));
                    cell.setColspan(2);
                    table.addCell(cell);
                   
                    String depssn = dependent.getOfficialSsn()!=null?JspUtils.displaySSN(dependent.getOfficialSsn().getSsnText()):"";
                    table.addCell(new Phrase("2B.CHILD SOCIAL SECURITY NUMBER\n\n"+depssn, font_courier));
                    
                    cell = new PdfPCell(new Phrase("2C.DATE CHILD BECAME YOUR DEPENDENT (mm/dd/yyyy)\n\n"+DateUtils.format(dependent.getStartDate(), null), font_courier));
                    cell.setColspan(2);
                    table.addCell(cell);
                    
                    cell = new PdfPCell(new Phrase("2D.CHILD'S DATE OF BIRTH(mm/dd/yyyy)\n\n"+DateUtils.format(dependent.getDob(), null), font_courier)); 
                    cell.setColspan(2);
                    table.addCell(cell);
                    
                    String disable = df.getIncapableOfSelfSupport()!=null && df.getIncapableOfSelfSupport().booleanValue()? "YES":"NO";
                    cell = new PdfPCell(new Phrase("2E.WAS CHILD PERMANENTLY AND TOTALY DISABLED BEFORE THE AGE OF 18?\n\n"+disable, font_courier));
                    cell.setColspan(3);
                    table.addCell(cell);
  
                    String attendschool = df.getAttendedSchool()!=null && df.getAttendedSchool().booleanValue()? "YES":"NO";
                    cell = new PdfPCell(new Phrase("2F.IF CHILD IS BETWEEN 18 and 23 YEARS OF AGE,DID CHILD ATTEND SCHOOL LAST CALENDAR YEAR?\n\n"+attendschool, font_courier));
                    cell.setColspan(2);
                    table.addCell(cell);
                    
                    String childschool = "";
                    String childsupport = "";
                    if (FinDisclosureFlag){
                    	childschool= df.getEducationExpense()!=null && df.getEducationExpense().getAmount()!=null? df.getEducationExpense().getAmount().toString():"";
                    	childsupport = df.getAmountContributedToSupport()!=null? df.getAmountContributedToSupport().toString():"";
                    }
                    cell = new PdfPCell(new Phrase("3.IF YOUR DEPENDENT CHILD DID NOT LIVE WITH YOU LAST YEAR ENTER THE AMOUNT YOU CONTRIBUTED TO THEIR SUPPORT.\n\n"+"CHILD     $     "+childsupport, font_courier));
                    cell.setColspan(3);
                    table.addCell(cell);
                    
                    cell = new PdfPCell(new Phrase("2G.EXPENSES PAID BY YOUR DEPENDENT CHILD FOR COLLEGE,VOCATIONAL REHABILITATION ORTRAINING(e.g.,tuition,books,materials)\n\n"+"$     "+childschool, font_courier));
                    cell.setColspan(2);
                    table.addCell(cell); 
                    additionalChildDocument.add(table);
                }
            }
        }
    }
 }
    /**
     * Fill SECTION VIII - PREVIOUS CALENDAR YEAR GROSS ANNUAL INCOME OF
     * VETERAN, SPOUSE AND DEPENDENT CHILDREN of the fillable PDF document
     * 
     * @throws DocumentException
     * @throws IOException
     */

     private void fillSection8_9_10() throws DocumentException, IOException 
     { 
    	 DependentFinancials df = null;  
    	 Dependent dependent = null;
    	 if (FinDisclosureFlag ){
    		if ( ftmt !=null ){
    			// veteran financial info
    			fields.setField((String)fieldMap.get("Section8:Question1:Veteran"),ftmt.getTotalEmploymentIncome()!=null && ftmt.getTotalEmploymentIncome().getAmount()!=null ? ftmt.getTotalEmploymentIncome().getAmount().toString():"");
    			fields.setField((String)fieldMap.get("Section8:Question2:Veteran"),ftmt.getFarmRanchBusinessIncome()!=null && ftmt.getFarmRanchBusinessIncome().getAmount()!=null ? ftmt.getFarmRanchBusinessIncome().getAmount().toString():"");
    			fields.setField((String)fieldMap.get("Section8:Question3:Veteran"),ftmt.getOtherIncome()!=null && ftmt.getOtherIncome().getAmount()!=null ? ftmt.getOtherIncome().getAmount().toString():"");
    			fields.setField((String)fieldMap.get("Section9:Question1"),ftmt.getNonReimbursableExpense()!=null && ftmt.getNonReimbursableExpense().getAmount()!=null? ftmt.getNonReimbursableExpense().getAmount().toString():"");
     			fields.setField((String)fieldMap.get("Section9:Question2"),ftmt.getFuneralBurialExpense()!=null && ftmt.getFuneralBurialExpense().getAmount()!=null ? ftmt.getFuneralBurialExpense().getAmount().toString(): "");
     			fields.setField((String)fieldMap.get("Section9:Question3"),ftmt.getVeteranEducationExpense()!=null && ftmt.getVeteranEducationExpense().getAmount()!=null ? ftmt.getVeteranEducationExpense().getAmount().toString():"");
     			fields.setField((String)fieldMap.get("Section10:Question1:Veteran"),ftmt.getCashAndBankAccountBalances()!=null && ftmt.getCashAndBankAccountBalances().getAmount()!=null ? ftmt.getCashAndBankAccountBalances().getAmount().toString():"");
     			fields.setField((String)fieldMap.get("Section10:Question2:Veteran"),ftmt.getRealEstate()!=null && ftmt.getRealEstate().getAmount()!=null? ftmt.getRealEstate().getAmount().toString():"");
     			fields.setField((String)fieldMap.get("Section10:Question3:Veteran"),ftmt.getOtherPropertyOrAssets()!=null && ftmt.getOtherPropertyOrAssets().getAmount()!=null? ftmt.getOtherPropertyOrAssets().getAmount().toString():"");
    		}
    		// spouse financial info
            if(sf != null) {  // :new BigDecimal(0d).toString()
            	fields.setField((String)fieldMap.get("Section8:Question1:Spouse"),sf.getTotalEmploymentIncome()!=null && sf.getTotalEmploymentIncome().getAmount()!=null ? sf.getTotalEmploymentIncome().getAmount().toString():"");
            	fields.setField((String)fieldMap.get("Section8:Question2:Spouse"),sf.getFarmRanchBusinessIncome()!=null && sf.getFarmRanchBusinessIncome().getAmount()!= null ? sf.getFarmRanchBusinessIncome().getAmount().toString():"");
            	fields.setField((String)fieldMap.get("Section8:Question3:Spouse"),sf.getOtherIncome()!=null && sf.getOtherIncome().getAmount()!=null ? sf.getOtherIncome().getAmount().toString():""); 
    			fields.setField((String)fieldMap.get("Section10:Question1:Spouse"),sf.getCashAndBankAccountBalances()!=null && sf.getCashAndBankAccountBalances().getAmount()!=null ? sf.getCashAndBankAccountBalances().getAmount().toString(): "");
     			fields.setField((String)fieldMap.get("Section10:Question2:Spouse"),sf.getRealEstate()!=null && sf.getRealEstate().getAmount()!=null ? sf.getRealEstate().getAmount().toString(): "");
     			fields.setField((String)fieldMap.get("Section10:Question3:Spouse"),sf.getOtherPropertyOrAssets()!=null && sf.getOtherPropertyOrAssets().getAmount()!=null? sf.getOtherPropertyOrAssets().getAmount().toString(): "");
            }
            // Children financial info 
           if (dependentFinancials!=null && dependentFinancials.size() != 0) { 
        	   boolean isFirst_8 = true;
        	   boolean isFirst_10 = true;   
        	   boolean addFirst = true;
        	   int childCnt = 0;
        	   int add_child_cnt = 0;
        	   // Section VIII
        	   for (Iterator itr =dependentFinancials.iterator(); itr.hasNext();) {
                    df = (DependentFinancials)itr.next();
                    dependent = df.getReportedOn();
                    if ( dependent != null) {
		                childCnt ++;
			            if (isFirst_8) { 
					 		fields.setField((String)fieldMap.get("Section8:Question1:Child"),df.getTotalEmploymentIncome()!=null && df.getTotalEmploymentIncome().getAmount()!=null? df.getTotalEmploymentIncome().getAmount().toString():"");
					 		fields.setField((String)fieldMap.get("Section8:Question2:Child"),df.getFarmRanchBusinessIncome()!=null && df.getFarmRanchBusinessIncome().getAmount()!=null ? df.getFarmRanchBusinessIncome().getAmount().toString():"");
					 		fields.setField((String)fieldMap.get("Section8:Question3:Child"),df.getOtherIncome()!=null && df.getOtherIncome().getAmount()!=null? df.getOtherIncome().getAmount().toString():""); 
					 		isFirst_8 = false; 
					 	}
					 	else { 
					 		 add_child_cnt ++;
					 		 if (add_child_cnt % 9 == 1  || addFirst ){
					 			 additionalChildDocument.newPage();
					 			 PdfPTable table=new PdfPTable(3);
					        	 table.setWidthPercentage(100f);
					        	 table.setSpacingAfter(0f);
					        	 table.setKeepTogether(true);
			               		 float[] widths = {45f,30f,25f};
			               		 table.setWidths(widths);
			               		 PdfPCell cell = new PdfPCell(new Paragraph("APPLICATION FOR HEALTH BENEFITS, Continued",FontFactory.getFont(FontFactory.HELVETICA, 11,Font.BOLD, Color.BLACK)));
			               		 cell.setHorizontalAlignment(Element.ALIGN_CENTER);
			               		 table.addCell(cell);
			               		 cell = new PdfPCell(new Paragraph("VETERAN'S NAME  (Last, First, Middle)\n\n"+veteranName, font_times));
			               		 cell.setHorizontalAlignment(Element.ALIGN_LEFT);
			               		 table.addCell(cell);
			               		 table.addCell( new Phrase("SOCIAL SECURITY NUMBER\n\n"+veteranSSN, font_times));
			               		 cell = new PdfPCell (new Paragraph ("Section VIII - PREVIOUS CALENDAR YEAR GROSS ANNUAL INCOME OF VETERAN, SPOUSE AND DEPENDENT CHILDREN (Continued)",FontFactory.getFont(FontFactory.HELVETICA, 8,Font.BOLD, Color.BLACK ))); 
			               		 cell.setColspan (3);
			               		 cell.setHorizontalAlignment (Element.ALIGN_CENTER);
			               		 cell.setBackgroundColor (Color.LIGHT_GRAY); //cell.setPadding (10.0f);
			               		 table.addCell (cell); 
			               		 addFirst = false;
			               		 additionalChildDocument.add(table);
			               	}
					 		PdfPTable table=new PdfPTable(3);
				        	table.setWidthPercentage(100f);
				        	table.setKeepTogether(true);
			    		 	float[] widths = {34f,33f,33f};
			    		 	table.setWidths(widths);
			    		 	table.setSpacingAfter(0f);
			    		 	PdfPCell cell = new PdfPCell (new Paragraph ("Child #"+childCnt,FontFactory.getFont(FontFactory.HELVETICA, 8,Font.BOLD, Color.BLACK ))); 
			    		 	cell.setColspan (3);
			    		 	cell.setHorizontalAlignment (Element.ALIGN_CENTER);
			    		 //	cell.setBackgroundColor (Color.LIGHT_GRAY); //cell.setPadding (10.0f);
			    		 	table.addCell (cell); 
			    		 	String grossincome = df.getTotalEmploymentIncome()!=null && df.getTotalEmploymentIncome().getAmount()!=null? df.getTotalEmploymentIncome().getAmount().toString():"";
			    		 	table.addCell(new Phrase("1.GROSS ANNUAL INCOME FROM EMPLOYMENT  * (wages,bonuses,tips,etc.) EXCLUDING INCOME FROM YOUR FARM,RANCH,PROPERTYOR BUSINESS\n\n"+this.addCurrencySymbol(grossincome), font_courier)); 
			    		 	String netincome =df.getFarmRanchBusinessIncome()!=null && df.getFarmRanchBusinessIncome().getAmount()!=null? df.getFarmRanchBusinessIncome().getAmount().toString():"";
			    		 	table.addCell(new Phrase("2.NET INCOME FROM YOUR FARM,RANCH,PROPERTY,OR BUSINESS\n\n"+this.addCurrencySymbol(netincome), font_courier));
			    		 	String otherincome = df.getOtherIncome()!=null && df.getOtherIncome().getAmount()!= null ? df.getOtherIncome().getAmount().toString():"";
			    		 	table.addCell(new Phrase("3.LIST OTHER INCOME AMOUNTS (eg.Social Security, compensation, pension interest,dividends),EXCLUDING WELFARE.\n\n"+this.addCurrencySymbol(otherincome), font_courier));
			    		 	// additionalChildFinDocument.add(table);
			    		 	additionalChildDocument.add(table);
					 	} 
		            }
        	   }
        	  
        	   // Section X
		       childCnt = 0;
		       add_child_cnt = 0;
		       addFirst = true;
               for (Iterator itr =dependentFinancials.iterator(); itr.hasNext();) {
                    df = (DependentFinancials) itr.next();
                    dependent = df.getReportedOn();
                    if ( dependent != null) {
		                childCnt ++;
	    			 	if (isFirst_10){
	    			 		fields.setField((String)fieldMap.get("Section10:Question1:Child"),df.getCashAndBankAccountBalances()!=null && df.getCashAndBankAccountBalances().getAmount()!=null ? df.getCashAndBankAccountBalances().getAmount().toString(): "");
	    	     			fields.setField((String)fieldMap.get("Section10:Question2:Child"),df.getRealEstate()!=null && df.getRealEstate().getAmount()!=null ? df.getRealEstate().getAmount().toString():"");
	    	     			fields.setField((String)fieldMap.get("Section10:Question3:Child"),df.getOtherPropertyOrAssets()!=null && df.getOtherPropertyOrAssets().getAmount()!=null? df.getOtherPropertyOrAssets().getAmount().toString(): "");
	    	     			isFirst_10 = false;
	    			 	}
	    			 	else{
	    			 		// new code
	    			 		 add_child_cnt ++;
					 		 if (add_child_cnt % 7 == 1  || addFirst ){
					 			 additionalChildDocument.newPage();
					 			 PdfPTable table=new PdfPTable(3);
					        	 table.setWidthPercentage(100f);
					        	 table.setSpacingAfter(0f);
					        	 table.setKeepTogether(true);
			               		 float[] widths = {45f,30f,25f};
			               		 table.setWidths(widths);
			               		 PdfPCell cell = new PdfPCell(new Paragraph("APPLICATION FOR HEALTH BENEFITS, Continued",FontFactory.getFont(FontFactory.HELVETICA, 11,Font.BOLD, Color.BLACK)));
			               		 cell.setHorizontalAlignment(Element.ALIGN_CENTER);
			               		 table.addCell(cell);
			               		 cell = new PdfPCell(new Paragraph("VETERAN'S NAME  (Last, First, Middle)\n\n"+veteranName, font_times));
			               		 cell.setHorizontalAlignment(Element.ALIGN_LEFT);
			               		 table.addCell(cell);
			               		 table.addCell( new Phrase("SOCIAL SECURITY NUMBER\n\n"+veteranSSN, font_times));
			               		 cell = new PdfPCell (new Paragraph ("Section X - PREVIOUS CALENDAR YEAR NET WORTH (Continued)",FontFactory.getFont(FontFactory.HELVETICA, 8,Font.BOLD, Color.BLACK ))); 
			               		 cell.setColspan (3);
			               		 cell.setHorizontalAlignment (Element.ALIGN_CENTER);
			               		 cell.setBackgroundColor (Color.LIGHT_GRAY); //cell.setPadding (10.0f);
			               		 table.addCell (cell); 
			               		 addFirst = false;
			               		 additionalChildDocument.add(table);
					 		 }
					 		// end of new code
	    			 		PdfPTable table=new PdfPTable(3);
    					    table.setWidthPercentage(100f);
    					    float[] widths = {34f,33f,33f};
    			   		 	table.setWidths(widths);
    			   		 	table.setSpacingAfter(0f);
    			   		 	/*
	    			 		if (addFirst){
			               		 PdfPCell cell = new PdfPCell (new Paragraph ("Section X - PREVIOUS CALENDAR YEAR NET WORTH (Continued)",FontFactory.getFont(FontFactory.HELVETICA, 8,Font.BOLD, Color.BLACK ))); 
			               		 cell.setColspan (3);
			               		 cell.setHorizontalAlignment (Element.ALIGN_CENTER);
			               		 cell.setBackgroundColor (Color.LIGHT_GRAY); //cell.setPadding (10.0f);
			               		 table.addCell (cell); 
			               		 addFirst = false;
			               	}
			               	*/
			    		 	PdfPCell cell = new PdfPCell (new Paragraph ("Child #"+childCnt,FontFactory.getFont(FontFactory.HELVETICA, 8,Font.BOLD, Color.BLACK ))); 
			    		 	cell.setColspan (3);
			    		 	cell.setHorizontalAlignment (Element.ALIGN_CENTER);
			    		 	table.addCell (cell); 
			    		 	String bankamt  = df.getCashAndBankAccountBalances()!=null && df.getCashAndBankAccountBalances().getAmount()!=null ? df.getCashAndBankAccountBalances().getAmount().toString(): "";
	    			 		table.addCell(new Phrase("1.CASH,AMOUNT IN BANK ACCOUNTS.\n\n"+this.addCurrencySymbol(bankamt),font_courier)); 
	    			 		String marketvalue = df.getRealEstate()!=null && df.getRealEstate().getAmount()!=null ? df.getRealEstate().getAmount().toString(): "";
	    			 		table.addCell(new Phrase("2.MARKET VALUE OF LAND AND BUILDINGS MINUS MORTGAGES AND LIENS. (e.g. second homes and non-income producing property. Do not countyour primary home.)\n\n"+this.addCurrencySymbol(marketvalue), font_courier)); 
	    			 		String othervalue = df.getOtherPropertyOrAssets()!=null && df.getOtherPropertyOrAssets().getAmount()!=null? df.getOtherPropertyOrAssets().getAmount().toString(): "";
	    			 		table.addCell(new Phrase("3.VALUE OF OTHER PROPERTY OR ASSETS (e.g. art, rare coins,collectables) MINUS THE AMOUNT YOU OWE ON THESE ITEMS. INCLUDE VALUE OFFARM, RANCH OR BUSINESS ASSETS. Exclude household effects and family vehicles.\n\n"+this.addCurrencySymbol(othervalue), font_courier)); 
	    			 		additionalChildDocument.add(table);
	    			 	}
                    }
               }
           }
       }
  }
    /**
     * Fill SECTION XII - ASSIGNMENT OF BENEFITS of the fillable PDF document
     * 
     * @throws DocumentException
     * @throws IOException
     */

    private void fillSection12() throws DocumentException, IOException {
        fields.setField((String) fieldMap.get("Section12:Date"),
                new SimpleDateFormat("MM/dd/yyyy").format(new Date()));
    }

    /**
     * Remove the currency symbol (i.e. $)
     * 
     * @param currency
     *            String value containing a currency amount that includes
     *            currency symbol (i.e. $)
     * @return String value containing a currency amount excluding currency
     *         symbol (i.e. $) since it is already provided on the form
     */
    
     private String removeCurrencySymbol(String currency) { 
         if (currency != null && StringUtils.isEmpty(currency)) 
         { return currency.substring(1); } 
         return ""; 
     }
     
     private String addCurrencySymbol(String currency) { 
         if (currency != null && StringUtils.isNotEmpty(currency)) 
         { return "$"+" "+currency; } 
         return "$"; 
     }
    private String getInsuranceCompanyAddrPhone(InsurancePolicy insurance) {
        StringBuffer insuranceInfo = new StringBuffer();
        if (insurance.getCompanyName() != null && StringUtils.isNotEmpty(insurance.getCompanyName()))
            insuranceInfo.append(insurance.getCompanyName());
        
        if (insurance.getAddress() != null && StringUtils.isNotEmpty(insurance.getAddress().getFormattedAddress())) {
            if (insuranceInfo.length() > 0) {
                insuranceInfo.append("\n");
            }
            insuranceInfo.append(insurance.getAddress().getFormattedAddress());
        }
        if (insurance.getBusinessPhone() != null && StringUtils.isNotEmpty(insurance.getBusinessPhone().getFormattedPhone())) {
            if (insuranceInfo.length() > 0) {
                insuranceInfo.append("\n");
            }
            insuranceInfo.append(insurance.getBusinessPhone().getFormattedPhone());
        }
        return insuranceInfo.toString();
    }

	private String getEmployerCompanyAddrPhone(Employment em){ 
		StringBuffer emInfo = new StringBuffer(); 
		if (em.getEmployerName() !=null && StringUtils.isNotEmpty(em.getEmployerName()) )
			emInfo.append(em.getEmployerName()); 
		
		if (em.getEmployerAddress() !=null && StringUtils.isNotEmpty(em.getEmployerAddress().getFormattedAddress()) ){
			if(emInfo.length() > 0) { emInfo.append("\n"); }
				emInfo.append(em.getEmployerAddress().getFormattedAddress()); 
		} 
		
		if(em.getEmployerPhone() !=null && StringUtils.isNotEmpty(em.getEmployerPhone())) { 
			if(emInfo.length() > 0) { emInfo.append("\n"); }
				emInfo.append(em.getEmployerPhone()); 
		} 
		return emInfo.toString(); 
	}
    
    private String getIndicatorCode(Boolean flag) {
        if(flag == null) {
            return Indicator.UNKNOWN.getCode();
        } else {
            return (flag.equals(Boolean.TRUE)) ? Indicator.YES.getCode() : Indicator.NO.getCode();
        }
    }
    
    private boolean isTrue(Boolean bool) {
        return bool != null && bool.booleanValue();
    }

    private boolean isTrue(Indicator ind) {
        return ind != null && ind.getCode().equals(Indicator.YES.getCode());
    }
    
    private boolean isFalse(Boolean bool) {
        return bool != null && !bool.booleanValue();
    }
    
    private boolean isFalse(Indicator ind) {
        return ind != null && ind.getCode().equals(Indicator.NO.getCode());
    }

    private boolean isUnknown(Boolean bool) {
        return bool == null;
    }
    
    private boolean isUnknown(Indicator ind) {
        return ind != null && ind.getCode().equals(Indicator.UNKNOWN.getCode());
    }      

    private List getPatientVisitSummaries() {
    	//  Map facilities = new HashMap ();
        Map patientVisits = null;
        Set facilityKeys = null;
        if (this.patientVisitSummaries == null) 
			this.patientVisitSummaries = new ArrayList ();
    	patientVisits = person.getPatientVisitSummaries();
    	if ( patientVisits !=  null)
    		facilityKeys = patientVisits.keySet();
    	
    	if ( facilityKeys != null){
			for (Iterator iter=facilityKeys.iterator(); iter.hasNext();) {
				SiteYear siteYear = (SiteYear) iter.next ();
		        if(siteYear != null) {
		        	VAFacility facility = siteYear.getFacility();
		        	if ( facility!=null && facility.getType()!=null && facility.getType().getIsMedicalTreating()){
		        		PatientVisitSummary patientVisitSummary = (PatientVisitSummary)patientVisits.get(siteYear);
		        		this.patientVisitSummaries.add(patientVisitSummary);
		        	}
		        }
			}
    	}
    	return patientVisitSummaries;
	}
    
	private MilitaryServiceSiteRecord getLatestMilitaryServiceSiteRecord()  throws Exception
	{
	    boolean isHECsiteRecordPresent = false;
	    boolean isVOAsiteRecordPresent = false;
	    boolean isVAMCsiteRecordPresent = false;
	    MilitaryServiceSiteRecord latestSiteRec = null;
		MilitaryServiceSiteRecord siteRec = null;
    
		Iterator itr = null;
		if(this.militaryService != null) {
			Set siteRecords = militaryService.getMilitaryServiceSiteRecords();
			itr = siteRecords != null ? siteRecords.iterator() : null;
			MilitaryServiceSiteRecord latestHECSiteRec = null;
			MilitaryServiceSiteRecord latestVOASiteRec = null;
			MilitaryServiceSiteRecord latestVAMCSiteRec = null;
			
			// take the "latest" site data ESR has on file
			while(itr != null && itr.hasNext()) {
				siteRec = (MilitaryServiceSiteRecord) itr.next();
				// HEC
				if ( siteRec.getSite()!=null && VAFacility.CODE_HEC.getName().equalsIgnoreCase(siteRec.getSite().getName())){
					isHECsiteRecordPresent = true;
					if(latestHECSiteRec == null) {
						latestHECSiteRec = siteRec;
					} else {
						// if(latestHECSiteRec.getCreatedOn().before(siteRec.getCreatedOn()))
						 if(latestHECSiteRec.getModifiedOn().before(siteRec.getModifiedOn()))
							latestHECSiteRec = siteRec;
					}
				}
				
				else  // VOA CODE_MHV
					if ( siteRec.getSite()!=null && VAFacility.CODE_MHV.getName().equalsIgnoreCase(siteRec.getSite().getName())){
							isVOAsiteRecordPresent = true;
							if(latestVOASiteRec == null) {
								latestVOASiteRec = siteRec;
							} 
							else {
								if(latestVOASiteRec.getModifiedOn().before(siteRec.getModifiedOn()))
									latestVOASiteRec = siteRec;
							}
						}
					else { // anything else consider VAMC 
						isVAMCsiteRecordPresent = true;
						if(latestVAMCSiteRec == null) {
							latestVAMCSiteRec = siteRec;
						} 
						else {
							if(latestVAMCSiteRec.getModifiedOn().before(siteRec.getModifiedOn()))
								latestVAMCSiteRec = siteRec;
						}
					}
				}
			
			if (isHECsiteRecordPresent) latestSiteRec = latestHECSiteRec;
			else if (isVAMCsiteRecordPresent) latestSiteRec = latestVAMCSiteRec;
			else if (isVOAsiteRecordPresent) latestSiteRec = latestVOASiteRec;
	
	     	}
    	return latestSiteRec;
	}
    
    private MilitaryServiceEpisode getLatestMilitaryServiceEpisode(MilitaryServiceSiteRecord mtsiterecords)  throws Exception
	{
    	MilitaryServiceEpisode latestEpisode = null;
        Iterator itr = null;
        
		if(mtsiterecords != null) {
			Set episodes = mtsiterecords.getMilitaryServiceEpisodes();
			itr = episodes != null ? episodes.iterator() : null;
			
			MilitaryServiceEpisode episode = null;
			// take the "latest" episode for this guy
			while(itr != null && itr.hasNext()) {
				episode = (MilitaryServiceEpisode) itr.next();
				if(latestEpisode == null) {
					latestEpisode = episode;
				} else {
					if(latestEpisode.getEndDate() != null && latestEpisode.getEndDate().compareTo(episode.getEndDate()) < 0)
						latestEpisode = episode;
				}
			}
		}
		return latestEpisode;		
	}

    private List sortPrivateInsurance(Set pinsr )
    {   
    	if ( pinsr == null ) return null;
        List privateList = new ArrayList(pinsr);
        if(!privateList.isEmpty())
        {
            Comparator comparator = new Comparator()
            {
                public int compare(Object pObject1, Object pObject2)
                {
                    Date startImDate1 = (pObject1 != null && pObject1 instanceof PrivateInsurance) ? ((PrivateInsurance)pObject1).getModifiedOn() : null;
                    Date startImDate2 = (pObject2 != null && pObject2 instanceof PrivateInsurance) ? ((PrivateInsurance)pObject2).getModifiedOn() : null;
                    if(startImDate1 != null && startImDate2 != null)
                    {
                        // return (startImDate1.compareTo(startImDate2)); ascending oldest -> latest date
                    	long thisTime = startImDate1.getTime();
                    	long secondTime = startImDate2.getTime();
                    	return (thisTime > secondTime ? -1 : (thisTime==secondTime ? 0 : 1));
                    }
                    return 0;
                }
            };
            Collections.sort(privateList,comparator);
        }
        return privateList;
    }
    
 private String getLastFirstMiddleByComma(Name name){
    	
    	StringBuffer buf = new StringBuffer();
	    if (name == null )
	    	return null;

	    String flname = name.getFormattedName();
	    String middleName = name.getMiddleName();
	    if (StringUtils.isNotBlank(flname))
        {
	    	buf.append(flname);
        }
	    if (StringUtils.isNotBlank(middleName))
        {
            if (buf.length() > 0)
            {
                buf.append(", ");
            }
            buf.append(middleName);
        }
        return buf.toString().toUpperCase();
	}
 
 public void display1010EZ(HttpServletResponse resp)throws Exception{
	 InputStream input= null;
	 DataOutputStream dataostream = null;
	 String finalFname = this.getOutputFilePath() + this.getFinalFileName();
	 try {
	 	
	 	input= new FileInputStream(finalFname);
	    resp.setContentType("application/pdf");
	    resp.setHeader("Content-Disposition", "inline");
	    resp.setHeader("Accept-Ranges", "bytes");
	     
	    dataostream = new DataOutputStream(resp.getOutputStream());
	    byte[] buf = new byte[4096];
	    int bytesRead;
	    while ((bytesRead = input.read(buf)) > 0) {
	    	dataostream.write(buf, 0, bytesRead);
	    }
	     
	 } finally {
		 if (input != null) {
			 input.close();
		 }
		 if (dataostream != null) {
			 dataostream.close();
		 }		 
		 File file_temp = new File(finalFname);
		 if ( file_temp!=null && file_temp.exists()) {
	    	 file_temp.delete();	
	     }
	 }
 }
	     
 private String getBirthPlace(String city, String state) {
		StringBuffer birthPlace = new StringBuffer();
		if (city!= null) {
			birthPlace.append(city);
		}
		if (birthPlace.length() > 0) {
				birthPlace.append(", ");
		}
		if (state != null) {
			birthPlace.append(state);
		}
		return birthPlace.toString();
	}
 
 	private InputStream readTemplate(String fillableFormFileName)throws Exception
 	{ 
 		FileInputStream inFillablePDFFile = new FileInputStream(fillableFormFileName);
 		InputStream inFillablePDF = (InputStream)inFillablePDFFile;
 		return inFillablePDF;
 	}
    
}
