package gov.va.med.esr.voa.validator;

import gov.va.med.esr.voa.common.ErrorCode;
import gov.va.med.esr.voa.webservice.EmailInfo;
import gov.va.med.fw.service.AbstractComponent;
import gov.va.med.fw.util.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

/**
 * Validate email address
 */
public class EmailInfoValidator extends AbstractComponent implements Validator {

	private static final Pattern pattern = Pattern
			.compile("^([0-9a-zA-Z]([-\\.\\w]*[0-9a-zA-Z])*@([0-9a-zA-Z][-\\w]*[0-9a-zA-Z]\\.)+[a-zA-Z]{2,9})$");

	@SuppressWarnings("rawtypes")
	@Override
	public boolean supports(Class clazz) {
		return EmailInfo.class.equals(clazz);
	}

	@Override
	public void validate(Object target, Errors errors) {
		logger.debug("start email address validation...");
		
		if (target != null) {
			EmailInfo emailInfo = (EmailInfo) target;

			// Null allowed; If not null then the value must be at least 3
			// characters but no more than 50 characters in length, with "@" not
			// in
			// the first or last character
			String emailAddress = emailInfo.getAddress();
			if (StringUtils.isNotBlank(emailAddress)) {
				Matcher matcher = pattern.matcher(emailAddress);
				boolean matchFound = matcher.matches();
				if (!matchFound) {
					errors.reject(ErrorCode.VOA_0036,
							"Invalid Veteran email address.");
					logger.debug("email address " + emailAddress
							+ " is not valid.");
				} else {
					int numberOfChars = emailAddress.length();
					if (numberOfChars < 3 || numberOfChars > 50) {
						errors.reject(ErrorCode.VOA_0036,
								"Invalid Veteran email address.");
						logger.debug("email address " + emailAddress
								+ " has less than 3 digits.");
					}
				}
			}
		}

		logger.debug("end email address validation.");
	}

}
