/**
 * 
 */
package gov.va.med.esr.voa.webservice.builder.impl;


import java.util.Iterator;
import java.util.List;

import gov.va.med.esr.common.infra.ImpreciseDate;
import gov.va.med.esr.common.model.ee.Application;
import gov.va.med.esr.common.model.insurance.InsurancePolicy;
import gov.va.med.esr.common.model.insurance.InsuranceSubscriber;
import gov.va.med.esr.common.model.insurance.Medicare;
import gov.va.med.esr.common.model.insurance.PrivateInsurance;
import gov.va.med.esr.common.model.lookup.NameType;
import gov.va.med.esr.common.model.lookup.SSNType;
import gov.va.med.esr.common.model.party.Address;
import gov.va.med.esr.common.model.party.Email;
import gov.va.med.esr.common.model.party.Phone;
import gov.va.med.esr.common.model.person.Association;
import gov.va.med.esr.common.model.person.BirthRecord;
import gov.va.med.esr.common.model.person.Employment;
import gov.va.med.esr.common.model.person.Ethnicity;
import gov.va.med.esr.common.model.person.Name;
import gov.va.med.esr.common.model.person.Person;
import gov.va.med.esr.common.model.person.PreferredFacility;
import gov.va.med.esr.common.model.person.Race;
import gov.va.med.esr.common.model.person.SSN;
import gov.va.med.esr.service.PersonIdentityTraits;
import gov.va.med.esr.voa.webservice.AddressInfo;
import gov.va.med.esr.voa.webservice.ApplicationInfo;
import gov.va.med.esr.voa.webservice.AssociationInfo;
import gov.va.med.esr.voa.webservice.BaseAddressInfo;
import gov.va.med.esr.voa.webservice.EmailInfo;
import gov.va.med.esr.voa.webservice.EmploymentInfo;
import gov.va.med.esr.voa.webservice.InsuranceInfo;
import gov.va.med.esr.voa.webservice.PersonInfo;
import gov.va.med.esr.voa.webservice.PhoneInfo;
import gov.va.med.esr.voa.webservice.SubmitFormRequest;
import gov.va.med.esr.voa.webservice.VeteranIdentifier;
import gov.va.med.esr.voa.webservice.builder.DemographicsBuilderFactory;
import gov.va.med.ps.model.PersonVPID;

/**
 * @author DNS   CHINNR
 *
 */
public class DemographicsBuilderFactoryImpl implements DemographicsBuilderFactory {
	
	private TypesHelper typesHelper = new TypesHelper();
	private BuildHelper buildHelper = new BuildHelper();
	
	@SuppressWarnings("rawtypes")
	public void buildDemographics(Person person,SubmitFormRequest submitFormRequest){
		
		PersonInfo personInfo = submitFormRequest.getForm().getSummary().getPersonInfo();
		
		 VeteranIdentifier vid = submitFormRequest.getIdentity().getVeteranIdentifier();
		 PersonIdentityTraits pidt = new PersonIdentityTraits();
		 Name name = new Name();
		 if(personInfo != null){
		 if(personInfo.getLastName() != null)name.setFamilyName(personInfo.getLastName());
		 if(personInfo.getFirstName() != null)name.setGivenName(personInfo.getFirstName());
		 // As per the discussion, the nametype is set as legal name. VOA won't send the name type
		 name.setType(this.typesHelper.getNameType(NameType.LEGAL_NAME.getCode()));
		 if(personInfo.getMiddleName() != null)name.setMiddleName(personInfo.getMiddleName());
		// if(personInfo.getPrefix()!=null)name.setPrefix(personInfo.getPrefix());
		 
		 if(personInfo.getSuffix() != null)name.setSuffix(personInfo.getSuffix());
		 person.addName(name);
		 pidt.addName(name);
		 if(personInfo.getMothersMaidenName() != null){
			 person.setMothersMaidenName(personInfo.getMothersMaidenName());
			 pidt.setMothersMaidenName(personInfo.getMothersMaidenName());
			}
		 if (vid != null) {
			 PersonVPID vpid = new PersonVPID(vid.getValue());
			 person.setVPID(vpid);
		 }
		 SSN ssn = new SSN();
		 if(personInfo.getSsnText() != null){
			 ssn.setSsnText(personInfo.getSsnText());
			 //ssntype is active default
			 ssn.setType(typesHelper.getSSNType(SSNType.CODE_ACTIVE.getCode()));
		 }
		
		 pidt.setSsn(ssn);
		 person.addSsn(ssn);
		 //Gender
		 if(personInfo.getGender() != null){
			 person.setGender(typesHelper.getGender(personInfo.getGender()));
			 pidt.setGender(typesHelper.getGender(personInfo.getGender()));
		 }
		 
		 //CCR13311  remove veteran other name for 2014 OMB approved form
/*		 if(personInfo.getNames() != null && personInfo.getNames().getName() != null){
		 List<NameInfo> nameList = personInfo.getNames().getName();
		 Iterator nameIter = nameList.iterator();
			while (nameIter.hasNext()) {
				 NameInfo nInfo = (NameInfo)nameIter.next();
				 Name otherName = new Name();
				 if(nInfo.getOtherName() != null){
				 otherName.setFamilyName(nInfo.getOtherName());
				 otherName.setType(this.typesHelper.getNameType(NameType.ALIAS_NAME.getCode()));
				 }
				person.setAliasName(otherName);
				pidt.addName(otherName);
			}
		 } */
  }
 // Birth record
		 
		 BirthRecord birthRecord = person.getBirthRecord();
		 if(birthRecord == null ) birthRecord = new BirthRecord();
		 if(personInfo.getPlaceOfBirthCity() != null){
			 birthRecord.setCity(personInfo.getPlaceOfBirthCity());
			 pidt.setPlaceOfBirthCity(personInfo.getPlaceOfBirthCity());
		 }
		 if(personInfo.getPlaceOfBirthState() != null){
			 birthRecord.setState(personInfo.getPlaceOfBirthState());
			 pidt.setPlaceOfBirthState(personInfo.getPlaceOfBirthState());
		 }
		 if(personInfo.getDob() != null){
			 ImpreciseDate id = new ImpreciseDate(buildHelper.stringToDate(personInfo.getDob()));
			 birthRecord.setBirthDate(id);
		 }
		 person.setBirthRecord(birthRecord);
		 pidt.setBirthRecord(birthRecord);
		 //Death record  -- not required
		/*
		if( submitFormRequest.getForm().getSummary().getDeathRecond() != null){
			DeathRecord dr = new DeathRecord();
			dr.setDataSource(typesHelper.getDeathDataSource(submitFormRequest.getForm().getSummary().getDeathRecond().getDataSource()));
			if (submitFormRequest.getForm().getSummary().getDeathRecond().getDeathDate() != null){
			 dr.setDeathDate(new ImpreciseDate(submitFormRequest.getForm().getSummary().getDeathRecond().getDeathDate()));
			}
			person.setDeathRecord(dr);
		}
		*/
		//Marital status
		 
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().getMaritalStatus() != null){
			person.setMaritalStatus(this.typesHelper.getMaritalStatus(submitFormRequest.getForm().getSummary().getDemographics().getMaritalStatus()));
		}
		
		 //claim folder number  -- not required
		/*
		if(submitFormRequest.getForm().getSummary().getDemographics().getClaimFolderNumber() != null)
		 person.setClaimFolderNumber(submitFormRequest.getForm().getSummary().getDemographics().getClaimFolderNumber());
		 //appointmentRequestResponse
		  * 
		  */
	
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().isAppointmentRequestResponse() != null)
		 person.setAppointmentRequestResponse(submitFormRequest.getForm().getSummary().getDemographics().isAppointmentRequestResponse());
		
		 //ethinicity
		 if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().getEthnicity() != null){
			Ethnicity et = new Ethnicity();
			et.setEthnicityType(this.typesHelper.getEthnicityType(submitFormRequest.getForm().getSummary().getDemographics().getEthnicity()));
			person.setEthnicity(et);
			
		}
		
		 // Relogion
		 
		 if(submitFormRequest.getForm().getSummary().getDemographics() != null &&
				 submitFormRequest.getForm().getSummary().getDemographics().getReligion() != null && !submitFormRequest.getForm().getSummary().getDemographics().getReligion().isEmpty()){
		 	person.setReligion(this.typesHelper.getReligion(submitFormRequest.getForm().getSummary().getDemographics().getReligion()));
		}
		 //Races
		if( submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().getRaces() != null){
		   List<String> races = submitFormRequest.getForm().getSummary().getDemographics().getRaces().getRace();
		   
		if (races != null) {
			Iterator raceIter = races.iterator();
			while (raceIter.hasNext()) {
				String racestring = (String) raceIter.next();
				Race race = new Race();
				race.setRaceType(this.typesHelper.getRaceType(racestring));
				person.addRace(race);
			}
		}
		}
		//  preferred facility
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().getPreferredFacility() != null){
				
				PreferredFacility pf = new PreferredFacility();
				pf.setFacility(this.typesHelper.getVAFacility(submitFormRequest.getForm().getSummary().getDemographics().getPreferredFacility()));
				person.addPreferredFacility(pf);
			 
		}
		
		//CCR12789: Affordable Care Act acaIndicator
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().isAcaIndicator() != null){
			//TODO: waiting for person object to add the aca indicator method by other team
			//person.setAcaIndicator(submitFormRequest.getForm().getSummary().getDemographics().isAcaIndicator());
		}
		
		
		//permanent address
		Address address = person.getPermanentAddress();
		if(address == null) {
			address = new Address();
		}
		if(submitFormRequest.getForm().getSummary().getDemographics()!= null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo() != null && 
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getAddresses()!= null){
		List<AddressInfo> addressList = submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getAddresses().getAddress();
		Iterator addressIter = addressList.iterator();
		while (addressIter.hasNext()) {
		AddressInfo addressInfo = (AddressInfo)addressIter.next();
			   address = buildHelper.buildAddressInfo(addressInfo);
				person.addAddress(address);
				pidt.addAddress(address);
				
			}
		
		}
		//Phone
		if(submitFormRequest.getForm().getSummary().getDemographics() != null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo() != null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getPhones() != null){
			
		List<PhoneInfo> phoneList = submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getPhones().getPhone();
		Iterator phoneIter = phoneList.iterator();
		while (phoneIter.hasNext()) {
			PhoneInfo phInfo = (PhoneInfo) phoneIter.next();
			Phone phone = new Phone();
			if(phInfo != null ) {
//				if(phInfo.getType() != null)phone.setType(typesHelper.getPhoneType(phInfo.getType()));
				if(phInfo.getType() != null) {
					phone.setType(typesHelper.getPhoneType(phInfo.getType()));
				} else {
					phone.setType(typesHelper.getPhoneType("1"));
				}				
				if(phInfo.getPhoneNumber() != null)phone.setPhoneNumber(phInfo.getPhoneNumber());
				person.addPhone(phone);
			}

		}
		}
		//Email address
		if(submitFormRequest.getForm().getSummary().getDemographics() != null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo()!= null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getEmails() != null){
		List<EmailInfo> emailList = submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getEmails().getEmail();
		if(emailList != null){
		Iterator emailIter = emailList.iterator();
		while (emailIter.hasNext()) {
			EmailInfo emInfo = (EmailInfo) emailIter.next();
			Email email = new Email();
			if(emInfo != null) {	
//				if(emInfo.getType() != null )email.setType(typesHelper.getEmailType(emInfo.getType()));
				if(emInfo.getType() != null ) {
					email.setType(typesHelper.getEmailType(emInfo.getType()));
				} else {
					//set email type as Personal
					email.setType(typesHelper.getEmailType("1"));
				}				
				if(emInfo.getAddress() != null)email.setAddress(emInfo.getAddress());
				person.addEmail(email);
			}
				
		}
		}
		}
		//Association
		if(submitFormRequest.getForm().getSummary().getAssociations() != null &&
				submitFormRequest.getForm().getSummary().getAssociations().getAssociation() !=null){
		List<AssociationInfo> associationList = submitFormRequest.getForm().getSummary().getAssociations().getAssociation();
		Iterator associationIter = associationList.iterator();
		while (associationIter.hasNext()) {
			AssociationInfo asInfo = (AssociationInfo) associationIter.next();
			Association association = new Association();
			if(asInfo.getGivenName() != null)association.setFirstName(asInfo.getGivenName());
			if(asInfo.getFamilyName() != null)association.setLastName(asInfo.getFamilyName());
			if(asInfo.getRelationship() != null)association.setRelationship(asInfo.getRelationship());
			if(asInfo.getPrimaryPhone() != null)association.setPrimaryPhone(asInfo.getPrimaryPhone());
			if(asInfo.getAlternatePhone() != null)association.setAlternatePhone(asInfo.getAlternatePhone());
			if(asInfo.getContactType()!= null){
				association.setType(this.typesHelper.getAssociationType(asInfo.getContactType()));
			}
			
			if(asInfo.getAddress() != null){
			Address ad = new Address();
			BaseAddressInfo bsd = asInfo.getAddress();
			if(bsd !=null){
			if(bsd.getLine1() != null)ad.setLine1(bsd.getLine1());
			if(bsd.getLine2() != null)ad.setLine2(bsd.getLine2());
			if(bsd.getLine3() != null)ad.setLine3(bsd.getLine3());
			if(bsd.getCity() != null)ad.setCity(bsd.getCity());
			if(bsd.getCounty() != null)ad.setCounty(bsd.getCounty());
			if(bsd.getState() != null)ad.setState(bsd.getState());
			if(bsd.getZipCode() != null)ad.setZipCode(bsd.getZipCode());
			if(bsd.getZipPlus4() != null)ad.setZipPlus4(bsd.getZipPlus4());
			association.setAddress(ad);
			}
			}
			
			if(association != null) person.addAssociation(association);
			
		}
		}
		//person.addAllAssociations(associationSet);
		// Employment Info
		if(submitFormRequest.getForm().getSummary().getEmploymentInfo() != null){
		Employment employment = new Employment();  //person.getEmployment();
		 EmploymentInfo employmentInfo = (EmploymentInfo)submitFormRequest.getForm().getSummary().getEmploymentInfo();
		 if(employmentInfo != null) {
			 employment= buildHelper.buildEmployment(employmentInfo);
			 person.setEmployment(employment);
		 }
		}
		
		// identity traits
		// person.setIdentityTraits(pidt);
		 //Application
		 buildApplication(person,submitFormRequest);
		 //insurance
		 buildInsurance(person,submitFormRequest);

	}
	@SuppressWarnings("rawtypes")
	private void buildApplication(Person person,SubmitFormRequest submitFormRequest){
		
		if(submitFormRequest.getForm().getApplications().getApplicationInfo() != null && 
					submitFormRequest.getForm().getApplications().getApplicationInfo() != null){
		List<ApplicationInfo> appList = submitFormRequest.getForm().getApplications().getApplicationInfo();
		 Iterator itr =  appList.iterator();
			while (itr != null && itr.hasNext()) {
				ApplicationInfo applicationInfo = (ApplicationInfo)itr.next();
				Application application = new Application();
				
				if(applicationInfo != null ){
					if(applicationInfo.getAppMethod() != null)
					  application.setAppliactionMethod(this.typesHelper.getApplicationMethod("1"));
					  //application.setAppliactionMethod(this.typesHelper.getApplicationMethod("1"));
					 if(applicationInfo.getAppDate() != null)application.setApplicationDate(buildHelper.getDateXMLtoJava(applicationInfo.getAppDate()));
					 if(applicationInfo.getBenefitType() != null)
					 application.setBenefitType(this.typesHelper.getBenefitType(applicationInfo.getBenefitType()));
				}
				person.setApplication(application);
				
			}
	}
		 
 }
	@SuppressWarnings("rawtypes")
	private  void buildInsurance(Person person,SubmitFormRequest submitFormRequest) {
		//
		if(submitFormRequest.getForm().getSummary().getInsuranceList() != null && 
				submitFormRequest.getForm().getSummary().getInsuranceList().getInsurance() != null) {
		List<InsuranceInfo> insuranceList = submitFormRequest.getForm().getSummary().getInsuranceList().getInsurance();
		Iterator itr =  insuranceList.iterator();
		while (itr != null && itr.hasNext()) {
			InsuranceInfo insuranceInfo = (InsuranceInfo)itr.next();
			//InsurancePolicy insurance = (InsurancePolicy) itr.next();
			//InsuranceInfo insuranceInfo = new InsuranceInfo();
			InsurancePolicy insurance = null;
			
			if(insuranceInfo.getInsuranceMappingTypeName() != null ){
				//CCR 13311 remove medicare partB
				if( insuranceInfo.getInsuranceMappingTypeName().equalsIgnoreCase("MDCR") && 
						//CCR13302 make sure either PartA or PartB is true before creating Medicare object
						((insuranceInfo.isEnrolledInPartA() != null && insuranceInfo.isEnrolledInPartA()))) {/* ||	
						 (insuranceInfo.isEnrolledInPartB() != null && insuranceInfo.isEnrolledInPartB())) ){ */
					insurance = new Medicare();	
					
				}
			
				if(	insuranceInfo.getInsuranceMappingTypeName().equalsIgnoreCase("PI")){
					insurance = new PrivateInsurance();
					
				}
			}
		if(insurance != null){
			if(insuranceInfo.getCompanyName() != null)insurance.setCompanyName(insuranceInfo.getCompanyName());
			if(insuranceInfo.getGroupNumber() != null)insurance.setGroupNumber(insuranceInfo.getGroupNumber());
			if(insuranceInfo.getGroupName() != null)insurance.setGroupName(insuranceInfo.getGroupName());
			InsuranceSubscriber insubr = insurance.getSubscriber();
			if(insubr == null ) insubr = new InsuranceSubscriber();
			if(insuranceInfo.getPolicyHolderName() != null) insubr.setName(insuranceInfo.getPolicyHolderName());
			insurance.setSubscriber(insubr);
		   
			AddressInfo addressInfo = insuranceInfo.getInsAddress();
			if(addressInfo != null)
			insurance.setAddress(buildHelper.buildAddressInfo(addressInfo));
			//  There is  no phone number list for voa. phone is part of address.
			if(insuranceInfo.getInsurancePhones() != null && insuranceInfo.getInsurancePhones().getPhone() != null){
			List<PhoneInfo> phoneList = insuranceInfo.getInsurancePhones().getPhone();
			Iterator phoneItr = phoneList.iterator();
			
			while ( phoneItr.hasNext())
			{
				
				PhoneInfo pInfo = (PhoneInfo)phoneItr.next();
				Phone phone = new Phone();
				if(pInfo.getPhoneNumber() != null)phone.setPhoneNumber(pInfo.getPhoneNumber());
				
				
				if ( pInfo.getType() != null)
				{
					
					if(pInfo.getType().equalsIgnoreCase("2") || pInfo.getType().equalsIgnoreCase("6")){
					phone.setType(typesHelper.getPhoneType("2"));
					insurance.setBusinessPhone(phone);
					}
					
					if(pInfo.getType().equalsIgnoreCase("3")){
						phone.setType(typesHelper.getPhoneType(pInfo.getType()));
						insurance.setFax(phone);
						}
					if(pInfo.getType().equalsIgnoreCase("7")){
						phone.setType(typesHelper.getPhoneType(pInfo.getType()));
						insurance.setPreCertificationPhone(phone);
						}
				}
				
			}
			}
			
			if (insurance instanceof PrivateInsurance) {
				
				if ( insuranceInfo.getPolicyEffectiveDate() != null)
					((PrivateInsurance) insurance).setPolicyEffectiveDate(new ImpreciseDate(insuranceInfo.getPolicyEffectiveDate()));
				
				if ( insuranceInfo.getPolicyExpirationDate() != null)
					((PrivateInsurance) insurance).setPolicyExpirationDate(new ImpreciseDate(insuranceInfo.getPolicyExpirationDate()));
				if(insuranceInfo.getPolicyNumber() != null)((PrivateInsurance) insurance).setPolicyNumber(insuranceInfo.getPolicyNumber());
			//
				person.addInsurance(insurance);
				
				
			} else if (insurance instanceof Medicare) {
					if(insuranceInfo.isEnrolledInPartA() != null){
						
					//12094 - still set the indicator if it is false, and only set details if the indicator is true
					((Medicare) insurance).setEnrolledInPartA(insuranceInfo.isEnrolledInPartA());
					if (insuranceInfo.isEnrolledInPartA()) 
					{
						if(insuranceInfo.getPartAEffectiveDate() != null)
							((Medicare) insurance).setPartAEffectiveDate(new ImpreciseDate(insuranceInfo.getPartAEffectiveDate()));
						if(insuranceInfo.getSubscriber() != null) ((Medicare) insurance).setNameOnMedicareCard(insuranceInfo.getSubscriber());
						if(insuranceInfo.getPolicyNumber() != null)((Medicare) insurance).setMedicareClaimNumber(insuranceInfo.getPolicyNumber());
					}
					//part A and Part B true. then two medicare objects shouls be added
					// if part a is true and part b is flase then one medicare object
					// Part b never be true without part a true.
					
					//12094 - still set the indicator if it is false, and only set details if the indicator is true
					//CCR 13311 remove medicare partB
/*					if(insuranceInfo.isEnrolledInPartB() != null ){
						//insuranceB = insurance;
						((Medicare) insurance).setEnrolledInPartB(insuranceInfo.isEnrolledInPartB());
						if (insuranceInfo.isEnrolledInPartB())
						{
							if( insuranceInfo.getPartBEffectiveDate() != null)
							((Medicare) insurance).setPartBEffectiveDate(new ImpreciseDate(insuranceInfo.getPartBEffectiveDate()));
						}
						
					}*/
					person.addInsurance(insurance);
					
				}
					
			}
			//person.addInsurance(insurance); //insuranceSet.add(insurance);
			
		}	// insurance
		}
		//person.addAllInsurances(insuranceSet);
	}	
  }
	
	/**
	 * RTC Task 236652  - auxiliary method to handle the case when veteran selects No for the 
	 * disclose financial details from the VOA UI
	 * @return a partial person with identity traits and ICN
	 */
    @SuppressWarnings("rawtypes")
	public Person getPartialPerson(SubmitFormRequest submitFormRequest){
		
		 PersonInfo personInfo = submitFormRequest.getForm().getSummary().getPersonInfo();
		 Person person = new Person();
		 VeteranIdentifier vid = submitFormRequest.getIdentity().getVeteranIdentifier();
		 PersonIdentityTraits pidt = new PersonIdentityTraits();
		 
		 Name name = new Name();
		 if(personInfo != null){
			 if(personInfo.getLastName() != null)
				 name.setFamilyName(personInfo.getLastName());
			 if(personInfo.getFirstName() != null)
				 name.setGivenName(personInfo.getFirstName());
			 // As per the discussion, the nametype is set as legal name. VOA won't send the name type
			 name.setType(this.typesHelper.getNameType(NameType.LEGAL_NAME.getCode()));
			 if(personInfo.getMiddleName() != null)
				 name.setMiddleName(personInfo.getMiddleName());
			 // if(personInfo.getPrefix()!=null)name.setPrefix(personInfo.getPrefix());
		 
			 if(personInfo.getSuffix() != null)name.setSuffix(personInfo.getSuffix());
			 person.addName(name);
			 pidt.addName(name);
			 if(personInfo.getMothersMaidenName() != null){
				 person.setMothersMaidenName(personInfo.getMothersMaidenName());
				 pidt.setMothersMaidenName(personInfo.getMothersMaidenName());
			}
			 if (vid != null) {
				 PersonVPID vpid = new PersonVPID(vid.getValue());
				 person.setVPID(vpid);
			 }
			 SSN ssn = new SSN();
			 if(personInfo.getSsnText() != null){
				 ssn.setSsnText(personInfo.getSsnText());
				 //ssntype is active default
				 ssn.setType(typesHelper.getSSNType(SSNType.CODE_ACTIVE.getCode()));
			 }
		
			 pidt.setSsn(ssn);
			 person.addSsn(ssn);
			 //Gender
			 if(personInfo.getGender() != null){
				 person.setGender(typesHelper.getGender(personInfo.getGender()));
				 pidt.setGender(typesHelper.getGender(personInfo.getGender()));
			 }
		 }
		 // Birth record		 
		 BirthRecord birthRecord = person.getBirthRecord();
		 if(birthRecord == null ) birthRecord = new BirthRecord();
		 if(personInfo.getPlaceOfBirthCity() != null){
			 birthRecord.setCity(personInfo.getPlaceOfBirthCity());
			 pidt.setPlaceOfBirthCity(personInfo.getPlaceOfBirthCity());
		 }
		 if(personInfo.getPlaceOfBirthState() != null){
			 birthRecord.setState(personInfo.getPlaceOfBirthState());
			 pidt.setPlaceOfBirthState(personInfo.getPlaceOfBirthState());
		 }
		 if(personInfo.getDob() != null){
			 ImpreciseDate id = new ImpreciseDate(buildHelper.stringToDate(personInfo.getDob()));
			 birthRecord.setBirthDate(id);
		 }
		 person.setBirthRecord(birthRecord);
		 pidt.setBirthRecord(birthRecord);
		
		//Marital status		 
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().getMaritalStatus() != null){
			person.setMaritalStatus(this.typesHelper.getMaritalStatus(submitFormRequest.getForm().getSummary().getDemographics().getMaritalStatus()));
		}
		
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().isAppointmentRequestResponse() != null)
		 person.setAppointmentRequestResponse(submitFormRequest.getForm().getSummary().getDemographics().isAppointmentRequestResponse());
		
		//  preferred facility
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().getPreferredFacility() != null){
				
				PreferredFacility pf = new PreferredFacility();
				pf.setFacility(this.typesHelper.getVAFacility(submitFormRequest.getForm().getSummary().getDemographics().getPreferredFacility()));
				person.addPreferredFacility(pf);
			 
		}
				
		//permanent address
		Address address = person.getPermanentAddress();
		if(address == null) {
			address = new Address();
		}
		if(submitFormRequest.getForm().getSummary().getDemographics()!= null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo() != null && 
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getAddresses()!= null){
		List<AddressInfo> addressList = submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getAddresses().getAddress();
		Iterator addressIter = addressList.iterator();
		while (addressIter.hasNext()) {
		AddressInfo addressInfo = (AddressInfo)addressIter.next();
			   address = buildHelper.buildAddressInfo(addressInfo);
				person.addAddress(address);
				pidt.addAddress(address);
				
			}
		
		}
		//Phone
		if(submitFormRequest.getForm().getSummary().getDemographics() != null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo() != null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getPhones() != null){
			
		List<PhoneInfo> phoneList = submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getPhones().getPhone();
		Iterator phoneIter = phoneList.iterator();
		while (phoneIter.hasNext()) {
			PhoneInfo phInfo = (PhoneInfo) phoneIter.next();
			Phone phone = new Phone();
			if(phInfo != null ) {
				//if(phInfo.getType() != null)phone.setType(typesHelper.getPhoneType(phInfo.getType()));
				if(phInfo.getType() != null) {
					phone.setType(typesHelper.getPhoneType(phInfo.getType()));
				} else {
					phone.setType(typesHelper.getPhoneType("1"));
				}
				if(phInfo.getPhoneNumber() != null)phone.setPhoneNumber(phInfo.getPhoneNumber());
				person.addPhone(phone);
			}

		}
		}
		//Email address
		if(submitFormRequest.getForm().getSummary().getDemographics() != null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo()!= null &&
				submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getEmails() != null){
		List<EmailInfo> emailList = submitFormRequest.getForm().getSummary().getDemographics().getContactInfo().getEmails().getEmail();
		if(emailList != null){
		Iterator emailIter = emailList.iterator();
		while (emailIter.hasNext()) {
			EmailInfo emInfo = (EmailInfo) emailIter.next();
			Email email = new Email();
			if(emInfo != null) {	
				//if(emInfo.getType() != null )email.setType(typesHelper.getEmailType(emInfo.getType()));
				if(emInfo.getType() != null ) {
					email.setType(typesHelper.getEmailType(emInfo.getType()));
				} else {
					//set email type as Personal
					email.setType(typesHelper.getEmailType("1"));
				}
				if(emInfo.getAddress() != null)email.setAddress(emInfo.getAddress());
				person.addEmail(email);
			}
				
		}
	   }
	 }
		return person;
	}	
		
}
