/**
 * 
 */
package gov.va.med.esr.voa.webservice.builder.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import gov.va.med.esr.common.infra.ImpreciseDate;
import gov.va.med.esr.common.model.financials.Asset;
import gov.va.med.esr.common.model.financials.DependentFinancials;
import gov.va.med.esr.common.model.financials.Expense;
import gov.va.med.esr.common.model.financials.FinancialStatement;
import gov.va.med.esr.common.model.financials.Income;
import gov.va.med.esr.common.model.financials.IncomeTest;
import gov.va.med.esr.common.model.financials.SpouseFinancials;
import gov.va.med.esr.common.model.lookup.IncomeTestSource;
import gov.va.med.esr.common.model.lookup.SSNType;
import gov.va.med.esr.common.model.lookup.VAFacility;
import gov.va.med.esr.common.model.party.Address;
import gov.va.med.esr.common.model.person.Dependent;
import gov.va.med.esr.common.model.person.Name;
import gov.va.med.esr.common.model.person.Person;
import gov.va.med.esr.common.model.person.SSN;
import gov.va.med.esr.common.model.person.Spouse;
import gov.va.med.esr.common.model.person.VOAApplication;
import gov.va.med.esr.voa.webservice.AddressInfo;
import gov.va.med.esr.voa.webservice.AssetInfo;
import gov.va.med.esr.voa.webservice.DependentFinancialsInfo;
import gov.va.med.esr.voa.webservice.DependentInfo;
import gov.va.med.esr.voa.webservice.ExpenseInfo;
import gov.va.med.esr.voa.webservice.FinancialStatementInfo;
import gov.va.med.esr.voa.webservice.FinancialsInfo;
import gov.va.med.esr.voa.webservice.IncomeInfo;
import gov.va.med.esr.voa.webservice.IncomeTestInfo;
import gov.va.med.esr.voa.webservice.SpouseFinancialsInfo;
import gov.va.med.esr.voa.webservice.SpouseInfo;
import gov.va.med.esr.voa.webservice.SsnInfo;
import gov.va.med.esr.voa.webservice.SubmitFormRequest;
import gov.va.med.esr.voa.webservice.builder.DemographicsBuilderFactory;
import gov.va.med.esr.voa.webservice.builder.FinancialsBuilderFactory;
import gov.va.med.esr.voa.webservice.AuthenticationLevel;
import gov.va.med.esr.voa.webservice.PersonInfo;
import gov.va.med.esr.voa.webservice.VeteranIdentifier;
import gov.va.med.fw.service.AbstractComponent;
import gov.va.med.fw.service.ServiceException;
import gov.va.med.esr.service.VOAApplicationService;


/**
 * @author DNS   CHINNR
 *
 */
public class FinancialsBuilderFactoryImpl extends AbstractComponent implements FinancialsBuilderFactory  {
	
	private TypesHelper typesHelper = new TypesHelper();
	private BuildHelper buildHelper = new BuildHelper();
	private DemographicsBuilderFactory demographicsBuilderFactory = new DemographicsBuilderFactoryImpl();
	
	
	/**
	 * RTC Task 236652  - When a veteran answers no to the disclose financial details option the VOA form, ES/VOA 
	 * backend sets some default fields to the IncomeTest so that Means Test does not go into the Pending status. 
	 * included VOAApplicationService in the method signature
	 */
	public void buildFinancialServices(Person person, SubmitFormRequest submitFormRequest, VOAApplicationService voaApplicationService){
		buildFinancials(person,submitFormRequest, voaApplicationService);
		
	}
	
	/**
	 * * RTC Task 236652  - When a veteran answers no to the disclose financial details option the VOA form, ES/VOA 
	 * backend sets some default fields to the IncomeTest so that Means Test does not go into the Pending status. 
	 * Added a case to handle the scenario for no incomeTest passed by VOA.
	 */
	public void buildFinancials(Person person, SubmitFormRequest submitFormRequest, VOAApplicationService voaApplicationService) {	
		if (person != null){
			getIncomeTestInfo(person, submitFormRequest, voaApplicationService);
			//Prepare the financial statement for the current income year.
			getFinancialStatement(person, submitFormRequest);			
		}
	}
		
	private  void getIncomeTestInfo(Person person, SubmitFormRequest submitFormRequest, VOAApplicationService voaApplicationService){	
		
		IncomeTest currIncomeTest = new IncomeTest();
		
		/**
		 * RTC Task 236652  - veteran selects No for the disclose financial details from the VOA UI
		 * FinancialStatement tag comes as null and so no incomeTest
		 */
		
		FinancialsInfo financialsInfo = submitFormRequest.getForm().getSummary().getFinancialsInfo();
		if(financialsInfo == null){ 		
			
			logger.info("incomeTestInfo - isDisclosed = No");
			logger.info("incomeTestInfo - Agrees to pay deductible = Yes");
				
			//takes the form and convert to a partial voapplication DTO
			VOAApplication voaApplicationDTO = getVoaApplicationDTO(submitFormRequest);	
			//logger.info("FQID in voa submission: " + voaApplicationDTO.getFullQualifiedId());
						
			Person incomingPerson = demographicsBuilderFactory.getPartialPerson(submitFormRequest);
			Person adrPerson = null;
			
			if (voaApplicationService == null){
				adrPerson = null;
			}
			//check if the incoming person is coming from Anonymous or authenticated VOA
			//retrieve the person if already exists in ES/ADR
			if (voaApplicationService.isDSLogonLevel2(voaApplicationDTO)){  //veteran comes in with DSLogon2 or higher credentials
				try {
					if (logger.isDebugEnabled() && voaApplicationDTO != null){
						logger.debug("Logged in with DSlogon 2 or higher credentials");
						logger.debug("fqid is: " + voaApplicationDTO.getFullQualifiedId());
					}
					adrPerson = voaApplicationService.getOnFilePersonForPersonWithDSLogon(voaApplicationDTO, incomingPerson);
					
					if (logger.isDebugEnabled() && adrPerson != null){
						logger.debug("ds logon user details: " + adrPerson.toString());
					}
					
				} catch (ServiceException e) {
					logger.error("error occurred while trying to retirve person with DS logon from ES/MVI"); 
					adrPerson = null;
				}				
			}
			else {  //veteran comes in as an anonymous user
				
				try {
					logger.info("getting anonymous person from ES/MVI via extended srch");
					adrPerson = voaApplicationService.getOnFilePersonPersonForAnonymousPerson(voaApplicationDTO, incomingPerson);
				
					if (logger.isDebugEnabled() && adrPerson != null){
						logger.debug("anonymous user details: " + adrPerson.toString());
					}
					
				} catch (ServiceException e) {
					logger.error("error occurred while trying to retirve anon person from ES/MVI"); 
					adrPerson = null;
				}				
			}
								
			//get the incomeTest for the ES/adrPerson for the current Income Year
			IncomeTest onFileIncomeTest = null;
			if (adrPerson != null){ //person is in ES/ADR
				onFileIncomeTest = adrPerson.getIncomeTest(typesHelper.getIncomeYear());				
			}
			
			//only set all the required fields when no incomeTest exists.  If income test already exists, no need to make any changes
			//If the person does not exist in ADR/ES or no income test exists, then just set the default fields 
			if (adrPerson == null || onFileIncomeTest == null) {  
				if (logger.isDebugEnabled()){
					logger.debug("no income test exists for current income year or person does not exist");								
				}
				
				currIncomeTest.setDiscloseFinancialInformation(Boolean.FALSE);  //NO
				currIncomeTest.setAgreesToPayDeductible(Boolean.TRUE);
				
				//Set HEC as facility 
				currIncomeTest.setSiteConductingTest(typesHelper.getVAFacility(VAFacility.CODE_HEC.getCode()));
							
				//Set HEC as Source of financial info 						
				currIncomeTest.setSource(typesHelper.getIncomeTestSource(IncomeTestSource.CODE_HEC.getCode()));				
				
				//set MT as primary test as in the FinancialInputParameter class
				//currIncomeTest.setType(typesHelper.getIncomeTestType(IncomeTestType.CODE_MEANS_TEST.getCode()));
				
				//sets today's date as the completed and effective dates
				Date rightNow = Calendar.getInstance().getTime();
				currIncomeTest.setCompletedDate(rightNow);
				
                //CR14145 ( RTCdefect 193158) - set MT effective date to application date rather  
                //                              than process date
                
                //currIncomeTest.setEffectiveDate(rightNow);
                Date applicationDate = null;
                if(person != null && person.getApplication() != null) {
                   applicationDate = person.getApplication().getApplicationDate();
                }

                if(applicationDate != null){
                    currIncomeTest.setEffectiveDate(applicationDate);
                }
                else{
                    currIncomeTest.setEffectiveDate(rightNow);
                }
                
				/** 
				 * no income year from the FinacialStatement element as it is empty, 
				 * so set the incomeTest for the default income year	
				 */
				
				 currIncomeTest.setIncomeYear(typesHelper.getIncomeYear());
				 person.setIncomeTest(typesHelper.getIncomeYear(), currIncomeTest);		
				
				 
			}
		}			
		else if (submitFormRequest.getForm().getSummary().getFinancialsInfo().getIncomeTest()!= null) {
			IncomeTestInfo  incomeTestInfo = submitFormRequest.getForm().getSummary().getFinancialsInfo().getIncomeTest();
			
			if(incomeTestInfo != null){
				/*if(incomeTestInfo.isDiscloseFinancialInformation() != null){
					currIncomeTest.setDiscloseFinancialInformation(incomeTestInfo.isDiscloseFinancialInformation());  //always sets to True as VOA only sends when dis = yes
				}*/
				
				//RTC 236652 - setting defaults 
				currIncomeTest.setDiscloseFinancialInformation(Boolean.TRUE); 
				currIncomeTest.setAgreesToPayDeductible(Boolean.TRUE);
								
				if(incomeTestInfo.getTotalIncome() != null){
					currIncomeTest.setTotalIncome(incomeTestInfo.getTotalIncome());
				}
				if(incomeTestInfo.getNetIncome() != null){
					currIncomeTest.setNetIncome(incomeTestInfo.getNetIncome());
				}
				if (incomeTestInfo.getIncomeYear() != null){
					currIncomeTest.setIncomeYear(incomeTestInfo.getIncomeYear());
					person.setIncomeTest(incomeTestInfo.getIncomeYear(), currIncomeTest);
					
				}
				else {
					currIncomeTest.setIncomeYear(typesHelper.getIncomeYear());
					person.setIncomeTest(typesHelper.getIncomeYear(), currIncomeTest);
				}
			}
			
		}		
	}
	
	@SuppressWarnings("rawtypes")
	private  void getFinancialStatement(Person person, SubmitFormRequest submitFormRequest) {

		if(submitFormRequest.getForm().getSummary().getFinancialsInfo() != null &&
					submitFormRequest.getForm().getSummary().getFinancialsInfo().getFinancialStatement()!= null){
				
				FinancialStatementInfo fStatementInfo = submitFormRequest.getForm().getSummary().getFinancialsInfo().getFinancialStatement();
				FinancialStatement statement = new FinancialStatement();
				
				if (fStatementInfo != null) {
					
					if(fStatementInfo.isMarriedLastCalendarYear() != null)
						statement.setMarriedLastCalendarYear(fStatementInfo.isMarriedLastCalendarYear());
					
									
					if(fStatementInfo.getNumberOfDependentChildren() != null)
						statement.setNumberOfDependentChildren(fStatementInfo
							.getNumberOfDependentChildren());
	
					if(fStatementInfo.isContributedToSpouse() != null)
						statement.setContributedToSpouseInd( fStatementInfo.isContributedToSpouse());	
								
					getDependentFinInfo(statement, fStatementInfo);
					
					//income
					if(fStatementInfo.getIncomes() != null && 
							fStatementInfo.getIncomes().getIncome() != null){
					List<IncomeInfo> incomeList = fStatementInfo.getIncomes().getIncome();
					if (incomeList != null) {
						Iterator incomeIter = incomeList.iterator();
						while (incomeIter.hasNext()) {
							IncomeInfo incomeinfo = (IncomeInfo) incomeIter.next();
							if (incomeinfo != null && incomeinfo.getType() != null) {
								Income income = buildHelper.buildIncomeInfo(incomeinfo);
								statement.addIncome(this.typesHelper.getIncomeType(incomeinfo.getType()), income);
							}
						}
					}
				}
					
					//asset		
				if(fStatementInfo.getAssets() != null && 
						fStatementInfo.getAssets().getAsset() != null){
					List<AssetInfo> assetList = fStatementInfo.getAssets().getAsset();
					
					if (assetList != null) {
						Iterator assetIter = assetList.iterator();
						while (assetIter.hasNext()) {
							AssetInfo assetInfo = (AssetInfo)assetIter.next();
							if(assetInfo != null && assetInfo.getAssetType() != null){
								Asset asset = buildHelper.buildAssetInfo(assetInfo);
								statement.addAsset(this.typesHelper.getAssetType(assetInfo.getAssetType()), asset);
								}
						}
					}
				}
					
					//expense
			 if(fStatementInfo.getExpenses() != null && 
					 fStatementInfo.getExpenses().getExpense() != null){
					List<ExpenseInfo> expenseList = fStatementInfo.getExpenses().getExpense();
					if (expenseList != null) {
						Iterator expenseIter = expenseList.iterator();
						
						while (expenseIter.hasNext()) {
							ExpenseInfo expenseInfo = (ExpenseInfo) expenseIter.next();
							if (expenseInfo != null) {
								Expense expense = buildHelper.buildExpenseInfo(expenseInfo);
								statement.addExpense(this.typesHelper.getExpenseType(expenseInfo.getExpenseType()), expense);
							}
						}
					}
			 }	
					
			
			 //if(submitFormRequest.getForm().getSummary().getFinancialsInfo().getFinancialStatement().getIncomeYear() != null){
			 //VOA UI does not send any income Year, so need to default to the current Income Year
			 Integer incomeYear = null;
			 if (fStatementInfo.getIncomeYear() == null){
				 incomeYear = typesHelper.getIncomeYear();
			 }
			 else 
				 incomeYear = fStatementInfo.getIncomeYear();
				
			 //calculate spouse financials
			 statement.setIncomeYear(incomeYear);	
			 getSpouseFinInfo(statement,fStatementInfo);

			}
				
			if (fStatementInfo.getIncomeYear() != null){
				person.setFinancialStatement(fStatementInfo.getIncomeYear(), statement);
			} else{
				person.setFinancialStatement(typesHelper.getIncomeYear(), statement);
			}
		}
		else {
			//RTC 236652- default blank statement added when non-disclosure = Yes
			FinancialStatement statement = new FinancialStatement();
			statement.setIncomeYear(typesHelper.getIncomeYear());
			person.setFinancialStatement(typesHelper.getIncomeYear(), statement);			
			
		}
	}

	@SuppressWarnings("rawtypes")
	private  void getDependentFinInfo(FinancialStatement statement, FinancialStatementInfo fStatementInfo) {
	
	 if(fStatementInfo.getDependentFinancialsList() != null && 
				fStatementInfo.getDependentFinancialsList().getDependentFinancials() != null){
		List<DependentFinancialsInfo> dependentFinanList = fStatementInfo.getDependentFinancialsList().getDependentFinancials();
		
		Iterator itr = dependentFinanList != null ? dependentFinanList.iterator()
				: null;
		
		while (itr != null && itr.hasNext()) {
			DependentFinancialsInfo dependentFinInfo = (DependentFinancialsInfo) itr
					.next();
			DependentFinancials dependentFinancials = new DependentFinancials();
			
			if (dependentFinInfo != null && dependentFinInfo.getDependentInfo() != null)
			{
				dependentFinancials.setReportedOn(buildDependent(dependentFinInfo.getDependentInfo()));
				
			}
			
			if(dependentFinInfo.isAttendedSchool() != null)dependentFinancials.setAttendedSchool(dependentFinInfo.isAttendedSchool());
					
			if(dependentFinInfo.isHasIncome() != null)dependentFinancials.setHasIncome(dependentFinInfo.isHasIncome());
			//CCR13373: existing code has the indicator and setting it already
			if(dependentFinInfo.isContributedToSupport() != null)dependentFinancials.setContributedToSupport(dependentFinInfo.isContributedToSupport());
					
			if(dependentFinInfo.isIncapableOfSelfSupport() != null)dependentFinancials.setIncapableOfSelfSupport(dependentFinInfo.isIncapableOfSelfSupport());
					
			if(dependentFinInfo.isIncomeAvailableToPatient() != null)dependentFinancials.setIncomeAvailableToPatient(dependentFinInfo.isIncomeAvailableToPatient());
			
			//income
			if(dependentFinInfo.getIncomes() != null && 
					dependentFinInfo.getIncomes().getIncome() != null) {
			List<IncomeInfo> incomeList = dependentFinInfo.getIncomes().getIncome();
			if (incomeList != null) {
				Iterator incomeIter = incomeList.iterator();
				while (incomeIter.hasNext()) {
					IncomeInfo incomeinfo = (IncomeInfo) incomeIter.next();
					if (incomeinfo != null && incomeinfo.getType() != null) {
						Income income = buildHelper.buildIncomeInfo(incomeinfo);
						
						dependentFinancials.addIncome(this.typesHelper.getIncomeType(incomeinfo.getType()), income);
					}
				}
			}
		}
			//asset
			//getAssetInfo(dependentFinancials,dependentFinInfo);
		if(dependentFinInfo.getAssets() != null && 
				dependentFinInfo.getAssets().getAsset() != null){
			List<AssetInfo> assetList = dependentFinInfo.getAssets().getAsset();
			
			if (assetList != null) {
				Iterator assetIter = assetList.iterator();
				while (assetIter.hasNext()) {
					AssetInfo assetInfo = (AssetInfo)assetIter.next();
					if(assetInfo != null && assetInfo.getAssetType() != null){
						Asset asset = buildHelper.buildAssetInfo(assetInfo);
							dependentFinancials.addAsset(this.typesHelper.getAssetType(assetInfo.getAssetType()), asset);
						}
				}
			}
		}
		
		//expense
	
		if(dependentFinInfo.getExpenses() != null && 
				dependentFinInfo.getExpenses().getExpense() != null){
			List<ExpenseInfo> expenseList = dependentFinInfo.getExpenses().getExpense();
			if (expenseList != null) {
				Iterator expenseIter = expenseList.iterator();
				
				while (expenseIter.hasNext()) {
					ExpenseInfo expenseInfo = (ExpenseInfo) expenseIter.next();
					if (expenseInfo != null) {
						Expense expense = buildHelper.buildExpenseInfo(expenseInfo);
						dependentFinancials.addExpense(this.typesHelper.getExpenseType(expenseInfo.getExpenseType()), expense);
					}
				}
			}
		}
			
		
		    if(dependentFinInfo.isLivedWithPatient() != null){
				dependentFinancials.setLivedWithPatient(dependentFinInfo.isLivedWithPatient());
		    }
		    statement.addDependentFinancials(dependentFinancials);
		}
	
		}
 
	}

	@SuppressWarnings("rawtypes")
	private  void getSpouseFinInfo(FinancialStatement statement, FinancialStatementInfo fStatementInfo) {
			
		if(fStatementInfo.getSpouseFinancialsList() != null && fStatementInfo.getSpouseFinancialsList().getSpouseFinancials() !=null){
			logger.info("Spouse financial info is not null");
			
			List<SpouseFinancialsInfo> spouseList = fStatementInfo.getSpouseFinancialsList().getSpouseFinancials();
			logger.info("spouse list size is " + spouseList.size());
			Iterator itr = spouseList != null ? spouseList.iterator(): null;
		   
				
			while (itr != null && itr.hasNext()) {
				
					SpouseFinancialsInfo spouseFinInfo = (SpouseFinancialsInfo)itr.next();
					SpouseFinancials spouseFinancials = new SpouseFinancials();
				  							
					if(spouseFinInfo.isLivedWithPatient() != null){
						spouseFinancials.setLivedWithPatient(spouseFinInfo.isLivedWithPatient());
			
					}
						
					
						if(spouseFinancials.getValidDependent()!= null)
							spouseFinancials.setValidDependent(spouseFinancials.getValidDependent());
					
					
					if (spouseFinInfo.getSpouse()!= null ){
					 Spouse spouse = buildSpouse(spouseFinInfo.getSpouse());
					 spouseFinancials.setReportedOn(spouse);
					
					}
					//		income		
				 if(spouseFinInfo.getIncomes() != null && 
						 spouseFinInfo.getIncomes().getIncome() != null){	
					List<IncomeInfo> incomeList = spouseFinInfo.getIncomes().getIncome();
					if (incomeList != null) {
						Iterator incomeIter = incomeList.iterator();
						while (incomeIter.hasNext()) {
							IncomeInfo incomeinfo = (IncomeInfo) incomeIter.next();
							if (incomeinfo != null && incomeinfo.getType() != null) {
								Income income = buildHelper.buildIncomeInfo(incomeinfo);
								spouseFinancials.addIncome(this.typesHelper.getIncomeType(incomeinfo.getType()), income);
							}
						}
					}
				 }
					
					//asset
				 if(spouseFinInfo.getAssets() != null && 
						 spouseFinInfo.getAssets().getAsset() != null){
					List<AssetInfo> assetList = spouseFinInfo.getAssets().getAsset();
					
					if (assetList != null) {
						Iterator assetIter = assetList.iterator();
						while (assetIter.hasNext()) {
							AssetInfo assetInfo = (AssetInfo)assetIter.next();
							if(assetInfo != null && assetInfo.getAssetType() != null){
								Asset asset = buildHelper.buildAssetInfo(assetInfo);
								spouseFinancials.addAsset(this.typesHelper.getAssetType(assetInfo.getAssetType()), asset);
								}
						}
					}
				 }
					
					//expense
				 if(spouseFinInfo.getExpenses() != null && 
						 spouseFinInfo.getExpenses().getExpense() != null){
					List<ExpenseInfo> expenseList = spouseFinInfo.getExpenses().getExpense();
					if (expenseList != null) {
						Iterator expenseIter = expenseList.iterator();
						
						while (expenseIter.hasNext()) {
							ExpenseInfo expenseInfo = (ExpenseInfo) expenseIter.next();
							if (expenseInfo != null) {
								Expense expense = buildHelper.buildExpenseInfo(expenseInfo);
								spouseFinancials.addExpense(this.typesHelper.getExpenseType(expenseInfo.getExpenseType()), expense);
							}
						}
					}
				 }
				 		
						statement.addSpouseFinancials(spouseFinancials);
			     }
		}
	}


	public  Dependent buildDependent(DependentInfo dependentInfo)
	{
		
		Dependent dependent = new Dependent();
		dependent = populateDependentInfo(dependent, dependentInfo);
		
		return dependent;
	}


	private  Spouse buildSpouse(SpouseInfo spouseInfo) {
		
		Spouse	spouse = new Spouse();
		
		spouse = populateSpouseInfo(spouse, spouseInfo);
		
		if(spouseInfo.getAddress() != null){
		AddressInfo addressInfo= spouseInfo.getAddress();
		Address address = buildHelper.buildAddressInfo(addressInfo);
		spouse.setAddress(address);
		}
	
		return spouse;
	}

	
	/**
	 * Other than spouse and dependednts
	 * @param person
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	private  Dependent populateDependentInfo(Dependent relation, DependentInfo relationInfo) {
		
		
		Name name = new Name();
		if (relationInfo != null) {
			if(relationInfo.getFamilyName() != null)name.setFamilyName(relationInfo.getFamilyName());
			if(relationInfo.getGivenName() != null) name.setGivenName(relationInfo.getGivenName());
			if(relationInfo.getMiddleName() != null)name.setMiddleName(relationInfo.getMiddleName());
			if(relationInfo.getSuffix() != null)name.setSuffix(relationInfo.getSuffix()); //CCR13238
			relation.setName(name);
		}
		
		if(relationInfo.getDob() != null && !relationInfo.getDob().isEmpty())relation.setDob(new ImpreciseDate(relationInfo.getDob()));
		if(relationInfo.getGender() != null){
			relation.setGender(this.typesHelper.getGender(relationInfo.getGender()));
		}
		
		if(relationInfo.getStartDate()!= null && !relationInfo.getStartDate().isEmpty())relation.setStartDate(new ImpreciseDate(relationInfo.getStartDate()));
	
		if(relationInfo.getRelationship() != null){
			relation.setRelationship(this.typesHelper.getRelationship(relationInfo.getRelationship()));
		}
	
		if(relationInfo.getSsns() != null && relationInfo.getSsns().getSsn() != null){
		List<SsnInfo> ssnList= relationInfo.getSsns().getSsn();
		if (ssnList != null && ssnList.size() > 0) {
			Iterator ssnItr = ssnList != null ? ssnList.iterator() : null;
			
			while (ssnItr != null && ssnItr.hasNext()) {
				SsnInfo ssnInfo = (SsnInfo) ssnItr.next();
				SSN ssn = new SSN();
				if(ssnInfo.getSsnText() != null){
					ssn.setSsnText(ssnInfo.getSsnText());
					ssn.setType(typesHelper.getSSNType(SSNType.CODE_ACTIVE.getCode()));
				}
				relation.addSsn(ssn);
				
			}// end for ssn while
	      
			
		}// end for ssn if
		}
		
		return relation;
	}

	@SuppressWarnings("rawtypes")
	private  Spouse populateSpouseInfo(Spouse relation, SpouseInfo relationInfo) {
		
	
		Name name = new Name();
		if (relationInfo != null) {
			if(relationInfo.getFamilyName() != null)name.setFamilyName(relationInfo.getFamilyName());
			if(relationInfo.getGivenName() != null)name.setGivenName(relationInfo.getGivenName());
			if(relationInfo.getMiddleName() != null)name.setMiddleName(relationInfo.getMiddleName());
			if(relationInfo.getSuffix() != null)name.setSuffix(relationInfo.getSuffix()); //CCR13238
			
			relation.setName(name);
		}
		
		if(relationInfo.getDob() != null && !relationInfo.getDob().isEmpty())relation.setDob(new ImpreciseDate(relationInfo.getDob()));
		if(relationInfo.getGender() != null){
			relation.setGender(this.typesHelper.getGender(relationInfo.getGender()));
		}
		
		if(relationInfo.getStartDate()!= null && !relationInfo.getStartDate().isEmpty())relation.setStartDate(new ImpreciseDate(relationInfo.getStartDate()));
		if(relationInfo.getRelationship() != null){
			relation.setRelationship(this.typesHelper.getRelationship(relationInfo.getRelationship()));
		}
	
		if(relationInfo.getSsns() != null && relationInfo.getSsns().getSsn() != null){
		List<SsnInfo> ssnList= relationInfo.getSsns().getSsn();
		if (ssnList != null && ssnList.size() > 0) {
			Iterator ssnItr = ssnList != null ? ssnList.iterator() : null;
			
			while (ssnItr != null && ssnItr.hasNext()) {
				SsnInfo ssnInfo = (SsnInfo) ssnItr.next();
				SSN ssn = new SSN();
				if(ssnInfo.getSsnText() != null){
					ssn.setSsnText(ssnInfo.getSsnText());
					ssn.setType(typesHelper.getSSNType(SSNType.CODE_ACTIVE.getCode()));
				}
				relation.addSsn(ssn);
				
			}// end for ssn while
	      
			
		}// end for ssn if
		}
		
		return relation;
	}


	/**
	 ** RTC Task 236652  - When a veteran answers no to the disclose financial details option the VOA form, ES/VOA 
	 *  Backend sets some default fields to the IncomeTest so that Means Test does not go into the Pending status. 
	 *  This auxiliary method works as a DTO for VOAApplication.
	 */
	private VOAApplication getVoaApplicationDTO(SubmitFormRequest submitFormRequest) {
	
		TypesHelper typesHelper = new TypesHelper();
		BuildHelper bh = new BuildHelper();
	
		VOAApplication voaApp = new VOAApplication();
		if(submitFormRequest.getForm() != null && submitFormRequest.getForm().getSummary() != null &&
				submitFormRequest.getForm().getSummary().getPersonInfo() != null){
			PersonInfo personInfo = submitFormRequest.getForm().getSummary().getPersonInfo();
			if (personInfo != null) {
	
				if (personInfo.getFirstName() != null)
					voaApp.setFirstName(personInfo.getFirstName());
				if (personInfo.getLastName() != null)
					voaApp.setLastName(personInfo.getLastName());
				if (personInfo.getMiddleName() != null)
					voaApp.setMiddleName(personInfo.getMiddleName());
				if (personInfo.getDob() != null)
					voaApp.setDateOfBirth(bh.stringToDate(personInfo.getDob()));
				if (personInfo.getGender() != null)
					voaApp.setGender(typesHelper.getGender(personInfo.getGender()));
				if (personInfo.getSsnText() != null)
					voaApp.setSsn(personInfo.getSsnText());
			}
		}
	
		//set PREFERRED_FACILITY_ID
		if(submitFormRequest.getForm().getSummary().getDemographics() != null && submitFormRequest.getForm().getSummary().getDemographics().getPreferredFacility() != null){
			voaApp.setPreferredFacility(typesHelper.getVAFacility(submitFormRequest.getForm().getSummary().getDemographics().getPreferredFacility()));
		}
	
		if(submitFormRequest.getIdentity() != null && submitFormRequest.getIdentity().getAuthenticationLevel() != null){
			AuthenticationLevel auth = submitFormRequest.getIdentity().getAuthenticationLevel();
			if (auth != null && auth.getType() != null) {
				voaApp.setAuthenticationLevel(typesHelper.getAuthenticationLevel(auth.getType()));
				logger.info("auth level code in convertToVoaApplication(): " + voaApp.getAuthenticationLevel().getCode());
				logger.info("auth level desc  in convertToVoaApplication(): " + voaApp.getAuthenticationLevel().getDescription());
			}
		}
		// // Veteran Identifier
		if(submitFormRequest.getIdentity() != null && submitFormRequest.getIdentity().getVeteranIdentifier() != null){
			VeteranIdentifier vid = submitFormRequest.getIdentity().getVeteranIdentifier();
			if (vid != null && vid.getType() != null) {	
				voaApp.setFullQualifiedId(vid.getValue());
				voaApp.setIdentityType(typesHelper.getVeteranIdentifierType(vid.getType()));
				logger.info("FQID in convertToVoaApplication(): " + voaApp.getFullQualifiedId());
				logger.info("FQID Type in convertToVoaApplication(): " + voaApp.getIdentityType().getCode());
			}
		}
		// Form type
	
		if (submitFormRequest.getForm() != null &&
				submitFormRequest.getForm().getFormIdentifier() != null &&
				submitFormRequest.getForm().getFormIdentifier().getType() != null) {
			voaApp.setFormType(typesHelper.getVOAFormType(submitFormRequest
					.getForm().getFormIdentifier().getType()));
	
		}
		return voaApp;
	
	}

}
