package gov.va.med.esr.webservices.client.vadir;

import gov.va.schema.militaryhistory.MilitaryHistory;
import gov.va.schema.militaryhistoryservice.GetMilitaryHistory;
import gov.va.schema.militaryhistoryservice.GetMilitaryHistoryResponse;
import gov.va.schema.militaryhistoryservice.SearchCriteria;

import java.util.Date;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.springframework.stereotype.Repository;
import org.springframework.ws.client.WebServiceClientException;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.apache.log4j.Logger;

/**
 * Client for JAX-WS VADIR Web Service.
 * 
 * @author DNS   barryc
 */
@Repository(value = "vadirWSClient")
public class VadirWSClientImpl implements VadirWSClient {

	
	private static final Logger logger = Logger.getLogger(VadirWSClientImpl.class);
	
	@Autowired
	private WebServiceTemplate vadirWebServiceTemplate;
	
	
	private int throttleCount = -1;
	private int throttleStartTime = 0;
	private int throttleEndTime = 0;
	
	public int getThrottleCount() {
		return throttleCount;
	}

	public void setThrottleCount(int throttleCount) {
		this.throttleCount = throttleCount;
	}

	public int getThrottleEndTime() {
		return throttleEndTime;
	}

	public void setThrottleEndTime(int throttleEndTime) {
		this.throttleEndTime = throttleEndTime;
	}

	public int getThrottleStartTime() {
		return throttleStartTime;
	}

	public void setThrottleStartTime(int throttleStartTime) {
		this.throttleStartTime = throttleStartTime;
	}

	public MilitaryHistory getMilitaryHistory(SearchCriteria searchCriteria)
			throws WebServiceClientException {
		GetMilitaryHistory request = new GetMilitaryHistory();
		request.setCriteria(searchCriteria);

		boolean requireThrottle = VadirThrottleController.requireThrottle();
		
		if ( requireThrottle) {
			VadirThrottleController.increaseCount();
		}
		logger.debug("About to call the service ");
		
		Object resultObj = null;
		
		
		Date startTime = new Date();
		try{
			
			resultObj = vadirWebServiceTemplate
				.marshalSendAndReceive(request);
			

		}catch(WebServiceClientException e) {
			//e.printStackTrace();
			throw e;
		}
		finally {
			if ( requireThrottle) {
				VadirThrottleController.decreseCount();
			}
			Date endTime = new Date();
			logger.debug("Vadir Call Start and End Time :" + startTime + "\t" + endTime);
			//System.out.println("Vadir Call Start and End Time :" + startTime.getTime() + "\t" + endTime.getTime());
		}
		Date endTime = new Date();
		logger.debug("Vadir Call Start and End Time :" + startTime + "\t" + endTime);
		//System.out.println("Vadir Call Start and End Time :" + startTime + "\t" + endTime);
		
		if (resultObj instanceof GetMilitaryHistoryResponse == false)
			throw new RuntimeException(
					"Unexpected type returned from web service: "
							+ resultObj.getClass());

		GetMilitaryHistoryResponse response = (GetMilitaryHistoryResponse) resultObj;
		return response.getGetMilitaryHistoryReturn();
	}


	
	
	public static void main(String[] args) throws Exception {
		String cp = System.getProperty( "java.class.path" );		
		System.out.println("CP :" + cp);
		
		//ApplicationContext appContext = new ClassPathXmlApplicationContext(
		//		"applicationContext-vadir.xml");
		
		ApplicationContext appContext = new FileSystemXmlApplicationContext("D:\\bea\\projects\\ESR.3.0.0\\DNS   kavurt_HECMS_Websrv_2\\vobs\\HECMS_websrv\\target_vadir\\msds-ws\\WEB-INF\\classes\\spring\\applicationContext-vadir.xml");
		System.setProperty("vadir.throttle.starttime", "0820");
		System.setProperty("vadir.throttle.endtime", "1730");
		System.setProperty("vadir.throttle.count", "3");
		
		
		for ( int i=0; i< 300 ; i++ ) {
			VadirClientThread th = new VadirClientThread();
			th.setThreadNumber(i);
			th.setApplicationContext(appContext);
			th.start();
		}
		
		/**
		VadirWSClientImpl client = (VadirWSClientImpl) appContext
				.getBean("vadirWSClient");
		
		
		
		
		
		DatatypeFactory dtf = DatatypeFactory.newInstance();
		
		System.setProperty("vadir.throttle.starttime", "0820");
		System.setProperty("vadir.throttle.endtime", "1730");
		System.setProperty("vadir.throttle.count", "10");
		
		try {
			SearchCriteria sc = new SearchCriteria();
			sc.setSSN("810907001");
			sc.setLastName("JOHNSON");
			sc.setDateOfBirth(dtf.newXMLGregorianCalendar("1952-03-10"));
			sc.setICN("1234567890");
			MilitaryHistory mh = client.getMilitaryHistory(sc);

			if (mh == null) {
				System.out.println("No results found.");
			} else {
				System.out.println(ToStringBuilder.reflectionToString(mh,
						ToStringStyle.MULTI_LINE_STYLE));
				System.out.println("\nService Periods:");
				for (ServicePeriod period : mh.getVADIR().getServicePeriods()) {
					System.out.println(ToStringBuilder.reflectionToString(
							period, ToStringStyle.MULTI_LINE_STYLE));
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		**/
	}
	
	
	public void afterPropertiesSet() {
		logger.debug("Testing*******************");
		//System.out.println("Testing*******************");
	}
	
	
	/**
	static
    {
		VadirWSClientImpl client = (VadirWSClientImpl) appContext
		.getBean("vadirWSClient");
		
		vadirWSClient.setThrottleCount(VadirThrottleController.getThrottleCount());
		vadirWSClient.setThrottleStartTime(Integer.parseInt(VadirThrottleController.getThrottleStartTime()));
		vadirWSClient.setThrottleEndTime(Integer.parseInt(VadirThrottleController.getThrottleEndTime()));
		
    }
	**/

}
